/*
*  Program name:             dbsel  ver. 1.1
*
*  Program function:         WDB-2 data base read routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jun 20, 1989
*
*  Date of revision:         Feb 2, 1993  by Y.Murata   change Arguments
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. data base file name
*                            2. point getting interval
*                            3. lower-left  corner longitude (unit:deg.min)
*                            4. lower-left  corner latitude  (unit:deg.min)
*                            5. upper-right corner longitude (unit:deg.min)
*                            6. upper-right corner latitude  (unit:deg.min)
*
*  Input:                    none
*                            program opens the WDB-2 data base.
*
*  Output:                   integer longitude and latitude (unit:second)
*                            to standard output
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	mesh, n, lat, lon, lato, lono, lat0, lon0, i, j, nr, nb ;
	long	latmin, lonmin, latmax, lonmax, inter ;
	long	mlatmin, mlatmax ;
	long	mlat, mlon ;
	double	rlatmin, rlonmin, rlatmax, rlonmax, deg, min, atof() ;
	long	slat, slon, ip, atoi() ;
	char	pen[4] ;
	FILE	*fp ;

	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr,
	    " USAGE: dbsel db {interval lonmin latmin lonmax latmax}\n") ;
	  return(0) ;
	  }

	if (argc > 2) inter   = atoi(argv[2])     ; else inter  =     1l ;
	if (argc > 3) rlonmin = atof(argv[3])     ; else rlonmin =   0.0 ;
	if (argc > 4) rlatmin = atof(argv[4])+90.0; else rlatmin =   0.0 ;
	if (argc > 5) rlonmax = atof(argv[5])     ; else rlonmax = 360.0 ;
	if (argc > 6) rlatmax = atof(argv[6])+90.0; else rlatmax = 180.0 ;
	if (inter   <    1l) inter  =    1l ;
	if (rlatmin <   0.0) latmin =   0.0 ;
	if (rlatmax > 180.0) latmax = 180.0 ;

	deg = (int)(rlatmin) ;
	min = (rlatmin - deg) * 100.0 ;
	latmin = (int)(deg * 3600.0 + min * 60.0 - 324000.0) ;
	deg = (int)(rlonmin) ;
	min = (rlonmin - deg) * 100.0 ;
	lonmin = (int)(deg * 3600.0 + min * 60.0) ;
	deg = (int)(rlatmax) ;
	min = (rlatmax - deg) * 100.0 ;
	latmax = (int)(deg * 3600.0 + min * 60.0 - 324000.0) ;
	deg = (int)(rlonmax) ;
	min = (rlonmax - deg) * 100.0 ;
	lonmax = (int)(deg * 3600.0 + min * 60.0) ;
	if (rlonmin <   0.0) lonmin = lonmin + 360.0 ;
	if (rlonmax <   0.0) lonmax = lonmax + 360.0 ;
	if (rlonmin > 360.0) lonmin = lonmin - 360.0 ;
	if (rlonmax > 360.0) lonmax = lonmax - 360.0 ;

	if (latmin >= latmax) {
	  fprintf(stderr, " Lat parameter error.\n") ;
	  return(-1) ;
	  }

	mlatmin = (int)rlatmin / 10l ;
	mlatmax = (int)rlatmax / 10l + 1l ;

	if ((fp = fopen(argv[1], "r")) == NULL) {
	  fprintf(stderr, " Can't open db '%s'.", argv[1]) ;
	  return(-1) ;
	  }

	ip = 3 ;
	pen[2] = 'l' ;
	pen[3] = 'm' ;
	while (fscanf(fp, " %ld %ld %ld ", &mesh, &nr, &nb) == 3) {
	  mlat = mesh / 100l ;
	  mlon = mesh % 100l ;
	  if (mlat >= mlatmax) break ;
	  if (mlat < mlatmin) {
	    fseek(fp, nb, 0) ;
	    continue ;
	    }
	  fprintf(stderr, " mesh code = %ld\n", mesh) ;
	  lat0 = mlat * 36000l - 324000l ;
	  lon0 = mlon * 36000l ;
	  for (j = 0; j < nr; j++) {
	    fscanf(fp, " %ld %ld %ld ", &lat, &lon, &n) ;
	    lato = lat + lat0 ;
	    lono = lon + lon0 ;
	    if (lato<latmin || lato>latmax) ip = 3 ;
	    else if (lonmin<lonmax && (lono<lonmin || lono>lonmax)) ip = 3 ;
	    else if (lonmin>lonmax && (lono<lonmin && lono>lonmax)) ip = 3 ;
	    else {
	      printf("m %ld %ld\n", lono, lato) ;
	      ip = 2 ;
	      }
	    for (i = 1l; i < n+1l; i++) {
	      slat = getc(fp) - 150 ;
	      slon = getc(fp) - 150 ;
	      lato = lato + slat ;
	      lono = lono + slon ;
	      if (i%inter != 0 && i != n) continue ;
	      if (lato<latmin || lato>latmax) ip = 3 ;
	      else if (lonmin<lonmax && (lono<lonmin || lono>lonmax)) ip = 3 ;
	      else if (lonmin>lonmax && (lono<lonmin && lono>lonmax)) ip = 3 ;
	      else {
	        printf("%c %ld %ld\n", pen[ip], lono, lato) ;
	        ip = 2 ;
	        }
	      }
	    }
	  }
	fclose(fp) ;
	return(0) ;
	}
