/*
*  Program name:             gsi10000  ver. 1.2
*
*  Program function:         GSI data base (1:10,000) read routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Feb 23, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. item number (1101 - 8103)
*                            2. line color or point number */
/* -------------------------------------------------------------------- */
/*         区                                          図式分類コード   */
/*                         名          称              レイヤ  データ項 */
/*         分                                          コード  目コード */
/* -------------------------------------------------------------------- */
/*         行          都  道  府  県  界                １１      ０１ */
/*                     郡市界・特別区の界                １１      ０３ */
/*         政          指定都市の区界・町村界            １１      ０４ */
/*                     町（丁）・  大字界                １１      ０６ */
/*         界          丁  目  ・  字  界                １１      ０７ */
/* -------------------------------------------------------------------- */
/*         道          一  般  道  路  （地上）          ２１      ０１ */
/*                     一  般  道  路  （地下）          ２１      ０７ */
/*         路          有  料  道  路  （地上）          ２１      １１ */
/*                     有  料  道  路  （地下）          ２１      １７ */
/* -------------------------------------------------------------------- */
/*         鉄          本  線  （地上）                  ２３      ０１ */
/*                     本  線  （地下）                  ２３      ０７ */
/*         道          駅  （地上）                      ２３      １１ */
/*                     駅  （地下）                      ２３      １７ */
/* -------------------------------------------------------------------- */
/*                     神          社                    ３５      ２１ */
/*                     寺          院                    ３５      ２２ */
/*                     キ  リ  ス  ト  教  会            ３５      ２３ */
/*         建          学          校                    ３５      ２４ */
/*                     病    院  ・  医    院            ３５      ３２ */
/*                     都 道 府 県 庁 ・ 市 区 町 村                    */
/*         物                の 役 場 ・ 出 張 所        ３５      １９ */
/*                     警    察    署                    ３５      １４ */
/*                     郵    便    局                    ３５      ０９ */
/*         記          消    防    署                    ３５      １６ */
/*                     そ  の  他  の  官  署            ３５      ０３ */
/*                     幼  稚  園・保  育  園            ３５      ２５ */
/*         号          図    書    館                    ３５      ２８ */
/*                     公  会  堂  ・  公  民  館        ３５      ２６ */
/*                     銀          行                    ３５      ３４ */
/*                     デパート・スーパーマーケット      ３５      ３９ */
/*                     ホ  テ  ル  ・  旅  館            ３５      ４０ */
/*                     工          場                    ３５      ４８ */
/*                     倉          庫                    ３５      ４５ */
/*                     史跡・名勝・天然記念物            ３５      ９９ */
/*                     発  電  所  ・  変  電  所        ３５      ４９ */
/* -------------------------------------------------------------------- */
/*        水部         水    涯    線                    ５１      ００ */
/* -------------------------------------------------------------------- */
/*         基          三  角  点  （国土地理院）        ７３      ０１ */
/*                     水  準  点  （国土地理院）        ７３      ０２ */
/*         準          多  角  点  等                    ７３      ０３ */
/*                     三  角  点  （公共）              ７３      ０４ */
/*         点          水  準  点  （公共）              ７３      ０５ */
/*                     標石のない標高点                  ７３      １１ */
/* -------------------------------------------------------------------- */
/*                     市・特別区                        ８１      １０ */
/*         注          区・町・村                        ８１      １１ */
/*                     居住地名                          ８１      ０１ */
/*                     水部名                            ８１      ５１ */
/*                     道路名・通り名                    ８１      ２１ */
/*         記          鉄道名                            ８１      ２３ */
/*                     駅名                              ８１      ２４ */
/*                 その他（指示点又は副記号あり）        ８１      ０２ */
/*                 その他（指示点、副記号共になし）      ８１      ０３ */
/* -------------------------------------------------------------------- */
/*
*  Input:                    gsi 1:10,000 data, RST file (from stdin)
*
*  Output:                   JKG data (to stdout)
*
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, n, item, color[10000], atoi(), x, y, x0, y0, now ;
	char	line[86], tx[7], hed ;

	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: gsi10000 item color {item color ...}\n") ;
	  return(0) ;
	  }

	for (i = 0; i < 10000; i++) color[i] = 0 ;

	for (i = 1; i < argc-1; i=i+2) {
	  if ((item = atoi(argv[i])) <= 0 ||
	      (color[item] = atoi(argv[i+1])) <= 0) {
	    fprintf(stderr, " Item number or color error %s %s.\n",
	      argv[i], argv[i+1]) ;
	    return(-1) ;
	    }
	  }

	now = 0 ;
	while (fgets(line, 86, stdin) != NULL) {
	  if (strncmp(line, "M ", 2) == 0) {
	    fgets(line, 86, stdin) ;
	    tx[0] = line[0] ;
	    tx[1] = line[1] ;
	    tx[2] = line[2] ;
	    tx[3] = line[3] ;
	    tx[4] = line[4] ;
	    tx[5] = line[5] ;
	    tx[6] = line[6] ;
	    x0 = atoi(tx) ;
	    tx[0] = line[7] ;
	    tx[1] = line[8] ;
	    tx[2] = line[9] ;
	    tx[3] = line[10] ;
	    tx[4] = line[11] ;
	    tx[5] = line[12] ;
	    tx[6] = line[13] ;
	    y0 = atoi(tx) ;
	    }
	  else if (strncmp(line, "E2", 2) == 0) {
	    tx[0] = line[2] ;
	    tx[1] = line[3] ;
	    tx[2] = line[4] ;
	    tx[3] = line[5] ;
	    tx[4] = '\0' ;
	    item = atoi(tx) ;
	    if (color[item] == 0) continue ;
	    if (now != color[item]) {
	      printf("Lc %d\n", color[item]) ;
	      now = color[item] ;
	      }
	    tx[0] = line[27] ;
	    tx[1] = line[28] ;
	    tx[2] = line[29] ;
	    tx[3] = line[30] ;
	    tx[4] = '\0' ;
	    n = atoi(tx) ;
	    hed = 'm' ;
	    for (i = 0; i < n; i++) {
	      scanf("%d", &x) ; scanf("%d", &y) ;
	      printf("%c %d %d\n", hed, y0+y, x0+x) ;
	      if (i%6 == 5 || i == n-1) while ( getchar() != '\n' ) ;
	      hed = 'l' ;
	      }
	    }
	  else if (strncmp(line, "E5", 2) == 0 || strncmp(line, "E7", 2)) {
	    tx[0] = line[2] ;
	    tx[1] = line[3] ;
	    tx[2] = line[4] ;
	    tx[3] = line[5] ;
	    tx[4] = '\0' ;
	    item = atoi(tx) ;
	    if (color[item] == 0) continue ;
	    }
	  }
	return(0) ;
	}
