/*
*  Program name:             gsisel  ver. 1.2
*
*  Program function:         GSI data base read routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 30, 1990
*
*  Date of revision:         Feb 2, 1993 Change Arguments by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. data base name
*                            2. point select interval
*                            3. lower-left  corner longitude (unit:deg.min)
*                            4. lower-left  corner latitude  (unit:deg.min)
*                            5. upper-right corner longitude (unit:deg.min)
*                            6. upper-right corner latitude  (unit:deg.min)
*
*  Input:                    none
*                            program opens the GSI data base.
*
*  Output:                   JKG data in second (to stdout)
*/

#include <stdio.h>
#include <string.h>

#define	IN	0
#define	OUT	1

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	mesh, n, lat, lon, nr, nb ;
	double	lato, lono, latoo, lonoo, lat0, lon0, latb, lonb, deg, min, w ;
	double	latmin, lonmin, latmax, lonmax, atof() ;
	long	lat1, lon1, lat2, lon2 ;
	long	lats, lons, lats1, lons1, lats2, lons2 ;
	long	late, lone, late1, lone1, late2, lone2 ;
	long	inter ;

	int	slat, slon, ip, ipo, atoi() ;
	long	i, j ;
	FILE	*fp ;

	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr,
	    " USAGE: gsisel db {interval lonmin latmin lonmax latmax}\n") ;
	  return(0) ;
	  }
	if (argc > 2) inter  = atoi(argv[2]) ; else inter  =    1l ;
	if (argc > 3) lonmin = atof(argv[3]) ; else lonmin =   0.0 ;
	if (argc > 4) latmin = atof(argv[4]) ; else latmin =   0.0 ;
	if (argc > 5) lonmax = atof(argv[5]) ; else lonmax = 360.0 ;
	if (argc > 6) latmax = atof(argv[6]) ; else latmax =  90.0 ;
	if (latmin >= latmax || lonmin >= lonmax) {
	  fprintf(stderr, " lat, lon parameter error.\n") ;
	  return(-1) ;
	  }
							/* min */
	deg = (int)(latmin) ;
	min = (latmin - deg) * 100.0 ;
	w = (deg + min / 60.0) * 1.5 ;
	lats1 = (int)(w) ;
	lats2 = (int)((w - (double)lats1) * 8.0) ;
	lats = lats1 * 10 + lats2 ;
	latmin = deg * 3600.0 + min * 60.0 ;
	deg = (int)(lonmin) ;
	min = (lonmin - deg) * 100.0 ;
	w = (deg + min / 60.0) - 100.0 ;
	lons1 = (int)(w) ;
	lons2 = (int)((w - (double)lons1) * 8.0) ;
	lons = lons1 * 10 + lons2 ;
	lonmin = deg * 3600.0 + min * 60.0 ;
							/* max */
	deg = (int)(latmax) ;
	min = (latmax - deg) * 100.0 ;
	w = (deg + min / 60.0) * 1.5 ;
	late1 = (int)(w) ;
	w = (w - (double)late1) * 8.0 ;
	late2 = (int)(w) ;
	if (w != late2) late2 = late2 + 1 ;
	if (late2 == 8) {
	  late1 = late1 + 1 ;
	  late2 = 0 ;
	  }
	late = late1 * 10 + late2 ;
	latmax = deg * 3600.0 + min * 60.0 + 324000.0 ;
	deg = (int)(lonmax) ;
	min = (lonmax - deg) * 100.0 ;
	w = (deg + min / 60.0) - 100.0 ;
	lone1 = (int)(w) ;
	w = (w - (double)lone1) * 8.0 ;
	lone2 = (int)(w) ;
	if (w != lone2) lone2 = lone2 + 1 ;
	if (lone2 == 8) {
	  lone1 = lone1 + 1 ;
	  lone2 = 0 ;
	  }
	lone = lone1 * 10 + lone2 ;
	lonmax = deg * 3600.0 + min * 60.0 ;
	latb = 300.0 / 10000.0 ;
	lonb = 450.0 / 10000.0 ;
	if ((fp = fopen(argv[1], "r")) == NULL) {
	  fprintf(stderr, " Can't open file '%s'.\n", argv[1]) ;
	  return(-1) ;
	  }

	while (fscanf(fp, " %ld %ld %ld ", &mesh, &nr, &nb) == 3){
	  lat1 = mesh       / 10000 ;
	  lon1 = mesh%10000 / 100 ;
	  lat2 = mesh%100   / 10 ;
	  lon2 = mesh%10 ;
	  lat = lat1 * 10 + lat2 ;
	  lon = lon1 * 10 + lon2 ;
	  if (lat1 > late1) break ;
	  if (lat < lats || lat >= late || lon < lons || lon >= lone) {
	    fseek(fp, nb, 0) ;
	    continue ;
	    }
	  lat0 = (double)(lat1 * 2400 + lat2 * 300) ;
	  lon0 = (double)(lon1 * 3600 + lon2 * 450 + 360000) ;
	  for ( j = 0; j < nr; j++ ) {
	    fscanf(fp, " %ld %ld %ld ", &lat, &lon, &n) ;
	    lato = lat0 + (double)lat * latb ;
	    lono = lon0 + (double)lon * lonb ;
	    if (lato<latmin||lono<lonmin||lato>latmax||lono>lonmax) ip = OUT ;
	    else {
	      printf("m %10.3lf %10.3lf\n", lono, lato) ;
	      ip = IN ;
	      }
	    latoo = lato ;
	    lonoo = lono ;
	    ipo = ip ;
	    for ( i = 1l; i <= n; i++ ) {
	      slat = getc(fp) - 150 ;
	      slon = getc(fp) - 150 ;
	      lato = lato + (double)slat * latb ;
	      lono = lono + (double)slon * lonb ;
	      if (i%inter != 0 && i != n) continue ;
	      if (lato<latmin||lono<lonmin||lato>latmax||lono>lonmax) {
	        if (ipo == IN) printf("l %10.3lf %10.3lf\n", lono,  lato ) ;
	        ip = OUT ;
	        }
	      else {
	        if (ipo == IN) printf("l %10.3lf %10.3lf\n", lono,  lato ) ;
	        else {
	          printf("m %10.3lf %10.3lf\n", lonoo, latoo) ;
	          printf("l %10.3lf %10.3lf\n", lono,  lato ) ;
	          }
	        ip = IN ;
                }
	      latoo = lato ;
	      lonoo = lono ;
	      ipo = ip ;
	      }
	    }
	  }
	fclose(fp) ;
	return(0) ;
	}
