/*
*  Program name:             lamberta  ver. 2.0
*
*  Program function:         Lambert Azimuthal equal-area map projection
*                            forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         June 6, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. center latitude (unit:degree)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	Lat0, Lon0, Clat0, Slat0, Factr, B3600 ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	double	lat, lon, lat2, lon2, x, y ;
	char    hed, txt[500] ;
	void	Lambert() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: lambert {center-lon center-lat}\n") ;
		return(0) ;
		}
	if ( argc >= 2 ) sscanf(argv[1], "%lf", &Lon0) ; else Lon0 = 136.0 ;
	if ( argc >= 3 ) sscanf(argv[2], "%lf", &Lat0) ; else Lat0 =  36.0 ;
	if ( Lon0 <   0.0 ) Lon0 = Lon0 + 360.0 ;
	if ( Lon0 > 360.0 ) Lon0 = Lon0 - 360.0 ;
/* ---- parameter set ---- */
	Lat0  = Lat0 * 1.745329251993889e-2 ;
	Lon0  = Lon0 * 1.745329251993889e-2 ;
	Clat0 = cos(Lat0) ;
	Slat0 = sin(Lat0) ;
	Factr = 6377397.155 ;
	B3600 = 1.745329251993889e-2 / 3600.0 ;
/* ---- data read roop ---- */
	while ( fgets(txt, 500, stdin) != NULL ) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf", &hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				return(-1) ;
				}
			Lambert(lon, lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
			}
		else if (txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				return(-1) ;
				}
			Lambert(lon, lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf ", hed, x, y) ;
			Lambert(lon2, lat2, &x, &y) ;
			printf("%11.2lf %11.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Lambert(lon, lat, x, y)
double	lon, lat, *x, *y ;
{
	double	sin(), cos(), sqrt() ;
	double	clat, slat, clon, slon, rkp ;

	lat = lat * B3600 ;
	lon = lon * B3600 ;
	if(lat == Lat0 && lon == Lon0) {
		*x = 0.0 ;
		*y = 0.0 ;
		return ;
		}
	clat = cos(lat) ;
	slat = sin(lat) ;
	clon = cos(lon - Lon0) ;
	slon = sin(lon - Lon0) ;
	rkp = Factr * sqrt(2.0 / (1.0 + Slat0*slat + Clat0*clat*clon)) ;
	*x = rkp * clat * slon ;
	*y = rkp * (Clat0 * slat - Slat0 * clat * clon) ;
	}
