/*
*  Program name:             lambertR  ver. 2.0
*
*  Program function:         Lambert Azimusal equal area  map projection
*                            backward routine
*                            (x, y) ==>> (lat, lon)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         June 6, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. center latitude (unit:degree)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:second) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	Lat0, Lon0, Clat0, Slat0, Factr, B3600, Tlat0 ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	double	lat, lon, x, y, x2, y2 ;
	char    hed, txt[500] ;
	void	Lambert_Rev() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: lambert {center-lon center-lat}\n") ;
		return(0) ;
		}
	if ( argc >= 2 ) sscanf(argv[1], "%lf", &Lon0) ; else Lon0 = 136.0 ;
	if ( argc >= 3 ) sscanf(argv[2], "%lf", &Lat0) ; else Lat0 =  36.0 ;
	if ( Lon0 <   0.0 ) Lon0 = Lon0 + 360.0 ;
	if ( Lon0 > 360.0 ) Lon0 = Lon0 - 360.0 ;
/* ---- parameter set ---- */
	Lat0  = Lat0 * 1.745329251993889e-2 ;
	Lon0  = Lon0 * 1.745329251993889e-2 ;
	Clat0 = cos(Lat0) ;
	Slat0 = sin(Lat0) ;
	Tlat0 = Slat0 / Clat0 ;
	Factr = 6377397.155 ;
	B3600 = 3600.0 / 1.745329251993889e-2 ;
/* ---- data read roop ---- */
	while ( (fgets(txt, 500, stdin)) != NULL ) {
		hed = txt[0] ;
		if (hed == 'l' || hed == 'm') {
			if (sscanf(txt, "%*c %lf %lf", &x, &y) != 2) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				return(-1) ;
				}
			Lambert_Rev(&lon, &lat, x, y) ;
			printf("%c %9.2lf %9.2lf\n", hed, lon, lat) ;
			}
		else if (hed == 'A' || hed == 'W') {
			if (sscanf(txt, "%*c %lf %lf %lf %lf",
				&x, &y, &x2, &y2) != 4) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				return(-1) ;
				}
			Lambert_Rev(&lon, &lat, x, y) ;
			printf("%c %9.2lf %9.2lf ", hed, lon, lat) ;
			Lambert_Rev(&lon, &lat, x2, y2) ;
			printf("%9.2lf %9.2lf\n", lon, lat) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Lambert_Rev(lon, lat, x, y)
double	*lon, *lat, x, y ;
{
	double	atan(), sqrt() ;
	double	xx, yy, rr, rc, rho, xr, yr, a, b ;

	if (x == 0.0 && y == 0.0) {
		*lon = Lon0 * B3600 ;
		*lat = Lat0 * B3600 ;
		return ;
		}
	xx = x / Factr ;
	yy = y / Factr ;
	rr = xx * xx + yy * yy ;
	rc = sqrt(rr * (4.0 - rr)) / (2.0 - rr) ;
	rho = sqrt(rr) / rc ;
	xr = xx / rho ;
	yr = yy / rho ;
	a = 1.0 - yr * Tlat0 ;
	b = xr / Clat0 ;
	*lat = atan((Tlat0 + yr) / sqrt(a * a + b * b)) * B3600 ;
	*lon = (Lon0 + atan(b / a)) * B3600 ;
	}
