/*
*  Program name:             lambertc  ver. 1.0
*
*  Program function:         Lambert Conformal Conic Projection
*                            forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            June 27, 2007
*
*  Date of revision:         
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. center latitude-1 (unit:degree)
*                            3. center latitude-2 (unit:degree)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	Lat0, Lat1, Lat2, Lon0, Clat1, Clat2, Slat1, Slat2, Factr, B3600 ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	double	lat, lon, lat2, lon2, x, y ;
	char    hed, txt[500] ;
	void	Lambert() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: lambert {center-lon center-lat}\n") ;
		return(0) ;
		}
	if ( argc >= 2 ) sscanf(argv[1], "%lf", &Lon0) ; else Lon0 = 136.0 ;
	if ( argc >= 3 ) sscanf(argv[2], "%lf", &Lat0) ; else Lat0 =  36.0 ;
	if ( argc >= 4 ) sscanf(argv[3], "%lf", &Lat1) ; else Lat1 =  36.0 ;
	if ( argc >= 5 ) sscanf(argv[4], "%lf", &Lat2) ; else Lat2 =  37.0 ;
	if ( Lon0 <   0.0 ) Lon0 = Lon0 + 360.0 ;
	if ( Lon0 > 360.0 ) Lon0 = Lon0 - 360.0 ;
/* ---- parameter set ---- */
	Lon0  = Lon0 * 1.745329251993889e-2 ;
	Lat0  = Lat0 * 1.745329251993889e-2 ;
	Lat1  = Lat1 * 1.745329251993889e-2 ;
	Lat2  = Lat2 * 1.745329251993889e-2 ;
	Factr = 6378137 ;
	Factr = 6377397.155 ;
	B3600 = 1.745329251993889e-2 / 3600.0 ;
/* ---- data read roop ---- */
	while ( fgets(txt, 500, stdin) != NULL ) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf", &hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				return(-1) ;
				}
			Lambert(lon, lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
			}
		else if (txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				return(-1) ;
				}
			Lambert(lon, lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf ", hed, x, y) ;
			Lambert(lon2, lat2, &x, &y) ;
			printf("%11.2lf %11.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Lambert(lon, lat, x, y)
double	lon, lat, *x, *y ;
{
	double	sin(), cos(), tan(), log(), pow() ;
	double	n, f, rho, rho0 ;
	double	latl, lonl ;

	lat = lat * B3600 ;
	lon = lon * B3600 ;
/*	if(lat == Lat0 && lon == Lon0) {
		*x = 0.0 ;
		*y = 0.0 ;
		return ;
		}	*/

	n = log(cos(Lat1) / cos(Lat2)) /
		log(tan(M_PI_4 + Lat2/2.0) / tan(M_PI_4 + Lat1/2.0)) ;
	printf("n = %lf\n", n) ;

	f = cos(Lat1) * pow(tan(M_PI_4 + Lat1/2.0), n) / n ;
	printf("f = %lf\n", f) ;

	rho  = f * pow(1.0/tan(M_PI_4 + lat /2.0), n) ;
	printf("rho = %lf\n", rho) ;

	rho0 = f * pow(1.0/tan(M_PI_4 + Lat0/2.0), n) ;
	printf("rho0 = %lf\n", rho0) ;

	*x = rho * sin(n * (lon - Lon0)) * Factr ;

	*y = (rho0 - rho * cos(n * (lon - Lon0))) * Factr ;

	}
