/*
*  Program name:             legendR ver. 2.0
*
*  Program function:         legend data generate
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*                            May 20, 2004 Change Fb, Fc option by Y. Murata
*
*  Arguments:                1: legend upper left x-coordinate
*                            2: legend upper left y-coordinate
*                            3: legend width                 ( in cm )
*                            4: each cel height              ( in cm )
*                            5: distance height of each cell ( in cm )
*                            6: character size               ( in cm )
*
*  Input:                    JK-LEGEND data ( from standard input )
*
*  Output:                   JKG data ( to standard output )
*/
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	char	text1[200], text2[200] ;
	float	xstr, ystr, xwid, ywid, ydis, size, x, y, xw, yw ;
	int	i, j, icol, ipat, cpat, l1, l2 ;

	if (argc < 7) {
	  printf("Usage: legendR xstr ystr xwid ywid ydis size\n") ;
	  return(1) ;
	  }

	xstr = atof(argv[1]) ;
	ystr = atof(argv[2]) ;
	xwid = atof(argv[3]) ;
	ywid = atof(argv[4]) ;
	ydis = atof(argv[5]) ;
	size = atof(argv[6]) ;

	printf("Ss %f\n", size) ;
	size = size / 62.0 * 2.54 ;
	printf("Sv 0.5\n") ;
	printf("Sh 0.0\n") ;
	printf("m %f %f\n", xstr, ystr) ;

	x = xwid * 0.2 ;
	i = 0 ;
	while (fgets(text1, 200, stdin) != NULL) {
	  sscanf(text1, "%d %d %d", &icol, &ipat, &cpat) ;
	  i++ ;
	  y = -(float)(i-1) * ydis - ywid ;
	  fgets(text1, 200, stdin) ;
	  fgets(text2, 200, stdin) ;
	  l1 = 0 ;
	  for (j = 0; j < strlen(text1); j++) {
	    if (text1[j] != '\n' && text1[j] != ' ') l1 = j + 1 ;
	    }
	  text1[l1] = '\0' ;
	  l2 = 0 ;
	  for (j = 0; j < strlen(text2); j++) {
	    if (text2[j] != '\n' && text2[j] != ' ') l2 = j + 1 ;
	    }
	  text2[l2] = '\0' ;
	  if (l1 == 0 && l2 > 0) {
	    strcpy(text1, text2) ;
	    text2[0] = '\0' ;
	    l1 = l2 ;
	    l2 = 0 ;
	    }

	  xw = x + xwid * 1.2 ;
	  if (l1 != 0 && l2 == 0) {
	    yw = y + ywid / 2.0 ;
	    printf("rm %f %f\n", xw, yw) ;
	    printf("s %s\n", text1) ;
	    }
	  else if (l1 != 0 && l2 != 0) {
	    yw = y + (ywid - size * 2.3) / 2.0 ;
	    printf("rm %f %f\n", xw, yw) ;
	    printf("s %s\n", text2) ;
	    yw = yw + size * 1.3 ;
	    printf("rm %f %f\n", xw, yw) ;
	    printf("s %s\n", text1) ;
	    }

	  printf("Fb %d\n", icol) ;
	  printf("Fp %d\n", ipat) ;
	  if (ipat != 0) printf("Fc %d\n", cpat) ;
	  printf("n\n") ;
	  printf("rm %f %f\n", x,      y     ) ;
	  printf("rl %f %f\n", x+xwid, y     ) ;
	  printf("rl %f %f\n", x+xwid, y+ywid) ;
	  printf("rl %f %f\n", x,      y+ywid) ;
	  printf("rl %f %f\n", x,      y     ) ;
	  printf("f\n") ;
	  }
	return(0) ;
	}
