/*
*  Program name:             mby2jkg  ver. 1.0
*
*  Program function:         GSI Numerical Map (MBY) to JKG converter
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            December 12, 2003
*
*  Arguments:                1. Item-Number
*                            2. Line-Width
*                            3. Line-Color
*
* Item-Number Table
*  1: 都府県界
*  2: 北海道の支庁界
*  3: 郡市及び特別区界
*  4: 町村及び指定都市の区界
*  5: 海岸線
*  6: 湖沼の水涯線
*
*  Date of revision:
*
*  Input:                    MBY format
*
*  Output:                   JKG data
*
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	nitem, item[10], color[10] ;
	double	atof(), width[10] ;
	int	lat1, lon1, lat2, lon2 ;
	int	nlayer, nnode, nline, narea, npoint, nrecord ;
	int	layer_code ;
	int	data_code, line_number, line_code ;
	int	start_node, start_node_con, end_node, end_node_con ;
	int	area_code_left, area_numer_left ;
	int	area_code_right, area_numer_right, nattr, nattri ;
	int	nroop, nrline ;
	int	i, layer, node, line, point, area, roop, x, y, p, op ;
	char	hed, txt[74], name[21] ;
	double	lon, lat ;

	if (argc < 4 || strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: mby2jkg {item# width color} {---}\n") ;
	  return(0) ;
	  }

	nitem = (argc - 1) / 3 ;
	for (i = 0; i < nitem; i++) {
	  item[i] = atoi(argv[i*3+1]) ;
	  width[i] = atof(argv[i*3+2]) ;
	  color[i] = atoi(argv[i*3+3]) ;
	  }

	name[20] = '\0' ;
	op = 0 ;
	while ((fgets(txt, 74, stdin)) != NULL) {
	  sscanf(&txt[2], "%2d%2d%1d%1d", &lat1, &lon1, &lat2, &lon2) ;
	  strncpy(name, &txt[8], 20) ;
	  for (i = 28; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	  sscanf(&txt[28], "%3d", &nlayer) ;
	  sscanf(&txt[31], "%5d", &nnode) ;
	  sscanf(&txt[36], "%5d", &nline) ;
	  sscanf(&txt[41], "%5d", &narea) ;
	  sscanf(&txt[46], "%5d", &npoint) ;
	  sscanf(&txt[51], "%5d", &nrecord) ;
	  for (layer = 0; layer < nlayer; layer++) {
	    fgets(txt, 74, stdin) ;
	    for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	    sscanf(&txt[2], "%2d", &layer_code) ;
	    sscanf(&txt[4], "%5d", &nnode) ;
	    sscanf(&txt[9], "%5d", &nline) ;
	    sscanf(&txt[14], "%5d", &narea) ;
	    sscanf(&txt[19], "%5d", &npoint) ;
	    sscanf(&txt[24], "%5d", &nrecord) ;
	    for (node = 0; node < nnode; node++) fgets(txt, 74, stdin) ;
	    for (line = 0; line < nline; line++) {
	      fgets(txt, 74, stdin) ;
	      for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	      sscanf(&txt[4], "%2d", &data_code) ;
	      sscanf(&txt[6], "%5d", &line_number) ;
	      sscanf(&txt[11], "%6d", &line_code) ;
	      sscanf(&txt[17], "%5d", &start_node) ;
	      sscanf(&txt[22], "%1d", &start_node_con) ;
	      sscanf(&txt[23], "%5d", &end_node) ;
	      sscanf(&txt[28], "%1d", &end_node_con) ;
	      sscanf(&txt[29], "%5d", &area_code_left) ;
	      sscanf(&txt[34], "%5d", &area_numer_left) ;
	      sscanf(&txt[39], "%5d", &area_code_right) ;
	      sscanf(&txt[44], "%5d", &area_numer_right) ;
	      sscanf(&txt[49], "%6d", &npoint) ;
	      sscanf(&txt[55], "%2d", &nattr) ;
	      sscanf(&txt[57], "%3d", &nattri) ;
	      if (layer_code == 5) data_code = data_code + 4 ;
	      p = 0 ;
	      for (i = 0; i < nitem; i++) {
	        if (item[i] == data_code) {
	          p = data_code ;
	          if (p != op) {
	            if (width[i] != 0.0) printf("Lw %lf\n", width[i]) ;
	            if (color[i] != 0) printf("Lc %d\n", color[i]) ;
	            op = p ;
	            }
	          break ;
	          }
	        }
	      for (point = 0; point < npoint; point++) {
	        if (point%7 == 0) {
	          fgets(txt, 74, stdin) ;
	          for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	          }
	        sscanf(&txt[(point%7)*10], "%5d1", &x) ;
	        sscanf(&txt[(point%7)*10+5], "%5d", &y) ;
	        lon = 100.0 + (double)lon1 + (double)lon2 / 8.0 + x / 80000.0 ;
	        lat = (double)lat1 + (double)lat2 / 8.0 + y / 80000.0 ;
	        lon = lon * 3600.0 ;
	        lat = lat * 2400.0 ;
	        if (p != 0) {
	          if (point == 0) printf("m %lf %lf\n", lon, lat) ;
	          else printf("l %lf %lf\n", lon, lat) ;
	          }
	        }
	      }
	    for (area = 0; area < narea; area++) {
	      fgets(txt, 74, stdin) ;
	      for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	      sscanf(&txt[24], "%4d", &nroop) ;
	      for (roop = 0; roop < nroop; roop++) {
	        fgets(txt, 74, stdin) ;
	        for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	        sscanf(&txt[5], "%4d", &nrline) ;
	        while (nrline > 12) {
	          fgets(txt, 74, stdin) ;
	          for (i = 0; i < 74; i++) if (txt[i] == ' ') txt[i] = '0' ;
	          nrline = nrline - 12 ;
	          }
	        }
	      }
	    }
	  }
	return(0) ;
	}
