/*
*  Program name:             minmax  ver. 2.0
*
*  Program function:         min-max coordinates search and output
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            December 26, 1988
*
*  Date of revision:         December 9, 1994 Created the arguments.
*                            June 6, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f linewidth : draw the frame
*                            -w           : set window
*                            -e scale     : enlargement scale (default=1.0)
*
*  Input:                    JKG from stdin
*
*  Output:                   x-min, x-max, y-max and y-min to stdout
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	char	hed, txt[500] ;
	int	i, ip, atoi() ;
	double	dx, dy, xmin, ymin, xmax, ymax, scale, atof() ;

	xmin =  1.0 ;
	xmax = -1.0 ;
	ymin =  1.0 ;
	ymax = -1.0 ;
	scale = 1.0 ;

	ip = 0 ;
	while ((fgets(txt, 500, stdin)) != NULL) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			sscanf(txt, "%c %lf %lf", &hed, &dx, &dy) ;
			if (ip == 0) {
				xmin = dx ;
				xmax = dx ;
				ymin = dy ;
				ymax = dy ;
				ip = 1 ;
				}
			else {
				if(xmin > dx) xmin = dx ;
				if(xmax < dx) xmax = dx ;
				if(ymin > dy) ymin = dy ;
				if(ymax < dy) ymax = dy ;
				}
			}
		}

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-e") == 0) {
			scale = atof(argv[i+1]) ;
			if (scale <= 0.0) {
				fprintf(stderr, " Scale error %lf\n", scale) ;
				return(1) ;
				}
			scale = scale - 1.0 ;
			dx = (xmax - xmin) * scale / 2.0 ;
			xmin = xmin - dx ;
			xmax = xmax + dx ;
			dy = (ymax - ymin) * scale / 2.0 ;
			ymin = ymin - dy ;
			ymax = ymax + dy ;
			}
		}

	if (xmin > xmax) {
		fprintf(stderr, " JKG l/m Data record is not found.\n") ;
		return(1) ;
		}
	printf("A %lf %lf %lf %lf\n", xmin, ymin, xmax, ymax) ;

	for (i = 1; i < argc; i++) {
		if (strcmp(argv[i], "-f") == 0) {
			i = atoi(argv[i+1]) ;
			if (i == 0) break ;
			printf("Lw %d\n", i) ;
			printf("m %lf %lf\n", xmin, ymin) ;
			printf("l %lf %lf\n", xmin, ymax) ;
			printf("l %lf %lf\n", xmax, ymax) ;
			printf("l %lf %lf\n", xmax, ymin) ;
			printf("l %lf %lf\n", xmin, ymin) ;
			break ;
			}
		}

	for (i = 0; i < argc; i++) {
		if (strcmp(argv[i], "-w") == 0) {
			printf("W %lf %lf %lf %lf\n", xmin, ymin, xmax, ymax) ;
			break ;
			}
		}
	return(0) ;
	}
