/*
*  Program name:             ortho  ver. 2.0
*
*  Program function:         Orthographic map projection (sphere)
*                            forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         June 5, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. center latitude (unit:degree)
*                            3. GRS80 (default: Bessel)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	pi, r, c, b3600, lon0, lat0, sin_lat0, cos_lat0 ;
int	ip, grs ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	lat, lon, lat2, lon2, x, y ;
	char	hed, txt[500] ;
	void	ortho() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: ortho {center-lon center-lat}\n") ;
		return(0) ;
		}
	if (argc >= 2) sscanf(argv[1], "%lf", &lon0) ; else lon0 = 136.0 ;
	if (argc >= 3) sscanf(argv[2], "%lf", &lat0) ; else lat0 =  36.0 ;
	if (lon0 <   0.0) lon0 = lon0 + 360.0 ;
	if (lon0 > 360.0) lon0 = lon0 - 360.0 ;
	if (lat0 ==  90.0) lat0 =  89.99999 ;
	if (lat0 == -90.0) lat0 = -89.99999 ;
	grs = 0 ;
	if (argc >= 4 && strcmp("GRS80", argv[3])==0) grs = 1 ;
	if (argc >= 4 && strcmp("grs80", argv[3])==0) grs = 1 ;
	if (argc >= 4 && strcmp("grs", argv[3])==0) grs = 1 ;
/* ---- parameter set ---- */
	pi = 1.745329251993889e-2 ;
	lat0 = lat0 * pi ;
	lon0 = lon0 * pi ;
	if (grs == 0) r = 6377397.155 ;
	else r = 6378137.000 ;
	sin_lat0 = sin(lat0) ;
	cos_lat0 = cos(lat0) ;
	b3600 = 1.0 / 3600.0 ;
	ip = 0 ;
/* ---- data read roop ---- */
	while ( (fgets(txt, 500, stdin)) != NULL ) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf", &hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				return(-1) ;
				}
			ortho(&lon, &lat, &x, &y) ;
			if (ip == 0) {
				/* printf("# %s", txt) ; */
				continue ;
				}
			if (ip == 3) hed = 'm' ;
			printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
			}
		else if (txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				return(-1) ;
				}
			ortho(&lon, &lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf ", hed, x, y) ;
			ortho(&lon2, &lat2, &x, &y) ;
			printf("%11.2lf %11.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	ortho(lon, lat, x, y)
double	*lon, *lat, *x, *y ;
{
	*lat = *lat * b3600 * pi ;
	*lon = *lon * b3600 * pi - lon0 ;
	c = sin_lat0 * sin(*lat) + cos_lat0 * cos(*lat) * cos(*lon) ;
	if (c < 0.0) ip = 0 ;
	else {
		if (ip == 0) ip = 3 ;
		else ip = 2 ;
		*x = r*cos(*lat)*sin(*lon) ;
		*y = r*(cos_lat0*sin(*lat)-sin_lat0*cos(*lat)*cos(*lon)) ;
		}
	}
