/*
*  Program name:             polyc  ver. 2.1
*
*  Program function:         Polyconic map projection, forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         February 15, 1993 by Y. Murata (To be fast)
*                            June 6, 1995 for JKG by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*                            July 10, 2003 Add GRS80 option by Y. Murata
*                            July 31, 2003 Correct GRS80 option by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. center latitude  (unit:degree)
*                            3. GRS80 (default: Bessel)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	Pi_d, Pi_s, Lat0, Lon0, Ar, E2, A, B, C, D, E, F,
	Ds2, Ds4, Ds6, Ds8, Ds10, Old_lat=0.0 ;
double	S0, S2, S4, S6, S8, S10, Sss, Lat_c, Lat_s, Cl, Theta ;

main(argc, argv)
int     argc ;
char    *argv[] ;

{
	char    hed, txt[500] ;
	int	grs ;
	double  lat, lon, lat2, lon2, x, y, atof() ;
	void	POLY_INIT(), POLY() ;

	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: polyc {center-lon center-lat}\n") ;
		return(0) ;
		}
	grs = 0 ;
	if (argc > 1) Lon0 = atof(argv[1]) ; else Lon0 = 136.0 ;
	if (argc > 2) Lat0 = atof(argv[2]) ; else Lat0 =  36.0 ;
	if (argc > 3 && strcmp("GRS80", argv[3])==0) grs = 1 ;
	if (argc > 3 && strcmp("grs80", argv[3])==0) grs = 1 ;
	if (argc > 3 && strcmp("grs", argv[3])==0) grs = 1 ;

	POLY_INIT(grs) ;

	while ((fgets(txt, 500, stdin)) != NULL) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf",
				&hed, &lon, &lat) != 3) {
				fprintf(stderr, "JKG l/m data error.\n") ;
				break ;
				}
			POLY(&lon, &lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
			}
		else if(txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, "JKG A/W data error.\n") ;
				break ;
				}
			POLY(&lon, &lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf", hed, x, y) ;
			POLY(&lon2, &lat2, &x, &y) ;
			printf(" %11.2lf %11.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	POLY_INIT(grs)
int	grs ;
{
	double	f, e4, e6, e8, e10, e12, e14, e16 ;
	double	da, db, dc, dd, de, df ;

	Pi_d = 3.14159265358979323846 / 180.0 ;
	Pi_s = 3.14159265358979323846 / 648000.0 ;
	Lat0 = Lat0 * Pi_d ;
	Lon0 = Lon0 * Pi_d ;
	if (grs == 0) {
		Ar = 6377397.155 ;
		f = 1.0 / 299.1528128 ;
		}
	else {
		fprintf(stderr, "Set to GRS80\n") ;
		Ar = 6378137.000 ;
		f = 1.0 / 298.257222101 ;
		}
	E2 = f * (2.0 - f) ;
	e4  = E2  * E2 ;
	e6  = e4  * E2 ;
	e8  = e6  * E2 ;
	e10 = e8  * E2 ;
	e12 = e10 * E2 ;
	e14 = e12 * E2 ;
	e16 = e14 * E2 ;
	da = 1.0
	  + 3.0 / 4.0 * E2
	  + 45.0 / 64.0 * e4
	  + 175.0 / 256.0 * e6
	  + 11025.0 / 16384.0 * e8
	  + 43659.0 / 65536.0 * e10
	  + 693693.0 / 1048576.0 * e12
	  + 19324305.0 / 29360128.0 * e14
	  + 4927697775.0 / 7516192768.0 * e16 ;
	db =
	  + 3.0 / 4.0 * E2
	  + 15.0 / 16.0 * e4
	  + 525.0 / 512.0 * e6
	  + 2205.0 / 2048.0 * e8
	  + 72765.0 / 65536.0 * e10
	  + 297297.0 / 262144.0 * e12
	  + 135270135.0 / 117440512.0 * e14
	  + 547521975.0 / 469762048.0 * e16 ;
	dc =
	  + 15.0 / 64.0 * e4
	  + 105.0 / 256.0 * e6
	  + 2205.0 / 4096.0 * e8
	  + 10395.0 / 16384.0 * e10
	  + 1486485.0 / 2097152.0 * e12
	  + 45090045.0 / 58720256.0 * e14
	  + 766530765.0 / 939524096.0 * e16 ;
	dd =
	  + 35.0 / 512.0 * e6
	  + 315.0 / 2048.0 * e8
	  + 31185.0 / 131072.0 * e10
	  + 165165.0 / 524288.0 * e12
	  + 45090045.0 / 117440512.0 * e14
	  + 209053845.0 / 469762048.0 * e16 ;
	de =
	  + 315.0 / 16384.0 * e8
	  + 3465.0 / 65536.0 * e10
	  + 99099.0 / 1048576.0 * e12
	  + 4099095.0 / 29360128.0 * e14
	  + 348423075.0 / 1879048192.0 * e16 ;
	df =
	  + 693.0 / 131072.0 * e10
	  + 9009.0 / 524288.0 * e12
	  + 4099095.0 / 117440512.0 * e14
	  + 26801775.0 / 469762048.0 * e16 ;
	A  = da * (1.0 - E2) * Ar ;
	B  = db * (1.0 - E2) * Ar /  2.0 ;
	C  = dc * (1.0 - E2) * Ar /  4.0 ;
	D  = dd * (1.0 - E2) * Ar /  6.0 ;
	E  = de * (1.0 - E2) * Ar /  8.0 ;
	F  = df * (1.0 - E2) * Ar / 10.0 ;
	Ds2  =  0.951056516295055e-00 ;
	Ds4  =  0.587785252292987e-00 ;
	Ds6  = -0.587785252291703e-00 ;
	Ds8  = -0.951056516295546e-00 ;
	Ds10 = -0.158680577316903e-11 ;
	}
/* -------------------------------------------------------------------------- */
void	POLY(lon, lat, x, y)
double	*lon, *lat, *x, *y ;
{
	double	sin(), cos(), tan(), sqrt() ;

	if (*lat != Old_lat) {
		Old_lat = *lat ;
		*lat = *lat * Pi_s ;
		S0  = (*lat - Lat0)               * A ;
		S2  = (sin( 2.0 * (*lat)) - Ds2 ) * B ;
		S4  = (sin( 4.0 * (*lat)) - Ds4 ) * C ;
		S6  = (sin( 6.0 * (*lat)) - Ds6 ) * D ;
		S8  = (sin( 8.0 * (*lat)) - Ds8 ) * E ;
		S10 = (sin(10.0 * (*lat)) - Ds10) * F ;
		Sss = S0 - S2 + S4 - S6 + S8 - S10 ;
		Lat_c = cos(*lat) ;
		Lat_s = sin(*lat) ;
		Cl = Ar * Lat_c / (sqrt(1.0 - E2 * Lat_s * Lat_s) * Lat_s) ;
		}
	*lon = *lon * Pi_s ;
	Theta = (*lon - Lon0) * Lat_s ;
	*x = Cl * sin(Theta) ;
	*y = Sss + (*x) * tan(0.5 * Theta) ;
	}
