/*
*  Program name:             ppm_lib  ver. 2.0
*
*  Program function:         JKG-PPM bridge routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 15, 1990
*
*  Date of revision:         Dec 15, 1993 to use calloc by Y.Murata
*                            June 8, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -x #   x-direction window size
*                            -y #   y-direction window size
*                            -b #   ratio of the space (default 0.05)
*                            -bf $  background image file
*                            -bc #  background color
*                            -co    color overlay mode
*                            -res # resolution (default 90 dpi)
*                            -pic   PictroGraphy-3000 (Full size)
*                            -pic2  PictroGraphy-3000 (Half size)
*                            -spic  PictroGraphy-3000 (Full size, Special size)
*                            -spic2 PictroGraphy-3000 (Half size, Special size)
*/

#include	"ppm_inc.c"

unsigned char	*Frame_R, *Frame_G, *Frame_B ;

/* ------------------------------------------------- graphic init ----------- */
void	plotst_(xstr, ystr, xwid, ywid, ndev, argc, argv)
double	*xstr, *ystr, *xwid, *ywid ;
long	*ndev ;
int	argc ;
char	*argv[] ;
{
	long	i, n, bcolor=99999, bfile=0 ;
	float	space=0.05 ;
	void	Back_Read() ;

	Xdots = 0 ;
	Ydots = 0 ;
	PixelMode = 0 ;
	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: %s {-s # -r # -c $ -p $ $ ", argv[0]) ;
	    fprintf(stderr, "-x # -y # -b # -bf $ -bc # co ") ;
	    fprintf(stderr, "-res # [-pic -pic2]}\n") ;
	    exit(0) ;
	    }
	  else if (strcmp(argv[i], "-x")     == 0) Xdots = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-y")     == 0) Ydots = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-b")     == 0) space = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-co")    == 0) PixelMode = 1 ;
	  else if (strcmp(argv[i], "-bf")    == 0) bfile = ++i ;
	  else if (strcmp(argv[i], "-bc")    == 0) bcolor = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-res")   == 0) Resol = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-pic")   == 0) Pic = 1 ;
	  else if (strcmp(argv[i], "-pic2")  == 0) Pic = 2 ;
	  else if (strcmp(argv[i], "-spic")  == 0) Pic = 3 ;
	  else if (strcmp(argv[i], "-spic2") == 0) Pic = 4 ;
	  }
	Resol_Set(bcolor, space, xstr, ystr, xwid, ywid, ndev) ;
	n = Xdots * Ydots ;
	if ((Frame_R = (unsigned char *)calloc(n, sizeof(*Frame_R))) == NULL ||
	    (Frame_G = (unsigned char *)calloc(n, sizeof(*Frame_G))) == NULL ||
	    (Frame_B = (unsigned char *)calloc(n, sizeof(*Frame_B))) == NULL) {
		fprintf(stderr, " Can't alloc RGB Frame.\n") ;
		exit(-1) ;
		}
	if (bfile != 0) {
		Back_Read(argv[bfile]) ;
		RedB   = 0 ;
		GreenB = 0 ;
		BlueB  = 0 ;
		}
	else for (i = 0; i < n; i++) {
		Frame_R[i] = RedB ;
		Frame_G[i] = GreenB ;
		Frame_B[i] = BlueB ;
		}
	}
/* ------------------------------------------------- plot file read --------- */
void	Back_Read(dsn)
char	*dsn ;
{
	long	x, y, n, nx, ny ;
	char	tx[20] ;
	FILE	*fp ;

	fprintf(stderr, " Back Image read from %s.\n", dsn) ;
	if ((fp = fopen(dsn, "r")) == NULL) {
		fprintf(stderr, " Can't open background image file.\n");
		exit(-1) ;
		}

	if (Pic == 0) {
	  for (n = 0; n < 3; n++) {
	    fgets(tx, 20, fp) ;
	    if (tx[0] == '#' || tx[strlen(tx)-1] != '\n') n-- ;
	    else if (n == 1 && sscanf(tx, "%d %d", &nx, &ny) != 2) {
	      fprintf(stderr, " Can't read background image file.\n");
	      exit(-1) ;
	      }
	    }
	  if (nx < Xdots || ny < Ydots) for (n = 0; n < Xdots*Ydots; n++) {
	    Frame_R[n] = RedB   ;
	    Frame_G[n] = GreenB ;
	    Frame_B[n] = BlueB  ;
	    }
	  for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	    RedB   = getc(fp) ;
	    GreenB = getc(fp) ;
	    BlueB  = getc(fp) ;
	    if (x < Xdots && y < Ydots) {
	      n = x * Ydots + y ;
	      Frame_R[n] = RedB ;
	      Frame_G[n] = GreenB ;
	      Frame_B[n] = BlueB ; ;
	      }
	    }
	  }
	else if (Pic == 1 || Pic == 3) {
	  fscanf(fp, "%d %d %d %d", &n, &n, &ny, &nx) ;
	  while(getc(fp) != '\n') ;
	  for(x = 0; x < nx; x++) for(y = 0; y < ny; y++) {
	    n = x * Ydots + y ;
	    Frame_R[n] = getc(fp) ;
	    Frame_G[n] = getc(fp) ;
	    Frame_B[n] = getc(fp) ;
	    }
	  }
	else if (Pic == 2 || Pic == 4) {
	  fscanf(fp, "%d %d %d %d", &n, &n, &nx, &ny) ;
	  while(getc(fp) != '\n') ;
	  for (y = ny-1; y > -1; y--) for (x = 0; x < nx; x++) {
	    n = x * Ydots + y ;
	    Frame_R[n] = getc(fp) ;
	    Frame_G[n] = getc(fp) ;
	    Frame_B[n] = getc(fp) ;
	    }
	  }
	fclose(fp) ;
	}
/* ------------------------------------------------- plot file write -------- */
void	ploten_()
{
	long	x, y, n ;

	fprintf(stderr, "[Plot-End JKPPM]\n") ;

	if (Pic == 0) {
		printf("P6\n") ;
		printf("%5ld %5ld\n", Xdots, Ydots) ;
		printf("255\n") ;
		for (y = Ydots-1; y > -1; y--) for (x = 0; x < Xdots; x++) {
			n = x * Ydots + y ;
			putchar(Frame_R[n]) ;
			putchar(Frame_G[n]) ;
			putchar(Frame_B[n]) ;
			}
		}
	else if (Pic == 1 || Pic == 3) {
		printf("%d %d %d %d\n", Resol, 0, Ydots, Xdots) ;
		for(x = 0; x < Xdots; x++) for(y = 0; y < Ydots; y++) {
			n = x * Ydots + y ;
			putchar(Frame_R[n]) ;
			putchar(Frame_G[n]) ;
			putchar(Frame_B[n]) ;
			}
		}
	else if (Pic == 2 || Pic == 4) {
		printf("%d %d %d %d\n", Resol, 1, Xdots, Ydots) ;
		for (y = Ydots-1; y > -1; y--) for (x = 0; x < Xdots; x++) {
			n = x * Ydots + y ;
			putchar(Frame_R[n]) ;
			putchar(Frame_G[n]) ;
			putchar(Frame_B[n]) ;
			}
		}
	}
/* ------------------------------------------------- Frame pixel set -------- */
void	pixel(x, y)
long	x, y ;
{
	long	n ;

	if (x < Ixmin || x > Ixmax || y < Iymin || y > Iymax) return ;
	n = x * Ydots + y ;
	if (PixelMode == 0) {
	  Frame_R[n] = Red   ;
	  Frame_G[n] = Green ;
	  Frame_B[n] = Blue  ;
	  }
	else {
	  if ((int)Frame_R[n] > Red  ) Frame_R[n] = Red   ;
	  if ((int)Frame_G[n] > Green) Frame_G[n] = Green ;
	  if ((int)Frame_B[n] > Blue ) Frame_B[n] = Blue  ;
	  }
	}
