/*
*  Program name:             ps_lib
*
*  Program function:         JKG to PostScript bridge routine
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -head   (less PostScript-header option)
*                            -b4     (set the paper size to B4)
*                            -show   (less PostScript-footer option)
*                            -jp     (use Hukugou-Font)
*                            -symbol (use symbol-Font)
*                            -wtext  (use Wtext)
*                            -resol resolution (default 400dpi)
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "jk_macro.h"

#define	CV(a)	((a) * 28.3465+50)
#define	MAXP	1025

int	Resol=400, Wtext=0, Maxc= -1 ;
float	Sv=0.0, Svw, Sh=0.0, Sr=0.0, Ssize=0 ;
float	CurrentX, CurrentY ;

long	Show, Nstroke=0, CurrentColor ;
char	Patc[MAXP], Font_File[100] ;
void	stroke() ;

/* ------------------------------------------------- graphic init ----------- */
void	plotst_(xstr, ystr, xwid, ywid, ndev, argc, argv)
double	*xstr, *ystr, *xwid, *ywid ;
long	*ndev ;
int	argc ;
char	*argv[] ;
{
	int	i, j, jp, head, atoi() ;
	double	xw, yw, xd, yd ;
	void	Color_Set() ;
	char	ll[20] ;

	xw = 1.0 ;
	yw = 1.5 ;
	*xstr = xw ;
	*ystr = yw ;
	*xwid = 21.0 - xw * 2.0 ;
	*ywid = 29.7 - yw * 2.0 ;
	*ndev = 2 ;
	jp = 0 ;
	head = 1 ;
	Show = 1 ;
	Ssize = 12.0 ;
	Svw = -(float)(Ssize * 0.75) ;

	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: %s {-s # -r # -c $ -p $ $ ", argv[0]) ;
	    fprintf(stderr, "-b4 -head -show -wtext -resol #}\n") ;
	    exit(0) ;
	    }
	  else if (strcmp(argv[i], "-a4") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 21.0 - xw * 2.0 ;
	    *ywid = 29.7 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a4l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 29.7 - yw * 2.0 ;
	    *ywid = 21.0 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a3") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 29.7 - xw * 2.0 ;
	    *ywid = 42.0 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a3l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 42.0 - yw * 2.0 ;
	    *ywid = 29.7 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a2") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 42.0 - yw * 2.0 ;
	    *ywid = 59.4 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a2l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 59.4 - yw * 2.0 ;
	    *ywid = 42.0 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a1") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 59.4 - xw * 2.0 ;
	    *ywid = 84.1 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a1l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 84.1 - yw * 2.0 ;
	    *ywid = 59.4 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a0") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid =  84.1 - xw * 2.0 ;
	    *ywid = 118.9 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-a0l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 118.9 - yw * 2.0 ;
	    *ywid =  84.1 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b5") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 18.2 - xw * 2.0 ;
	    *ywid = 25.7 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b5l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 25.7 - yw * 2.0 ;
	    *ywid = 18.2 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b4") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 25.7 - xw * 2.0 ;
	    *ywid = 36.4 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b4l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 36.4 - yw * 2.0 ;
	    *ywid = 25.7 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b3") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 36.4 - xw * 2.0 ;
	    *ywid = 51.5 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b3l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 51.5 - yw * 2.0 ;
	    *ywid = 36.4 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b2") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 51.5 - xw * 2.0 ;
	    *ywid = 72.8 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b2l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 72.8 - yw * 2.0 ;
	    *ywid = 51.5 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b1") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid =  72.8 - xw * 2.0 ;
	    *ywid = 103.0 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b1l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 103.0 - yw * 2.0 ;
	    *ywid =  72.8 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b0") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    *xwid = 103.0 - xw * 2.0 ;
	    *ywid = 146.5 - yw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-b0l") == 0) {
	    *xstr = yw ;
	    *ystr = xw ;
	    *xwid = 146.5 - yw * 2.0 ;
	    *ywid = 103.0 - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-user") == 0) {
	    *xstr = xw ;
	    *ystr = yw ;
	    strcpy(ll, argv[++i]) ;
	    for (j = 0; j < 1000; j++) {
	      if (ll[j] == '\0') break ;
	      if (ll[j] == 'x') ll[j] = ' ' ;
	      if (ll[j] == '-') ll[j] = ' ' ;
	      }
	    sscanf(ll, "%lf %lf", &xd, &yd) ;
	    *xwid = xd - yw * 2.0 ;
	    *ywid = yd - xw * 2.0 ;
	    }
	  else if (strcmp(argv[i], "-jp")    == 0) jp    = 1 ;
	  else if (strcmp(argv[i], "-symbol")== 0) jp    = 2 ;
	  else if (strcmp(argv[i], "-head")  == 0) head  = 0 ;
	  else if (strcmp(argv[i], "-show")  == 0) Show  = 0 ;
	  else if (strcmp(argv[i], "-wtext") == 0) Wtext  = 1 ;
	  else if (strcmp(argv[i], "-resol") == 0) Resol = atoi(argv[++i]) ;
	  }

	if (head == 1) {
	  printf("%%!PS-Adobe-2.0 EPSF-2.0\n") ;
	  printf("%%%%Title: jkg.eps\n") ;
	  printf("%%%%Creator: Geological Survey of Japan\n") ;
	  printf("%%%%CreationDate: 7/20/94 9:00 AM\n") ;
	  printf("%%%%For: JKG Ploting System\n") ;
	  printf("%%%%BoundingBox: 0 0 %d %d\n",\
		(int)((((*xstr)*2.0+(*xwid)))/2.54*72.0+50.0),
		(int)((((*ystr)*2.0+(*ywid)))/2.54*72.0+50.0)) ;
	  printf("%%%%EndComments\n") ;
	  printf("\n") ;
	  printf("gsave\n") ;
	  printf("/M {moveto} def\n") ;
	  printf("/L {lineto} def\n") ;
	  printf("/N {newpath} def\n") ;
	  printf("/C {closepath} def\n") ;
	  printf("/G {setgray} def\n") ;
	  printf("/F {fill} def\n") ;
	  printf("/S {stroke} def\n") ;
	  printf("/W {setlinewidth} def\n") ;
	  printf("/V {gsave} def\n") ;
	  printf("/R {grestore} def\n") ;
	  printf("/Ssize 12 def\n") ;
	  printf("/Coff Ssize 0.14 mul def\n") ;
	  printf("/Sv 0 store\n") ;
	  printf("/Sh 0 store\n") ;
	  printf("/Sr 0 store\n") ;
	  printf("\n") ;
	  if (jp != 0) {
	    printf("/Hukugou-Font\n") ;
	    printf("12 dict begin\n") ;
	    printf("/FontName 1 index def\n") ;
	    printf("/FontType 0 def\n") ;
	    printf("/WMode 0 def\n") ;
	    printf("/FMapType 3 def\n") ;
	    printf("/FontMatrix matrix def\n") ;
	    printf("/Encoding [0 1] def\n") ;
	    printf("/FDepVector\n") ;
	    printf("  [/Helvetica  findfont 1.0 scalefont\n") ;
	    if (jp == 1) printf("   /GothicBBB-Medium-Ext-V  findfont 1.0 scalefont\n") ;
	    if (jp == 2) printf("   /Symbol     findfont 1.0 scalefont\n") ;
	    printf("  ] def\n") ;
	    printf("  currentdict\n") ;
	    printf("end\n") ;
	    printf("definefont pop\n") ;
	    printf("/Hukugou-Font findfont Ssize scalefont setfont\n") ;
	    }
	  else printf("/Helvetica findfont Ssize scalefont setfont\n") ;
	  printf("\n") ;
	  printf("/Wtext\n") ;
	  printf("  { /Str exch store\n") ;
	  printf("    gsave\n") ;
	  printf("      currentpoint translate\n") ;
	  printf("      Sr rotate\n") ;
	  printf("      /Shw Str stringwidth pop store\n") ;
	  printf("      0 setlinecap\n") ;
	  printf("      Ssize setlinewidth\n") ;
	  printf("      1 setgray\n") ;
	  printf("      Shw Sh mul Sv Ssize 2 div add Coff sub moveto\n") ;
	  printf("      Shw 0 rlineto\n") ;
	  printf("      stroke\n") ;
	  printf("      0 setgray\n") ;
	  printf("      Shw Sh mul Sv moveto\n") ;
	  printf("      Str show\n") ;
	  printf("    grestore\n") ;
	  printf("  } store\n") ;
	  printf("\n") ;
	  printf("/Wtext0\n") ;
	  printf("  { /Str exch store\n") ;
	  printf("    gsave\n") ;
	  printf("      currentpoint translate\n") ;
	  printf("      /Shw Str stringwidth pop store\n") ;
	  printf("      0 setlinecap\n") ;
	  printf("      Ssize setlinewidth\n") ;
	  printf("      1 setgray\n") ;
	  printf("      Shw Sh mul Sv Ssize 2 div add Coff sub moveto\n") ;
	  printf("      Shw 0 rlineto\n") ;
	  printf("      stroke\n") ;
	  printf("      0 setgray\n") ;
	  printf("      Shw Sh mul Sv moveto\n") ;
	  printf("      Str show\n") ;
	  printf("    grestore\n") ;
	  printf("  } store\n") ;
	  printf("\n") ;
	  printf("/Mtext\n") ;
	  printf("  { /Str exch store\n") ;
	  printf("    gsave\n") ;
	  printf("      currentpoint translate\n") ;
	  printf("      Sr rotate\n") ;
	  printf("      Str stringwidth pop Sh mul Sv moveto\n") ;
	  printf("      Str show\n") ;
	  printf("    grestore\n") ;
	  printf("  } store\n") ;
	  printf("\n") ;
	  printf("/Mtext0\n") ;
	  printf("  { /Str exch store\n") ;
	  printf("    gsave\n") ;
	  printf("      Str stringwidth pop Sh mul Sv rmoveto\n") ;
	  printf("      Str show\n") ;
	  printf("    grestore\n") ;
	  printf("  } store\n") ;
	  printf("\n") ;
	  printf("/P\n") ;
	  printf("  { /Gray exch store\n") ;
	  printf("    currentpoint /Ypos exch store /Xpos exch store \n") ;
	  printf("    gsave\n") ;
	  printf("    newpath\n") ;
	  printf("    Xpos Ypos moveto\n") ;
	  printf("    0 1.5 rlineto\n") ;
	  printf("    1.8 0 rlineto\n") ;
	  printf("    0 -1.5 rlineto\n") ;
	  printf("    closepath\n") ;
	  printf("    Gray setgray\n") ;
	  printf("    fill\n") ;
	  printf("    grestore\n") ;
	  printf("  } store\n") ;
	  printf("\n") ;
	  }
	else printf("V\n") ;
	printf("0 W\n") ;
	printf("1 setlinejoin\n") ;
	printf("1 setlinecap\n") ;
	printf("0 G\n") ;
	Color_Set(1) ;
	}
/* ------------------------------------------------- graphic close ---------- */
void	ploten_()
{
	fprintf(stderr, "[Plot-End PostScript]\n") ;
	stroke() ;
	if (Show == 1) printf("R\nshowpage\n") ;
	}
/* ------------------------------------------------- plot ------------------- */
void	plot_(x, y, ip)
float	*x, *y ;
long	*ip ;
{
	if (*ip == 2) {
	  if (Nstroke == 0) {
	    printf("V\n") ;
	    printf("%f %f M\n", CV(CurrentX), CV(CurrentY)) ;
	    }
	  Nstroke++ ;
	  printf("%f %f L\n", CV(*x), CV(*y)) ;
/*	  if (Nstroke%100000 == 0) {
	    printf("S R\n") ;
	    Nstroke = 0 ;
	    CurrentX = *x ;
	    CurrentY = *y ;
	    } */
	  }
	else {
	  stroke() ;
	  CurrentX = *x ;
	  CurrentY = *y ;
	  }
	}
/* ------------------------------------------------- chgpen ----------------- */
void	chgpen_(lwid, lcol, lpat, llen)
long	*lwid, *lcol, *lpat ;
float	*llen ;
{
	void	Color_Set() ;

	stroke() ;
	if (*lcol != 0) Color_Set(*lcol) ;
	if (*lwid != 0) printf("%f W\n", (float)(*lwid)/4.0) ;
	}
/* ------------------------------------------------------------- Color_Set -- */
void	coltbl_(color_file)
char	*color_file ;
{
	FILE	*color ;
	long	n ;
	int	red, green, blue ;

	if ((color = fopen(color_file, "r")) == NULL) {
	  fprintf(stderr, " Can't open color file '%s'.\n", color_file) ;
	  exit(-1) ;
	  }
	while (fscanf(color, "%ld %d %d %d", &n, &red, &green, &blue) == 4) {
	  printf("/CL%d {%f %f %f setrgbcolor} def\n",
	    n,(float)red/255.0,(float)green/255.0,(float)blue/255.0) ;
	  if (n > Maxc) Maxc = n ;
	  }
	fclose(color) ;
	}
/* ----------------------------------------------------------- Pattern_Set -- */
void	pattbl_(font_file, table_file)
char	*font_file, *table_file ;
{
	int	i ;

	for (i = 0; i < MAXP; i++) Patc[i] = ' ' ;
	strcpy(Font_File, font_file) ;

	}
/* ----------------------------------------------------------- Pattern_Set -- */
void	Pattern_Set(fpat)
long	fpat ;
{
	FILE	*font ;
	char	pt[8] ;
	char	hex[16] ;
	char	tx[36] ;
	int	npat ;
	long	ix, iy, k, n, pat[32][32] ;

	strcpy(hex, "0123456789abcdef") ;
	stroke() ;
	if (fpat < 0) fpat = -fpat ;
	if (fpat >= MAXP || Patc[fpat] != ' ') return ;
	Patc[fpat] = '*' ;

	if ((font = fopen(Font_File, "r")) == NULL) {
	  fprintf(stderr, " Can't open font file '%s'.\n", Font_File) ;
	  exit(0) ;
	  }

	while (1) {
	  if (fgets(tx, 36, font) == NULL) return ;
	  if (sscanf(tx, "%d", &npat) != 1) return ;
	  fgets(tx, 36, font) ;
	  if (npat == fpat) {
	    for (iy = 31; iy > -1; iy--) {
	      fgets(tx, 36, font) ;
	      for (ix = 0; ix < 32; ix++) {
	        if (tx[ix+2]=='r' || tx[ix+2]=='*' ) pat[ix][iy] = 0 ;
	        else                                 pat[ix][iy] = 1 ;
	        }
	      }
	    break ;
	    }
	  else for (iy = 0; iy < 32; iy++) fgets(tx, 36, font) ;
	  fgets(tx, 36, font) ;
	  }
	fclose(font) ;

	printf("<<\n") ;
	printf("/PatternType 1\n") ;
	printf("/PaintType 1\n") ;
	printf("/TilingType 1\n") ;
	printf("/BBox [0 0 7.2 7.2]\n") ;
	printf("/XStep 7.2\n") ;
	printf("/YStep 7.2\n") ;
	printf("/PaintProc\n") ;
	printf("{ begin\n") ;
	printf("7.2 7.2 scale\n") ;
	printf("32 32 1 [32 0 0 32 0 0]\n") ;
	printf("{<") ;
	for (ix = 0; ix < 32; ix++) {
	  for (iy = 0; iy < 32; iy=iy+4) {
	    k =     pat[iy  ][ix] * 8 ;
	    k = k + pat[iy+1][ix] * 4 ;
	    k = k + pat[iy+2][ix] * 2 ;
	    k = k + pat[iy+3][ix] ;
	    putchar(hex[k]) ;
	    }
	  }
	printf(">} image\n") ;
	printf("end\n") ;
	printf("} bind\n") ;
	printf(">> matrix makepattern\n") ;
	sprintf(pt, "%7d\0", fpat+10000) ;
	pt[0] = '/' ;
	pt[1] = 'P' ;
	pt[2] = 'T' ;
	printf("%s exch def\n", pt) ;
	}
/* ------------------------------------------------------- window clip ------ */
void	window_(in, x1, y1, x2, y2)
long	*in ;
float	*x1, *y1, *x2, *y2 ;
{
	stroke() ;
	if (*in == 1) printf("initclip\n") ;
	else {
	  printf("N\n") ;
	  printf("%f %f M\n", CV(*x1), CV(*y1)) ;
	  printf("%f %f L\n", CV(*x1), CV(*y2)) ;
	  printf("%f %f L\n", CV(*x2), CV(*y2)) ;
	  printf("%f %f L\n", CV(*x2), CV(*y1)) ;
	  printf("%f %f L\n", CV(*x1), CV(*y1)) ;
	  printf("C clip N\n") ;
	  }
	}
/* --------------------------------------------------- hatch area ----------- */
void	Area_Fill(xpaint, ypaint, npaint, fbkc, fpat, fptc)
long	*npaint, *fbkc, *fpat, *fptc ;
float	*xpaint, *ypaint ;
{
	char	hed[MAXD], pt[7] ;
	long	i, j, k ;
	void	Color_Set(), Pattern_Set() ;

	stroke() ;
/*	if (*fpat != 0) return ;	*/
	if (*fpat > 0) Pattern_Set(*fpat) ;
	printf("V\n") ;
	k = CurrentColor ;
	CurrentColor = 0 ;
	if (*fbkc != 0) Color_Set(*fbkc) ;
	CurrentColor = k ;
	for (i = 0; i < *npaint; i++) hed[i] = 'L' ;
	for (i = 1; i < *npaint; i++) {
	  for (j = i+1; j < *npaint; j++) {
	    if (xpaint[i]==xpaint[j-1]&&xpaint[i-1]==xpaint[j]&&
	        ypaint[i]==ypaint[j-1]&&ypaint[i-1]==ypaint[j]) {
	      hed[i] = 'M' ;
	      hed[j] = 'M' ;
	      }
	    }
	  }
	hed[0] = 'M' ;
	printf("N\n") ;
	for (i = 0; i < *npaint; i++) {
	  printf("%f %f %c\n", CV(xpaint[i]), CV(ypaint[i]), hed[i]) ;
	  }
	if (*fpat > 0) {
	  printf("C\n") ;
	  printf("/Pattern setcolorspace\n") ;
	  sprintf(pt, "%6d", abs(*fpat)+100000) ;
	  pt[0] = 'P' ;
	  pt[1] = 'T' ;
	  printf("%s setcolor F R\n", pt) ;
	  }
	else if (*fpat < 0) {
	  printf("C %f G F R\n", -(float)(*fpat)/100.0) ;
	  }
	else if (*fbkc != 0) printf("C F R\n") ;
	}
/* ------------------------------------------------------- color set -------- */
void	Color_Set(icolor)
long	icolor ;
{
	float	r, g, b ;

	if (icolor == CurrentColor) return ;
	stroke() ;
	CurrentColor = icolor ;
	if (icolor < 0) {
	  icolor = -icolor ;
	  r = (float)( icolor / 1000000) ;
	  g = (float)((icolor / 1000) % 1000) ;
	  b = (float)( icolor % 1000) ;
	  printf("%f %f %f setrgbcolor\n", r/255.0, g/255.0, b/255.0) ;
	  }
	else if (icolor <= Maxc) printf("CL%d\n", icolor) ;
	}
/* ----------------------------------------------- symbol write ------------- */
void	drwstr_(xx, yy, ssiz, sr, sh, sv, scol, nc, c)
float	*xx, *yy, *ssiz, *sr, *sh, *sv ;
char	*c ;
long	*scol, *nc ;
{
	long	ccol ;
	void	Color_Set() ;

	stroke() ;
	printf("%f %f M\n", CV(*xx), CV(*yy)) ;
	if (Ssize != *ssiz) {
	  printf("currentfont %f Ssize div scalefont setfont\n", *ssiz) ;
	  printf("/Ssize %f def\n", *ssiz) ;
	  printf("/Coff Ssize 0.14 mul def\n") ;
	  Ssize = *ssiz ;
	  Svw = - (float)(Ssize * 0.75) ;
	  }
	if (Sv != *sv) {
	  printf("/Sv %f def\n", *sv*Svw) ;
	  Sv = *sv ;
	  }
	if (Sh != *sh) {
	  printf("/Sh %f def\n", -(*sh)) ;
	  Sh = *sh ;
	  }
	if (Sr != *sr) {
	  printf("/Sr %f def\n", *sr) ;
	  Sr = *sr ;
	  }
	ccol = CurrentColor ;
	if (*scol != 0) Color_Set(*scol) ;
	if (Sr != 0.0) {
	  if (Wtext == 1)
	    printf("(%s) Wtext\n", c) ;
	  else if (Wtext == 0)
	    printf("(%s) Mtext\n", c) ;
	  }
	else {
	  if (Wtext == 1)
	    printf("(%s) Wtext0\n", c) ;
	  else if (Wtext == 0)
	    printf("(%s) Mtext0\n", c) ;
	  }
	if (*scol != 0) Color_Set(ccol) ;
	}
/* ----------------------------------------------- symbol write ------------- */
void	drwmrk_(x, y, mcod, msiz, mcol)
long	*mcod, *mcol ;
float	*x, *y, *msiz ;
{
	static int	markx[22][14] =
	{{-3,-3,-1, 1, 3, 3, 1,-1,-3, 9, 9, 9, 9, 9},
	 {-3, 3, 0,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 3, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 3, 0,-3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 0, 3, 0,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 0, 3,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3,-3, 3, 3,-3, 3, 3,-3, 9, 9, 9, 9, 9, 9},
	 {-3, 3, 0,-3, 3, 0,-3, 3, 0, 0, 0, 9, 9, 9},
	 {-3, 0, 3, 0,-3, 3, 0, 0, 0, 9, 9, 9, 9, 9},
	 {-3,-3,-1, 1, 3, 3, 1,-1,-3,-3, 3, 0, 0, 0},
	 { 0, 0, 0,-3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 3, 0,-3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-3,-3, 3, 3, 0, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-3, 0, 3, 0, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-3, 0, 3, 0, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-3, 0, 3, 0, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-1,-3,-3,-1, 1, 3, 3, 1, 0, 9, 9, 9},
	 {-3, 3,-3, 3,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 0, 0, 0, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0,-3, 3, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 3,-3, 3,-3, 0,-1, 1, 9, 9, 9, 9, 9, 9},
	 {-3,-1,-1,-3,-1, 1, 3, 1, 1, 3, 0, 1,-1, 9}} ;
	static int	marky[22][14] =
	{{ 1,-1,-3,-3,-1, 1, 3, 3, 1, 9, 9, 9, 9, 9},
	 {-3,-3, 3,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 0, 0, 3,-3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3,-3, 0,-3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0,-3, 0, 3, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3,-3, 3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3,-3,-3, 3, 3,-3, 3,-3, 9, 9, 9, 9, 9, 9},
	 { 3,-3, 0, 0, 0, 0,-3, 3, 0, 3,-3, 9, 9, 9},
	 { 0,-3, 0, 3, 0, 0, 0, 3,-3, 9, 9, 9, 9, 9},
	 { 1,-1,-3,-3,-1, 1, 3, 3, 1, 0, 0, 0, 3,-3},
	 { 3,-3, 0, 0, 0, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3,-3, 0,-3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 3, 3,-3,-3, 3, 3, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 3, 0,-3, 0, 3, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0,-3,-3, 3,-3,-3, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 3, 3,-3, 3, 3, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 0, 3, 3, 1,-1,-3,-3,-1, 1, 3, 3, 9, 9, 9},
	 { 3,-3,-3, 3, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3, 0,-3, 0, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 {-3, 3, 0, 0, 3, 9, 9, 9, 9, 9, 9, 9, 9, 9},
	 { 3, 3,-3,-3,-3, 0, 1,-1, 9, 9, 9, 9, 9, 9},
	 { 3, 1,-1,-3,-1,-1,-3,-1, 1, 3, 0, 1, 1, 9}} ;
	long	ccol, pen, j ;
	float	rx, ry, s ;
	void	Color_Set(), plot_() ;

	ccol = CurrentColor ;
	if (*mcol != 0) Color_Set(*mcol) ;
	if (*mcod < 1 || *mcod > 22) {
		fprintf(stderr, " mcod should be 1 - 22.\n") ;
		return ;
		}
	pen = 3 ;
	s = (float)((*msiz) / 72.0 * 2.54 / 3.0) ;
	for (j = 0; j < 14; j++) {
		if(markx[*mcod-1][j] == 9.0) break ;
		rx = *x + (float)markx[*mcod-1][j] * s ;
		ry = *y + (float)marky[*mcod-1][j] * s ;
		plot_(&rx, &ry, &pen) ;
		pen = 2 ;
		}
	if (*mcol != 0) Color_Set(ccol) ;
	}
/* ----------------------------------------------- symbol write ------------- */
void	stroke()
{
	if (Nstroke > 0) printf("S 0 0 M (a) Mtext R\n") ;
	Nstroke = 0 ;
	}
