/*
*  Program name:             scale  ver. 2.0
*
*  Program function:         JKG data scaling and shift
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         Jan 11, 1993 by Y. Murata
*                            June 7, 1995 for JKG by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -xsc x-scale (defaut 1.0)
*                            -xsh x-shift (defaut 0.0)
*                            -ysc y-scale (defaut 1.0)
*                            -ysh y-shift (defaut 0.0)
*                            -shf shift first (defaut none)
*                            -r   reverse x and y (defaut none)
*
*  Input:                    JKG data (from stdin)
*
*  Output:                   converted JKG data (to stdout)
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	x, y, x2, y2, xsc, xsh, ysc, ysh, atof() ;
	char	hed, txt[200] ;
	int	i, rev, shf ;

	rev = 0 ;
	shf = 0 ;
	xsc = 1.0 ;
	xsh = 0.0 ;
	ysc = 1.0 ;
	ysh = 0.0 ;

	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: scale -xsc # -xsh # -ysc # -ysh # -shf -r\n") ;
	    fprintf(stderr, " out = in * sc + sh (default)\n") ;
	    fprintf(stderr, " out = (in + sh) * sc (-shf)\n") ;
	    return(1) ;
	    }
	  else if (strcmp(argv[i], "-xsc") == 0) xsc = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-xsh") == 0) xsh = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ysc") == 0) ysc = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ysh") == 0) ysh = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-shf")   == 0) shf = 1 ;
	  else if (strcmp(argv[i], "-r")   == 0) rev = 1 ;
	  }

	while ((fgets(txt, 500, stdin)) != NULL) {
	  if (txt[0] == 'l' || txt[0] == 'm') {
	    if (sscanf(txt, "%c %lf %lf",
	      &hed, &x, &y) != 3) {
	        fprintf(stderr, " JKG l/m data error.\n") ;
	        break ;
	      }
	    if (shf == 0) {
	      x = x * xsc + xsh ;
	      y = y * ysc + ysh ;
	      }
	    else {
	      x = (x + xsh) * xsc ;
	      y = (y + ysh) * ysc ;
	      }
	    if (rev == 0) printf("%c %lf %lf\n", hed, x, y) ;
	    else printf("%c %lf %lf\n", hed, y, x) ;
	    }
	  else if(txt[0] == 'A' || txt[0] == 'W') {
	    if (sscanf(txt, "%c %lf %lf %lf %lf",
	      &hed, &x, &y, &x2, &y2) != 5) {
	      fprintf(stderr, " JKG A-W data error.\n") ;
	      break ;
	      }
	    if (shf == 0) {
	      x = x * xsc + xsh ;
	      y = y * ysc + ysh ;
	      }
	    else {
	      x = (x + xsh) * xsc ;
	      y = (y + ysh) * ysc ;
	      }
	    if (rev == 0) printf("%c %lf %lf", hed, x, y) ;
	    else printf("%c %lf %lf", hed, y, x) ;
	    if (shf == 0) {
	      x2 = x2 * xsc + xsh ;
	      y2 = y2 * ysc + ysh ;
	      }
	    else {
	      x2 = (x2 + xsh) * xsc ;
	      y2 = (y2 + ysh) * ysc ;
	      }
	    if (rev == 0) printf(" %lf %lf\n", x2, y2) ;
	    else printf(" %lf %lf\n", y2, x2) ;
	    }
	  else printf("%s", txt) ;
	  }
	return(0) ;
	}
