/*
*  Program name:             scale_br  ver. 2.01
*
*  Program function:         scale bar data generate
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         June 30, 1995 rewrite using C by Y. Murata
*                            Sep 9, 1995 set location to show character
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1: scale-bar left x-coordinate
*                            2: scale-bar left y-coordinate
*                            3: scale-bar length
*                            4: tic interval
*                            5: character size
*                            6: unit (m or km)
*
*  Input:                    none
*
*  Output:                   JKG data (to stdout)
*/

#include <stdio.h>
#include <string.h>

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	x, xstr, xend, ystr, yuni, length, interval, size, atof() ;
	long	n, unit=0 ;

	if (argc < 6) {
	  fprintf(stderr, " USAGE: scale_br xstr ystr length") ;
	  fprintf(stderr, " interval size [km m]\n") ;
	  return(0) ;
	  }

	xstr     = atof(argv[1]) ;
	ystr     = atof(argv[2]) ;
	length   = atof(argv[3]) ;
	interval = atof(argv[4]) ;
	size     = atof(argv[5]) ;
	xend = xstr + length ;
	yuni = interval / 30.0 ;
	if (argc > 6 && strcmp(argv[6], "km") == 0) unit = 1 ;

	if (size != 0.0) printf("Ss %lf\n\n", size) ;
	printf("Sr 0.0\n") ;
	printf("Sh 0.5\n") ;
	printf("Sv -0.3\n") ;
	printf("m %lf %lf\n", xstr, ystr) ;
	printf("l %lf %lf\n", xend, ystr) ;
	printf("l %lf %lf\n", xend, ystr+yuni*2.0) ;
	printf("l %lf %lf\n", xstr, ystr+yuni*2.0) ;
	printf("l %lf %lf\n", xstr, ystr) ;

	n = 0 ;
	for (x = 0.0; x <= length; x = x + interval) {
	  printf("m %lf %lf\n", xstr+x, ystr) ;
	  printf("l %lf %lf\n", xstr+x, ystr+yuni*3.0) ;
	  printf("m %lf %lf\n", xstr+x, ystr+yuni*3.0) ;
	  if (n%2 == 0) {
	    if (x < length) {
	      if (size != 0.0) {
	        if (unit == 0) printf("s %ld\n", (long)x) ;
	        else printf("s %ld\n", (long)x/1000) ;
	        }
	      printf("m %lf %lf\n", xstr+x,          ystr+yuni) ;
	      printf("l %lf %lf\n", xstr+x+interval, ystr+yuni) ;
	      }
	    else {
	      if (size != 0.0) {
	        if (unit == 0) printf("s %ld m\n", (long)x) ;
	        else printf("s %ld km\n", (long)x/1000) ;
	        }
	      }
	    }
	  n++ ;
	  }
	return(0) ;
	}
