/*
*  Program name:             transform  ver. 1.0
*
*  Program function:         JKG data transformation
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            November 14, 1997
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. tics file name (xold yold xnew ynew)
*                            2. formation order (default = 3)
*
*  Input:                    JKG data (from stdin)
*
*  Output:                   converted JKG data (to stdout)
*/

#include <stdio.h>
/*#include <math.h>*/

int	M ;
double	xa, xb, xc, xd, xe, xf ;
double	ya, yb, yc, yd, ye, yf ;
double	a[21][11] ;
 
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	int	n, i, rc, atoi() ;
	double	x1[21], y1[21], x2[21], y2[21] ;
	double	house(), s ;
	double	lon, lat, lon2, lat2, x, y ;
	char	hed, txt[500] ;
	void	PROJ() ;

	fp = fopen(argv[1], "r") ;
	for (n = 1; n <= 20; n++) {
		rc = fscanf(fp,"%lf %lf %lf %lf",&x1[n],&y1[n],&x2[n],&y2[n]) ;
		if (rc != 4) break ;
		}
	fclose(fp) ;
	if (rc == 4) {
		fprintf(stderr, "Tics > 20 points.\n") ;
		return(-1) ;
		}
	n-- ;

	if (argc > 2) M = atoi(argv[2]) ;
	else M = 3 ;
	for (i = 1; i <= n; i++) {
		a[i][1]   = 1.0 ;
		a[i][2]   = x1[i] ;
		a[i][3]   = y1[i] ;
		a[i][4]   = x1[i] * y1[i] ;
		a[i][5]   = x1[i] * x1[i] ;
		a[i][6]   = y1[i] * y1[i] ;
		a[i][M+1] = x2[i] ;
		}
	fprintf(stderr, "n, M = %d,%d\n", n, M) ;
	s = house(n, M) ;
	xa = a[1][M+1] ;
	xb = a[2][M+1] ;
	xc = a[3][M+1] ;
	xd = a[4][M+1] ;
	xe = a[5][M+1] ;
	xf = a[6][M+1] ;
	fprintf(stderr, "%f %f %f %f %f %f\n", xa, xb, xc, xd, xe, xf) ;
	fprintf(stderr, "Residual = %f\n", s) ;

	for (i = 1; i <= n; i++) {
		a[i][1]   = 1.0 ;
		a[i][2]   = y1[i] ;
		a[i][3]   = x1[i] ;
		a[i][4]   = y1[i] * x1[i] ;
		a[i][5]   = y1[i] * y1[i] ;
		a[i][6]   = x1[i] * x1[i] ;
		a[i][M+1] = y2[i] ;
		}
	s = house(n, M, a) ;
	ya = a[1][M+1] ;
	yb = a[2][M+1] ;
	yc = a[3][M+1] ;
	yd = a[4][M+1] ;
	ye = a[5][M+1] ;
	yf = a[6][M+1] ;
	fprintf(stderr, "%f %f %f %f %f %f\n", ya, yb, yc, yd, ye, yf) ;
	fprintf(stderr, "Residual = %f\n", s) ;

	while ((fgets(txt, 500, stdin)) != NULL) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf",
				&hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				break ;
				}
			PROJ(&lon, &lat, &x, &y) ;
			printf("%c %9.2lf %10.2lf\n", hed, x, y) ;
			}
		else if(txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				break ;
				}
			PROJ(&lon, &lat, &x, &y) ;
			printf("%c %9.2lf %10.2lf", hed, x, y) ;
			PROJ(&lon2, &lat2, &x, &y) ;
			printf(" %9.2lf %10.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}

void	PROJ(lon, lat, x, y)
double	*lon, *lat, *x, *y ;
{
	double	p2, p3, p4, p5, p6 ;

	p2 = (*lon) ;
	p3 = (*lat) ;
	p4 = (*lon) * (*lat) ;
	p5 = (*lon) * (*lon) ;
	p6 = (*lat) * (*lat) ;
	*x = xa ;
	if (M > 1) *x = *x + p2 * xb ;
	if (M > 2) *x = *x + p3 * xc ;
	if (M > 3) *x = *x + p4 * xd ;
	if (M > 4) *x = *x + p5 * xe ;
	if (M > 5) *x = *x + p6 * xf ;
	p2 = (*lat) ;
	p3 = (*lon) ;
	p4 = (*lat) * (*lon) ;
	p5 = (*lat) * (*lat) ;
	p6 = (*lon) * (*lon) ;
	*y = ya ;
	if (M > 1) *y = *y + p2 * yb ;
	if (M > 2) *y = *y + p3 * yc ;
	if (M > 3) *y = *y + p4 * yd ;
	if (M > 4) *y = *y + p5 * ye ;
	if (M > 5) *y = *y + p6 * yf ;
	}
 
double	house(n, m)
int	n, m ;
{
	int	i, j, k ;
	double	u, c1, d, x, v, s ;
	double	sqrt() ;

	for (k = 1; k <= m; k++) {
		s = 0.0 ;
		for (j = k; j <= n; j++) s = s + a[j][k] * a[j][k] ;
		s = sqrt(s) ;
		if (a[k][k] >= 0.0) s = -s ;
		u = s - a[k][k] ;
		for (i = k+1; i <= m+1; i++) {
			c1 = 0.0 ;
			for (j = k; j <= n; j++) c1 = c1 + a[j][k] * a[j][i] ;
			c1 = c1 / s ;
			d = (c1 - a[k][i]) / u ;
			a[k][i] = c1 ;
			for (j = k+1; j <= n; j++) {
				a[j][i] = a[j][i] - d * a[j][k] ;
				}
			}
		a[k][k] = s ;
		}
	a[m][m+1] = a[m][m+1] / a[m][m] ;
	for (k = 2; k <= m; k++) {
		j = m - k + 1 ;
		x = a[j][m+1] ;
		for (i = j+1; i <= m; i++) x = x - a[j][i] * a[i][m+1] ;
		a[j][m+1] = x / a[j][j] ;
		}
	v = 0.0 ;
	for (j = m+1; j <= n; j++) v = v + a[j][m+1] * a[j][m+1] ;
	s = v / (double)n ;
	return(s) ;
	}
