/*
*  Program name:             vander  ver. 2.0
*
*  Program function:         Vander Grinten map projection (sphere)
*                                  forward routine
*                            (lat, lon) ==>> (x, y)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 10, 1990
*
*  Date of revision:         June 5, 1995 for JKG by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. center longitude (unit:degree)
*                            2. GRS80 (default: Bessel)
*
*  Input:                    JKG (unit:second) from stdin
*                                 project only 'l' 'm' 'A' 'W' data
*
*  Output:                   projected JKG (unit:meter) to stdout
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

double	pi, pi180, piB, R, piR, b3600, lono, lon0 ;
int	ip, grs ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	double	lat, lon, lat2, lon2, x, y ;
	char	hed, txt[500] ;
	void	vander() ;
/* ---- argument read ---- */
	if (argc == 2 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: vander {center-lon}\n") ;
		return(0) ;
		}
	if (argc >= 2) sscanf(argv[1], "%lf", &lon0) ; else lon0 = 136.0 ;
	if ( lon0 <   0.0 ) lon0 = lon0 + 360.0 ;
	if ( lon0 > 360.0 ) lon0 = lon0 - 360.0 ;
	grs = 0 ;
	if (argc >= 3 && strcmp("GRS80", argv[3])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs80", argv[3])==0) grs = 1 ;
	if (argc >= 3 && strcmp("grs", argv[3])==0) grs = 1 ;
/* ---- parameter set ---- */
	pi = 3.14159265359 ;
	pi180 = pi / 180.0 ;
	piB = 1.0 / pi ;
	if (grs == 0) R = 6377397.155 ;
	else R = 6378137.000 ;
	piR = pi * R ;
	b3600 = 1.0 / 3600.0 ;
	lono = -999999.0 ;
/* ---- data read roop ---- */
	while ((fgets(txt, 500, stdin)) != NULL) {
		if (txt[0] == 'l' || txt[0] == 'm') {
			if (sscanf(txt, "%c %lf %lf", &hed, &lon, &lat) != 3) {
				fprintf(stderr, " JKG l/m data error.\n") ;
				return(-1) ;
				}
			vander(&lon, &lat, &x, &y) ;
			if (ip == 3) hed = 'm' ;
			printf("%c %11.2lf %11.2lf\n", hed, x, y) ;
			}
		else if (txt[0] == 'A' || txt[0] == 'W') {
			if (sscanf(txt, "%c %lf %lf %lf %lf",
				&hed, &lon, &lat, &lon2, &lat2) != 5) {
				fprintf(stderr, " JKG A/W data error.\n") ;
				return(-1) ;
				}
			vander(&lon, &lat, &x, &y) ;
			printf("%c %11.2lf %11.2lf ", hed, x, y) ;
			vander(&lon2, &lat2, &x, &y) ;
			printf("%11.2lf %11.2lf\n", x, y) ;
			}
		else printf("%s", txt) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	vander(lon, lat, x, y)
double	*lon, *lat, *x, *y ;
{
	double	theta, A, G, P, Q, GP2, G2P2, P2A2, xpiR, A2, G2, P2 ;
	double	fabs(), sqrt(), sin(), cos(), tan(), asin() ;

	*lat = *lat * b3600 ;
	*lon = *lon * b3600 - lon0 ;
	if(*lon < -180.0) *lon = *lon + 360.0 ;
	else if(*lon > 180.0) *lon = *lon - 360.0 ;
	if (fabs(*lon-lono) > 300.0) ip = 3 ;
	else ip = 2 ;
	lono = *lon ;
	*lat = *lat * pi180 ;
	*lon = *lon * pi180 ;

	if (*lat == 0.0) {
		*x = *lon * R ;
		*y = 0.0 ;
		}
	else if (*lon == 0.0) {
		theta = asin(2.0 * fabs(*lat) * piB) ;
		*x = 0.0 ;
		*y = piR * tan(0.5 * theta) ;
		if (*lat < 0.0) *y = -(*y) ;
		}
	else if (fabs(*lat) > 1.57079) {
		*x = 0.0 ;
		*y = piR ;
		if (*lat < 0.0) *y = -(*y) ;
		}
	else {
		theta = asin(2.0 * fabs(*lat) * piB) ;
		A = 0.5 * fabs(pi / (*lon) - (*lon) * piB) ;
		G = cos(theta) / (sin(theta) + cos(theta) - 1.0) ;
		P = G * (2.0 / sin(theta) - 1.0) ;
		A2 = A * A ;
		P2 = P * P ;
		G2 = G * G ;
		GP2 = G - P2 ;
		G2P2 = G2 - P2 ;
		P2A2 = P2 + A2 ;
		Q = sqrt(A2 * GP2 * GP2 - P2A2 * G2P2) ;
		*x = piR * (A * GP2 + Q) / P2A2 ;
		xpiR = *x / piR ;
		*y = piR * sqrt(1.0 - xpiR * xpiR - 2.0 * A * xpiR) ;
		if (*lon < 0.0) *x = -(*x) ;
		if (*lat < 0.0) *y = -(*y) ;
		}
	}
