/*
*  Program name:             arcg_g  ver. 1.0
*
*  Program function:         ArcInfo grid ascii file to JKM
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jun 28, 1998
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1: y scale (default = 0)
*                            2: y shift (default = 0)
*
*  Input:                    ArcInfo grid ascii file (from stdin)
*
*  Output:                   JKM file (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "jkm_lib.c"

main (argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy, n ;
	double	yscale, yshift ;
	void	JKM_Int_Write() ;

	if (argc > 1) yscale = atof(argv[1]) ;
	else yscale = 1.0 ;
	if (argc > 2) yshift = atof(argv[2]) ;
	else yshift = 0.0 ;

	scanf("%*s %ld\n", &Nx) ;
	scanf("%*s %ld\n", &Ny) ;
	scanf("%*s %lf\n", &Xstr) ;
	scanf("%*s %lf\n", &Ystr) ;
	scanf("%*s %lf\n", &Dx) ;
	scanf("%*s %ld\n", &Iund) ;

	Dy = Dx * yscale ;
	Ystr = Ystr * yscale + yshift ;
	
	strcpy(Cmnt, "Grid data generated by arcg_g") ;
	if ((Ig = (long *)calloc(Nx*Ny, sizeof(*Ig))) == NULL) {
		fprintf(stderr, " Can't alloc memory Ig.\n") ;
		return(-1) ;
		}
	for (n = 0; n < Nx*Ny; n++) Ig[n] = Iund ;

	for (iy = Ny-1; iy > -1; iy--) {
		for (ix = 0; ix < Nx; ix++) {
			n = ix * Ny + iy ;
			scanf("%ld", &Ig[n]) ;
			}
		}

	JKM_Int_Write() ;

	return(0) ;
	}
