/*
*  Program name:             birdP
*
*  Program function:         color-pattern bird-eye view plot
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            August 8, 1989
*
*  Date of revision:         February 11,1993 by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f  $   (JKM file name)
*                            -cf $   (color JKM file name)
*                            -sf $   (segment file name)
*                            -rz #   (z rad (default: 10.0 deg))
*                            -rx #   (x rad (default: 20.0 deg))
*                            -in #   (interval (default: 1))
*                            -sz #   (Hscale (default: 1.0))
*                            -sc #   (Scale (default: calcurated))
*                            -xs #   (Xshift (default: 0.0))
*                            -ys #   (Xshift (default: 0.0))
*                            -pm #   (mesh pen number)
*                            -ex #   (Eye_x position)
*                            -ey #   (Eye_y position)
*                            -py #   (projection screen position)
*                            -frame  (frame plot)
*
*  Input:                    JKM file (program open)
*
*  Output:                   JKG for color-bird-eye view
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "bird_inc.c"

#define	MAXSC	200

FILE	*Seg ;
long	*S ;
int	Stat_c=0, Stat_s=0, Inter, Lx, Ly ;
unsigned char	*C ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, stat_frame ;
	char	dsn_h[80], dsn_c[80], dsn_s[80] ;
	void	PPM_Read(), JKM_Read(), SEG_Read(), SEG_Color_Read(), cont4d() ;

	Inter = 1 ;
	stat_frame = 0 ;
	dsn_h[0] = '\0' ;
	dsn_c[0] = '\0' ;
	dsn_s[0] = '\0' ;
	parm_init(argc, argv) ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-f")  == 0) strcpy(dsn_h, argv[++i]) ;
	  else if (strcmp(argv[i], "-cf") == 0) strcpy(dsn_c, argv[++i]) ;
	  else if (strcmp(argv[i], "-sf") == 0) strcpy(dsn_s, argv[++i]) ;
	  else if (strcmp(argv[i], "-in") == 0) Inter = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-frame") == 0) stat_frame = 1 ;
	  else if (strcmp(argv[i], "-h")  == 0) {
	    fprintf(stderr, " USAGE: birdC -f $ -cf $ -rx # -rz # ") ;
	    fprintf(stderr, "-in # -sx # -sy # -sz # -sc # -xs # -xs # ") ;
	    fprintf(stderr, "-pm # -ex # -ey # -py # -frame\n") ;
	    return(0) ;
	    }
	  }
/* ----------------------------------------------------------- grid read ---- */
	if (dsn_c[0] != '\0') {
	  PPM_Read(dsn_c) ;
	  Stat_c = 1 ;
	  }

	JKM_Read(dsn_h) ;

	if (dsn_s[0] != '\0') {
	  SEG_Read(dsn_s) ;
	  Stat_s = 1 ;
	  }
/* ----------------------------------------------------------- bird init ---- */
	bird_init(stat_frame) ;

	if (Pen_m != 0) {
	  Cpen[0] = 'l' ;
	  printf("Lw %d\n", Pen_m) ;
	  }
	else Cpen[0] = 'm' ;

	cont4d(Nx-2, Nx_c,  -1, Ny-2, Ny_c,  -1) ;
	cont4d(0,    Nx_c-1, 1, Ny-2, Ny_c,  -1) ;
	cont4d(0,    Nx_c-1, 1,    0, Ny_c-1, 1) ;
	cont4d(Nx-2, Nx_c,  -1,    0, Ny_c-1, 1) ;

	free(H) ;
	if (Stat_c == 1) free(C) ;
	if (Stat_s == 1) free(S) ;

	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	PPM_Read(dsn)
char	dsn[] ;
{
	FILE	*fp ;
	int	ix, iy, n ;
	char	tx[80] ;

	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open PPM file.\n") ;
	  exit(-1) ;
	  }
	for (n = 0; n < 3; n++) {
	  fgets(tx, 80, fp) ;
	  if (tx[0] == '#') n-- ;
	  else if (n == 0) {
	    if (strncmp(tx, "P6", 2) != 0) {
	      fprintf(stderr, " Color file should be PPM (P6) file.\n") ;
	      fprintf(stderr, "%s\n", tx) ;
	      exit(-1) ;
	      }
	    }
	  else if (n == 1 && sscanf(tx, "%d %d", &Nx, &Ny) != 2) {
	    fprintf(stderr, " Can't read PPM header.\n") ;
	    exit(-1) ;
	    }
	  }
	Lx = ((Nx + Inter - 2) / Inter + 1) ;
	Ly = ((Ny + Inter - 2) / Inter + 1) ;
	if ((C = (unsigned char *)calloc(Lx*Ly*3, sizeof(*C))) == NULL) {
	  fprintf(stderr, " Can't alloc PPM color.\n") ;
	  exit(-1) ;
	  }
	for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
	  n = ((ix + Inter - 1) / Inter) * Ly ;
	  n = (n + (iy + Inter - 1) / Inter) * 3 ;
	  C[n  ] = fgetc(fp) ;
	  C[n+1] = fgetc(fp) ;
	  C[n+2] = fgetc(fp) ;
	  }
	fclose(fp) ;
	fprintf(stderr, " PPM file read end.\n") ;
	}
/* -------------------------------------------------------------------------- */
void	SEG_Read(dsn)
char	dsn[] ;
{
	int	ix, iy ;
	long	n, p, ftell() ;
	char	tx[200] ;

	if ((Seg = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open SEG file.\n") ;
	  exit(-1) ;
	  }
	if ((S = (long *)calloc(Nx*Ny, sizeof(*S))) == NULL) {
	  fprintf(stderr, " Can't alloc SEG pointer.\n") ;
	  exit(-1) ;
	  }
	for (n = 0; n < Nx*Ny; n++) S[n] = -1 ;
	while (1) {
	  p = ftell(Seg) ;
	  if (fgets(tx, 200, Seg) == NULL) break ;
	  sscanf(tx, "%d %d", &ix, &iy) ;
	  ix = (ix + Inter - 1) / Inter ;
	  iy = (iy + Inter - 1) / Inter ;
	  if (S[ix*Ny+iy] == -1) S[ix*Ny+iy] = p ;
	  }
	fprintf(stderr, " SEG file read end.\n") ;
	}
/* -------------------------------------------------------------------------- */
void	SEG_Write(ix, iy)
int	ix, iy ;
{
	int	i, jx, jy, kx, ky, num ;
	long	n, color, width ;
	float	h1, h2, h3, h4, x0, y0, xw, yw, hw, hw1, hw2, x, y ;
	char	hed ;
	void	conv() ;

	n = ix * Ny + iy ;
	if (S[n] == -1) return ;
	if (fseek(Seg, S[n], 0) != 0) {
	  fprintf(stderr, " Can't seek Seg file.\n") ;
	  exit(-1) ;
	  }
	h1 = H[n     ] ;
	h2 = H[n   +1] ;
	h3 = H[n+Ny+1] ;
	h4 = H[n+Ny  ] ;
	while (fscanf(Seg, "%d %d", &jx, &jy) == 2) {
	  if (Inter == 1) {
	    if (ix != jx || iy != jy) break ;
	    x0 = 0.0 ;
	    y0 = 0.0 ;
	    }
	  else {
	    jx = jx + Inter - 1 ;
	    jy = jy + Inter - 1 ;
	    kx = jx / Inter ;
	    ky = jy / Inter ;
	    if (ix != kx || iy != ky) break ;
	    x0 = (float)(jx % Inter) / (float)Inter ;
	    y0 = (float)(jy % Inter) / (float)Inter ;
	    }
	  fscanf(Seg, "%ld %ld %d", &color, &width, &num) ;
	  if (color != 0) printf("Lc %ld\n", color) ;
	  if (width != 0) printf("Lw %ld\n", width) ;
	  hed = 'm' ;
	  for (i = 0; i < num; i++) {
	    fscanf(Seg, "%f %f", &xw, &yw) ;
	    xw = xw / (float)Inter + x0 ;
	    yw = yw / (float)Inter + y0 ;
	    hw1 = (h2 - h1) * yw + h1 ;
	    hw2 = (h3 - h4) * yw + h4 ;
	    hw = (hw2 - hw1) * xw + hw1 ;
	    xw = (float)ix * Dx + xw * Dx ;
	    yw = (float)iy * Dy + yw * Dy ;
	    conv(xw, yw, hw, &x, &y) ;
	    printf("%c %f %f %f %f %f\n", hed, x, y, xw, yw, hw) ;
	    hed = 'l' ;
	    }
	  }
	if (color != 0) printf("Lc 1\n") ;
	if (width != 0) printf("Lw 1\n") ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Read(dsn)
char	dsn[] ;
{
	FILE	*fp ;
	int	ix, iy, jx, jy, mx, my, n1, n2 ;
	float	w ;

	if (strcmp(dsn, "-") == 0 || dsn[0] == '\0') fp = stdin ;
	else if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open float grid.\n") ;
	  exit(-1) ;
	  }
	while ((getc(fp)) != '\n') ;
	fscanf(fp, "%d %d %f %f %f %f %f %f %f",
	  &Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Hmin, &Hmax) ;
	mx = ((Nx + Inter - 2) / Inter + 1) ;
	my = ((Ny + Inter - 2) / Inter + 1) ;
	if ((H = (float *)calloc(mx*my, sizeof(*H))) == NULL) {
	  fprintf(stderr, " Can't alloc float grid.\n") ;
	  exit(-1) ;
	  }
	Hmin =  99999999.9 ;
	Hmax = -99999999.9 ;
	for (ix = 0; ix < Nx; ix++) {
	  jx = ((ix + Inter - 1) / Inter) * my ;
	  for (iy = 0; iy < Ny; iy++) {
	    fscanf(fp, "%f", &w) ;
	    jy = (iy + Inter - 1) / Inter ;
	    if (w != Und) {
	      Hmin = min(Hmin, w) ;
	      Hmax = max(Hmax, w) ;
	      }
	    H[jx+jy] = w ;
	    }
	  }
	if (fp != stdin) fclose(fp) ;
	fprintf(stderr, " Float grid read end.\n") ;
	Dx = Dx * Inter ;
	Dy = Dy * Inter ;
	Nx = mx ;
	Ny = my ;
	if (Stat_c == 1 && (Nx != Lx || Ny != Ly)) {
	  for (ix = 0; ix < Nx; ix++) {
	    for (iy = 0; iy < Ny; iy++) {
	      n1 = ix * Ny + iy ;
	      n2 = ix * Ly + iy ;
	      C[n1] = C[n2] ;
	      }
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	cont4d(ixs, ixe, ixd, iys, iye, iyd)
int	ixs, ixe, ixd, iys, iye, iyd ;
{
	int	ix, iy, n, nc ;
	float	*xd1, *yd1, *xd2, *yd2 ;
	double	fabs() ;
	void	SEG_Write(), conv_line() ;

	fprintf(stderr, " x = %5d to %5d by %5d / y = %5d to %5d by %5d\n",
		ixs, ixe, ixd, iys, iye, iyd) ;
	if ((ixe-ixs)*ixd < 0) return ;
	if ((iye-iys)*iyd < 0) return ;
	if ((xd1 = (float *)calloc(Ny, sizeof(*xd1))) == NULL ||
	    (yd1 = (float *)calloc(Ny, sizeof(*yd1))) == NULL ||
	    (xd2 = (float *)calloc(Ny, sizeof(*xd2))) == NULL ||
	    (yd2 = (float *)calloc(Ny, sizeof(*yd2))) == NULL) {
	  fprintf(stderr, " Can't alloc xd, yd.\n") ;
	  exit(-1) ;
	  }
	if (ixs < 0)    ixs = 0 ;
	if (iys < 0)    iys = 0 ;
	if (ixs > Nx-2) ixs = Nx-2 ;
	if (iys > Ny-2) iys = Ny-2 ;
	if (ixe < 0)    ixe = 0 ;
	if (iye < 0)    iye = 0 ;
	if (ixe > Nx-2) ixe = Nx-2 ;
	if (iye > Ny-2) iye = Ny-2 ;
	if (ixd == 1) conv_line(ixs, iys,iye,iyd, xd1,yd1, xd2,yd2) ;
	else conv_line(ixs+1, iys,iye,iyd, xd1,yd1, xd2,yd2) ;
	for (ix = ixs; ix != ixe + ixd; ix = ix + ixd) {
	  fprintf(stderr, " Line = %4d (%5.1lf%% to %4d)\n", ix,
	    fabs((double)(ix-ixs)/(double)(ixe-ixs)*100.0), ixe) ;
	  if (ixd == 1) conv_line(ix+1, iys,iye,iyd, xd1,yd1, xd2,yd2) ;
	  else conv_line(ix, iys,iye,iyd, xd1,yd1, xd2,yd2) ;
	  for (iy = iys; iy != iye + iyd; iy = iy + iyd) {
	    if (xd1[iy] > 9e28 || xd1[iy+1] > 9e28 ||
	         xd2[iy] > 9e28 || xd2[iy+1] > 9e28) continue ;
	    n = (ix * Ny + iy) * 3 ;
	    nc = 0 ;
	    if (Stat_c) {
	      nc = (long)C[n]*1000000+(long)C[n+1]*1000+(long)C[n+2] ;
	      if (nc == 0) nc = -001001001 ;
	      }
	    if (nc != 0) printf("Fb %d\nn\n", -nc) ;
	    printf("%c %f %f\n", 'm',     xd1[iy  ], yd1[iy  ]) ;
	    printf("%c %f %f\n", Cpen[0], xd1[iy+1], yd1[iy+1]) ;
	    printf("%c %f %f\n", Cpen[0], xd2[iy+1], yd2[iy+1]) ;
	    printf("%c %f %f\n", Cpen[0], xd2[iy  ], yd2[iy  ]) ;
	    if (Cpen[0] == 'l')
	    printf("%c %f %f\n", Cpen[0], xd1[iy  ], yd1[iy  ]) ;
	    if (nc != 0) printf("f\n") ;
	    if (Stat_s != 0) SEG_Write(ix, iy) ;
	    }
	  }
	free(xd1) ;
	free(yd1) ;
	free(xd2) ;
	free(yd2) ;
	}
/* -------------------------------------------------------------------------- */
void	conv_line(ix, iys, iye, iyd, xd1, yd1, xd2, yd2)
int	ix, iys, iye, iyd ;
float	*xd1, *yd1, *xd2, *yd2 ;
{
	int	iy ;
	float	xw, yw, hw ;
	void	conv() ;

	xw = (float)ix * Dx ;
	for (iy = min(iys,iye); iy <= max(iys,iye)+1; iy++) {
	  xd1[iy] = xd2[iy] ;
	  yd1[iy] = yd2[iy] ;
	  hw = H[ix*Ny+iy] ;
	  if (hw != Und) {
	    yw = (float)iy * Dy ;
	    conv(xw, yw, hw, &xd2[iy], &yd2[iy]) ;
	    }
	  else xd2[iy] = 9e29 ;
	  }
	}
