/*
*  Program name:             subroutines for bird-eyc programs
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            April 5, 1993
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*/

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))

int	Nx, Ny ;
float	Xstr, Ystr, Dx, Dy ;
float	Und, Hmin, Hmax, Hscale, *H ;
int	Nx_c, Ny_c ;
int	Pen_m, Pen_c, Cmax, Head ;
float	Z_rad, X_rad, C_Z_rad, S_Z_rad, C_X_rad, S_X_rad ;
float	X2min, Y2min, X2max, Y2max, Xshift, Yshift ;
float	X3str, Y3str, Z3str ;
float	X_size, Y_size, Scale ;
float	Eye_x, Eye_y, Pro_y ;
float	X_size, Y_size, Undc ;
char	Cpen[3] ;
/* -------------------------------------------------------------------------- */
void	parm_init(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i ;

	Z_rad = 10.0 ;
	X_rad = 20.0 ;
	X_size = 18.0 ;
	Y_size = 26.0 ;
	Hscale = -1.0 ;
	Pen_m = 1 ;
	Eye_x = 0.0 ;
	Eye_y = 0.0 ;
	Pro_y = 1.0 ;
	Xshift = 0.0 ;
	Yshift = 0.0 ;
	Scale = 0.0 ;
	Head = 1 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-rx") == 0) X_rad  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-rz") == 0) Z_rad  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-sx") == 0) X_size = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-sy") == 0) Y_size = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-sz") == 0) Hscale = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-sc") == 0) Scale  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-xs") == 0) Xshift = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-ys") == 0) Yshift = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-pm") == 0) Pen_m  = atoi(argv[++i]) ;
	      else if (strcmp(argv[i], "-ex") == 0) Eye_x  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-ey") == 0) Eye_y  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-py") == 0) Pro_y  = atof(argv[++i]) ;
	      else if (strcmp(argv[i], "-head") == 0) Head = 0 ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	bird_init(stat_frame)
int	stat_frame ;
{
	int	n ;
	void	minmax_cal(), eye_point(), frame_plot() ;

	if (Dx == 0.0 || Dy == 0.0 || Hscale == 0.0) {
	  fprintf(stderr, " Dx, Dy, Hscale error %f %f %f.\n", Dx, Dy, Hscale) ;
	  exit(-1) ;
	  }
	if (Eye_y != 0.0) Eye_y = -Eye_y ;
	if (Z_rad < 0.0) Z_rad = Z_rad + 360.0 ;
	X3str = (float)(Nx - 1) * Dx / 2.0 ;
	Y3str = (float)(Ny - 1) * Dy / 2.0 ;
/*	Z3str = (Hmax - Hmin) / 2.0 + Hmin ; */
	Z3str = Hmin ;
	X_rad = X_rad / 180.0 * 3.141592 ;
	Z_rad = Z_rad / 180.0 * 3.141592 ;
	C_Z_rad = cos(Z_rad) ;
	S_Z_rad = sin(Z_rad) ;
	C_X_rad = cos(X_rad) ;
	S_X_rad = sin(X_rad) ;
	if (Hscale < 0.0) {
	  Hscale = -max((float)Nx*Dx, (float)Ny*Dy) / (Hmax-Hmin) * Hscale ;
	  fprintf(stderr, " Hscale = %f\n", Hscale) ;
	  }
	for (n = 0; n < Nx*Ny; n++) H[n] = H[n] * Hscale ;
	Und  = Und  * Hscale ;
	Hmin = Hmin * Hscale ;
	Hmax = Hmax * Hscale ;
	minmax_cal(stat_frame) ;
	eye_point() ;
	if (Scale == 0.0)
	  Scale = min(X_size / (X2max-X2min), Y_size / (Y2max-Y2min)) ;
	X_size = (X2max - X2min) * Scale ;
	Y_size = (Y2max - Y2min) * Scale ;
	fprintf(stderr, " Scale, X_size, Y_size = %f %f %f\n",
	  Scale, X_size, Y_size) ;
	if (Head == 1) printf("A %f %f %f %f\n",
	  X2min*Scale+Xshift,        Y2min*Scale+Yshift,
	  X2min*Scale+Xshift+X_size, Y2min*Scale+Yshift+Y_size) ;
	if (stat_frame == 1) {
	  printf("Lw %d\n", max(Pen_m, 1)) ;
	  frame_plot() ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	conv(x, y, z, xd, yd)
float	x, y, z ;
float	*xd, *yd ;
{
	float	w ;
	void	rotate() ;

	rotate(x, y, z, xd, &w, yd) ;
	*xd = (*xd) * Scale + Xshift ;
	*yd = (*yd) * Scale + Yshift ;
	}

/* -------------------------------------------------------------------------- */
void	rotate(xi, yi, zi, xo, yo, zo)
float	xi, yi, zi ;
float	*xo, *yo, *zo ;
{
	float   x1, y1, z1, x2, y2, z2, w ;

	x1 = xi - X3str ;
	y1 = yi - Y3str ;
	z1 = zi - Z3str ;
	z2 = z1 ;
	if (S_Z_rad == 0.0) {
	  x2 = x1 ;
	  y2 = y1 ;
	  }
	else {
	  x2 = x1 * C_Z_rad - y1 * S_Z_rad ;
	  y2 = x1 * S_Z_rad + y1 * C_Z_rad ;
	  }
	x1 = x2 ;
	if (C_X_rad == 0.0) {
	  y1 = - z2 ;
	  z1 =   y2 ;
	  }
	else {
	  y1 = y2 * C_X_rad - z2 * S_X_rad ;
	  z1 = y2 * S_X_rad + z2 * C_X_rad ;
	  }
	if (Eye_y != 0.0) {
/*	  w = -y1 / (Eye_y + y1) ;	*/
/*	  *xo = x1 + w * (x1 + Eye_x) ;	*/
/*	  *yo = 0.0 ;			*/
/*	  *zo = z1 * (1.0 + w) ; 		*/
	  w = Pro_y / (y1 - Eye_y) ;
	  *xo = x1 * w ;
	  if (Pro_y == 1.0) *yo = 1.0 ;
	  else *yo = y1 - Eye_y - Pro_y ;
	  *zo = z1 * w ;
	  }
	else {
	  *xo = x1 ;
	  *yo = 1.0 ;
	  *zo = z1 ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	rotate_r(xi, yi, zi, xo, yo, zo)
float	xi, yi, zi ;
float	*xo, *yo, *zo ;
{
	float   x1, y1, z1 ;

	x1 =  xi ;
	if (C_X_rad == 0.0) {
	  y1 = -zi * S_X_rad ;
	  z1 =  yi * S_X_rad ;
	  }
	else {
	  y1 = yi * C_X_rad - zi * S_X_rad ;
	  z1 = yi * S_X_rad + zi * C_X_rad ;
	  }
	*zo = z1 + Z3str ;
	if (S_Z_rad == 0.0) {
	  *xo = x1 * C_Z_rad + X3str ;
	  *yo = y1 * C_Z_rad + Y3str ;
	  }
	else {
	  *xo = x1 * C_Z_rad - y1 * S_Z_rad + X3str ;
	  *yo = x1 * S_Z_rad + y1 * C_Z_rad + Y3str ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	minmax_cal(stat_frame)
int	stat_frame ;
{
	int	ix, iy ;
	float	x, y, z, xd, yd, zd ;
	void	rotate() ;

	X2min =  9.0e29 ;
	Y2min =  9.0e29 ;
	X2max = -9.0e29 ;
	Y2max = -9.0e29 ;
	if (stat_frame == 1) {
	  for (ix = 0; ix < 2; ix++) {
	    x = (float)((Nx - 1) * ix) * Dx ;
	    for (iy = 0; iy < 2; iy++) {
	      y = (float)((Ny - 1) * iy) * Dy ;
	      z = Hmin ;
	      rotate(x, y, z, &xd, &yd, &zd) ;
	      fprintf(stderr, " %12.3f %12.3f %12.3f -> %12.3f %12.3f %12.3f\n",
	        x, y, z, xd, yd, zd) ;
	      if (yd > 0.0) {
	        X2min = min(X2min, xd) ;
	        Y2min = min(Y2min, zd) ;
	        X2max = max(X2max, xd) ;
	        Y2max = max(Y2max, zd) ;
	        }
	      }
	    }
	  }
	for (ix = 0; ix < Nx; ix++) {
	  x = (float)ix * Dx ;
	  for (iy = 0; iy < Ny; iy++) {
	    z = H[ix*Ny+iy] ;
	    if (z != Und) {
	      y = (float)iy * Dy ;
	      rotate(x, y, z, &xd, &yd, &zd) ;
	      if (yd > 0.0) {
	        X2min = min(X2min, xd) ;
	        Y2min = min(Y2min, zd) ;
	        X2max = max(X2max, xd) ;
	        Y2max = max(Y2max, zd) ;
	        }
	      else if (xd < X2min || xd > X2max || yd < Y2min || yd > Y2max)
	        H[ix*Ny+iy] = Und ;
	      }
	    }
	  }
	if (X2min > X2max) {
	  fprintf(stderr, " Minmax calculation error %d.\n", stat_frame) ;
	  exit(-1) ;
	  }
	fprintf(stderr, " Minmax = %f %f %f %f\n", X2min, Y2min, X2max, Y2max) ;
	}
/* -------------------------------------------------------------------------- */
void	eye_point()
{
	float	x, y, z, xd, yd, zd ;
	void	rotate_r() ;

	x = Eye_x ;
	y = Eye_y ;
	if (y == 0.0) y = -9e29 ;
	z = 0.0 ;
	C_X_rad = cos(-X_rad) ;
	S_X_rad = sin(-X_rad) ;
	C_Z_rad = cos(-Z_rad) ;
	S_Z_rad = sin(-Z_rad) ;
	rotate_r(x, y, z, &xd, &yd, &zd) ;
	xd = xd / Dx + 0.5 ;
	yd = yd / Dy + 0.5 ;
	if (xd < 0.0) Nx_c = -1 ;
	else if (xd > (float)Nx) Nx_c = Nx ;
	else Nx_c = (int)xd ;
	if (yd < 0.0) Ny_c = -1 ;
	else if (yd > (float)Ny) Ny_c = Ny ;
	else Ny_c = (int)yd ;
	C_X_rad = cos(X_rad) ;
	S_X_rad = sin(X_rad) ;
	C_Z_rad = cos(Z_rad) ;
	S_Z_rad = sin(Z_rad) ;
	fprintf(stderr, " Nx_c, Ny_c %d %d\n", Nx_c, Ny_c) ;
	}
/* -------------------------------------------------------------------------- */
void	frame_plot()
{
	float	x, y, z, x1, y1, x2, y2 ;
	float	xd, yd ;
	int	i, j, k ;
	void	conv() ;

	for (k = 0; k < 5; k++) {
	  i = k % 4 ;
	  for (j = 0; j < 2; j++) {
	    z = Hmin ;
	    if (i == 0) {
	      x = 0.0 ;
	      y = 0.0 ;
	      if (j != 0) z = H[0] ;
	      }
	    else if (i == 1) {
	      x = (float)(Nx-1) * Dx ;
	      y = 0.0 ;
	      if (j != 0) z = H[Nx*Ny-Ny]  ;
	      }
	    else if (i == 2) {
	      x = (float)(Nx-1) * Dx ;
	      y = (float)(Ny-1) * Dy ;
	      if (j != 0) z = H[Nx*Ny-1]  ;
	      }
	    else {
	      x = 0.0 ;
	      y = (float)(Ny-1) * Dy ;
	      if (j != 0) z = H[Ny-1]  ;
	      }
	    if (z == Und) z = Hmax ;
	    conv(x, y, z, &xd, &yd) ;
	    x2 = xd ;
	    y2 = yd ;
	    if (j == 1) {
	      printf("m %f %f\n", x1, y1) ;
	      printf("l %f %f\n", x2, y2) ;
	      }
	    else {
	      if (k > 0) {
	        printf("m %f %f\n", x1, y1) ;
	        printf("l %f %f\n", x2, y2) ;
	        }
	      x1 = x2 ;
	      y1 = y2 ;
	      }
	    }
	  }
	conv(X3str, Y3str, Z3str, &xd, &yd) ;
	fprintf(stderr, " center = %f %f\n", xd, yd) ;
	}
