/*
*  Program name:             contour ver. 2.0
*
*  Program function:         contour line output
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         May 30, 1995 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f JKM filename ( - : standard input )
*                            -l value scale
*                            -m character size (mm)
*                            -s contour start value ( default : min )
*                            -e contour end   value ( default : max )
*                            -i contour interval
*                            -t main contour interval
*                            -head headder not out
*                            -window window option not out
*                            -a all contour line plot
*                            -c Title character
*                            -int Integer character
*                            -clr character horizontal position
*                            -v contour value only
*
*  Input:                    JKM grid file ( program open )
*
*  Output:                   JKG contour line vector
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define	bitset(a, b)	((a) |  (1L << (b)))
#define	bitchk(a, b)	((a) &  (1L << (b)))
#define	bitclr(a, b)	((a) & ~(1L << (b)))
#define	max(a, b)	(((a) > (b)) ? (a) : (b))

char	Cmnt[100] ;
long	Nx, Ny ;
double	Xstr, Ystr, Dx, Dy ;
float	Und, Zmin, Zmax ;
float	*Z ;
char	Form[17] ;

long	Inter, Nmain, Nsub, Zstr, Zend, Head, Window, Str_Rotate, Pen_Width ;
long	Intc, Cvo ;
float	Sm, Zscale, Clr ;
char	Dsn[80], Title[80] ;

#define	D(a, b)		((a) * Ny + (b))

unsigned long	Gx, Gy ;
unsigned long	*G ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	x, y ;
	long	i ;
	void	Command_Line(), Grid_Read(), Header_Write(), Draw_Contour() ;

	Command_Line(argc, argv) ;

	Grid_Read() ;

	fprintf(stderr, " Grid min = %f max = %f\n", Zmin, Zmax) ;

	if ((Zstr != -99999 && Zmax < (float)Zstr) ||
	    (Zend != -99999 && Zmin > (float)Zend)) {
		fprintf(stderr, " Start/End value error.\n") ;
		return(-1) ;
		}
	Und  = Und  * Zscale ;
	Zmin = Zmin * Zscale ;
	Zmax = Zmax * Zscale ;
	for (i = 0; i < Nx*Ny; i++) Z[i] = Z[i] * Zscale ;

	if (Zstr == -99999) Zstr = Zmin - 1.0 ;
	if (Zend == -99999) Zend = Zmax + 1.0 ;
	if (Zstr > Zend) {
	  fprintf(stderr, " Command line argument error.\n") ;
	  fprintf(stderr, " %d %d %d %d %f %f\n",
	    Inter,Nmain,Zstr,Zend,Zscale,Sm) ;
	  return(-1) ;
	  }

	Header_Write() ;
	if (Sm != 0.0) {
	  printf("Ss %f\n", Sm) ;
	  printf("Sr 0.0\n") ;
	  printf("Sh %f\n", Clr) ;
	  printf("Sv 0.0\n") ;
	  Str_Rotate = 0 ;
	  }

	Draw_Contour() ;

	free(Z) ;

	if (Window == 1) {
		x = (double)(Nx - 1) * Dx ;
		y = (double)(Ny - 1) * Dy ;
		printf("W %lf %lf %lf %lf\n", Xstr, Ystr, Xstr+x, Ystr+y) ;
		}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Command_Line(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	i ;
	double	atof() ;

	Zscale = 1.0 ;
	Sm = 0.0 ;
	Zstr = -99999 ;
	Zend = -99999 ;
	Clr = 1.0 ;
	Cvo = 0 ;
	Inter = 1 ;
	Intc = 0 ;
	Nmain = 1 ;
	Nsub = 0 ;
	Dsn[0] = '\0' ;
	Title[0] = '\0' ;
	Head = 1 ;
	Window = 1 ;

	for (i = 1; i < argc; i++) {
	       if (!strcmp(argv[i], "-f")) strcpy(Dsn, argv[++i]) ;
	  else if (!strcmp(argv[i], "-l")) Zscale = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-m")) Sm     = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-clr")) Clr  = atof(argv[++i]) ;
	  else if (!strcmp(argv[i], "-s")) Zstr   = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-e")) Zend   = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-i")) Inter  = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-t")) Nmain  = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-int")) Intc = 1 ;
	  else if (!strcmp(argv[i], "-v")) Cvo = 1 ;
	  else if (!strcmp(argv[i], "-a")) Nsub   = 1 ;
	  else if (!strcmp(argv[i], "-head")) Head  = 0 ;
	  else if (!strcmp(argv[i], "-window")) Window  = 0 ;
	  else if (!strcmp(argv[i], "-c")) strcpy(Title, argv[++i]) ;
	  else if (!strcmp(argv[i], "-h")) {
	    fprintf(stderr, " USAGE: contour -f $ -l # -m # -s # -e # ") ;
	    fprintf(stderr, "-i # -t # -a -head -window -c $ -b -w\n") ;
	    exit(0) ;
	    }
	  }
	if (Inter <= 0 || Zscale <= 0.0 || Sm < 0.0) {
	  fprintf(stderr, " Command line argument error.\n") ;
	  fprintf(stderr, " %d %d %d %d %f %f\n",
	    Inter,Nmain,Zstr,Zend,Zscale,Sm) ;
	  exit(-1) ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	Grid_Read()
{
	FILE	*fp ;
	long	n ;

	if (!strcmp(Dsn, "stdin") || Dsn[0] == '\0') fp = stdin ;
	else if ((fp = fopen(Dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open grid file (%s).\n", Dsn) ;
	  exit(-1) ;
	  }
	if ((fgets(Cmnt, 100, fp)) == NULL) {
	  fprintf(stderr, " Can't read comment.\n") ;
	  exit(-1) ;
	  }
	if ((fscanf(fp, "%d %d %lf %lf %lf %lf %f %f %f",
	  &Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Zmin, &Zmax)) != 9 ) {
	  fprintf(stderr, " Can't read header.\n") ;
	  exit(-1) ;
	  }
	if ((Z = (float *)calloc(Nx*Ny, sizeof(*Z))) == NULL) {
	  fprintf(stderr, "Can't alloc memory Z.\n") ;
	  exit(-1) ;
	  }
	for (n = 0; n < Nx*Ny; n++) if ((fscanf(fp, "%f", &Z[n])) != 1) {
	  fprintf(stderr, " Can't read grid %ld %ld %f %f.\n",
	    n, ftell(fp), Z[n-2], Z[n-1]) ;
	  exit(-1) ;
	  }
	if (fp != stdin) fclose(fp) ;

	if (Zmin >= Zmax) {
	  Zmin =  9.9e9 ;
	  Zmax = -9.9e9 ;
	  for (n = 0; n < Nx*Ny; n++) if (Z[n] != Und) {
	    if (Z[n] < Zmin) Zmin = Z[n] ;
	    if (Z[n] > Zmax) Zmax = Z[n] ;
	    }
	  }
	if (Zmin >= Zmax) {
	  fprintf(stderr, " Invarid grid data.\n") ;
	  exit(-1) ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	Header_Write()
{
	double	x, y ;

	x = (double)(Nx - 1) * Dx ;
	y = (double)(Ny - 1) * Dy ;
	if (Head == 1) {
	  printf("A %lf %lf %lf %lf\n", Xstr, Ystr, Xstr+x, Ystr+y) ;
	  printf("Lw 5\n") ;
	  printf("m %lf %lf\n", Xstr,   Ystr) ;
	  printf("l %lf %lf\n", Xstr,   Ystr+y) ;
	  printf("l %lf %lf\n", Xstr+x, Ystr+y) ;
	  printf("l %lf %lf\n", Xstr+x, Ystr) ;
	  printf("l %lf %lf\n", Xstr,   Ystr) ;
	  }
	if (Title[0] != '\0') {
	  printf("Ss %f\n", max(15.0, Sm*3.0)) ;
	  printf("Sr 0.0\n") ;
	  printf("Sv -0.2\n") ;
	  printf("Sh -0.5\n") ;
	  Str_Rotate = 0 ;
	  printf("m %lf %lf\n", Xstr, Ystr+y) ;
	  printf("s %s\n", Title) ;
	  }
	}
/* -------------------------------------------------------------------------- */
unsigned long	E(x, y, xy)
long	x, y, xy ;
{
	if (x < 0 || x >= Gx || y < 0 || y >= Gy) return(0L) ;
	return(G[(x * Gy + y) * 2 + xy]) ;
	}
/* -------------------------------------------------------------------------- */
void	Draw_Contour()
{
	long	mp, ix, iy, ij, i, b, labs() ;
	long	key, v, vs, ve ;
	float	dstep, val ;

	unsigned long	E() ;

	void	G_Alloc(), G_Set(), Do_For() ;
/* -------------------------------------------------------------------------- */
	val = max(Xstr+Dx*(double)Nx, Ystr+Dy*(double)Ny) ;
	     if (val < 10.0)        strcpy(Form, "%c%12.9lf%12.9lf\n") ;
	else if (val < 100.0)       strcpy(Form, "%c%12.8lf%12.8lf\n") ;
	else if (val < 1000.0)      strcpy(Form, "%c%12.7lf%12.7lf\n") ;
	else if (val < 10000.0)     strcpy(Form, "%c%12.6lf%12.6lf\n") ;
	else if (val < 100000.0)    strcpy(Form, "%c%12.5lf%12.5lf\n") ;
	else if (val < 1000000.0)   strcpy(Form, "%c%12.4lf%12.4lf\n") ;
	else if (val < 10000000.0)  strcpy(Form, "%c%12.3lf%12.3lf\n") ;
	else if (val < 100000000.0) strcpy(Form, "%c%12.2lf%12.2lf\n") ;

	if (Inter == 0 || Nmain == 0 || Zstr > Zend) {
	  fprintf(stderr, " Illegal argument for Draw_Contour %d %d %d %d %f\n",
	    Inter, Nmain, Zstr, Zend, Sm) ;
	  exit(-1) ;
	  }

	dstep = (float)(labs(Inter) * 2) ;
	vs = Zstr / Inter ;
	ve = Zend / Inter ;
	G_Alloc() ;
	printf("Lw 1\n") ;
	Pen_Width = 1 ;

	for (v = vs; v <= ve; v++ ) {
	  val = (float)(v * Inter) ;
	  fprintf(stderr, " val = %f\n", val) ;
	  printf("# %f\n", val) ;
/*	  printf("m %lf %lf\n", Xstr, Ystr) ;
	  printf("s A\n") ;				*/
	  key = v - v / Nmain * Nmain ;
	  if (key == 0) {
	    if (Pen_Width != 3) {
	      if (Cvo == 0) printf("Lw 3\n") ;
	      Pen_Width = 3 ;
	      }
	    }
	  else {
	    if (Pen_Width != 1) {
	      if (Cvo == 0) printf("Lw 1\n") ;
	      Pen_Width = 1 ;
	      }
	    }
	  G_Set(val) ;
	  mp = 1 ;
	  for (iy = 0; iy < Ny; iy++ ) for (ij = 0; ij < 2; ij++) {
	    i = 0 ;
	    for (ix = 0; ix < Gx; ix++) {
	      if (E(ix, iy, ij) != 0) {
	        for (b = 0; b < 32; b++) {
	          if (bitchk(E(ix,iy,ij), b) == 0) continue ;
	          Do_For(&mp, ix, iy, ij, i, b, key, val, dstep) ;
	          }
	        }
	      i = i + 32 ;
	      }
	    }
	  }
	free(G) ;
	}
/* -------------------------------------------------------------------------- */
void	Do_For(mp, ix, iy, ij, i, b, key, val, dstep)
long	*mp, iy, ij, ix, i, b, key ;
float	val, dstep ;
{
	long	bb, m, n, jx, jy, kx, kl ;
	float	x, y, vv, vn ;

	unsigned long	E(), w0, w1 ;

	static long	rm[] = {12,0,8,4,8,4,12,0,4,8,0,12,0,12,4,8} ;
	static long	ad[] = {90, 270, 0, 180} ;

	void	Draw_Seg(), Draw_Val(), G_Shift() ;

	jx = i + b ;
	kx = ix ;
	jy = iy ;
	bb = b ;
	kl = ij ;
	m = ij * 8 ;
	n = 1 ;

Point1:	vv = Z[D(jx, jy)] ;
	vn = Z[D(jx+1-kl, jy+kl)] ;
	if (key == 0 || Nsub == 1 || (long)((vn - vv) / dstep) == 0)
	  Draw_Seg(jx, jy, val, vv, vn, kl, &x, &y, mp) ;
	else *mp = 1 ;

Point2:	G_Shift(&m, &jx, &jy, &kx, &bb) ;
	if (m%4 == 0) {
	  if (key == 0 && Nmain >= 0 && Sm != 0.0) {
/*	    printf("# m = %d\n", m) ;	*/
	    m = m / 4 ;
	    Draw_Val(x, y, val, ad[m-1]) ;
	    }
	  *mp = 1 ;
	  if (n != 0) {
	    if (bitchk(E(ix, iy, ij), b) == 0) return ;
	    jx = i + b ;
	    kx = ix ;
	    jy = iy ;
	    bb = b ;
	    kl = ij ;
	    m = ij * 8 + 4 ;
	    n = 0 ;
	    G[(kx*Gy+jy)*2+kl] = bitclr(E(kx,jy,kl), bb) ;
	    goto Point1 ;
	    }
	  }

	else {
	  w0 = E(kx, jy, 0) ;
	  w1 = E(kx, jy, 1) ;
	  if ( (m%2 == 1 && m < 8) || (m%2 == 0 && m > 8) ) {
	    if (w1 == 0 || bitchk(w1, bb) == 0) goto Point2 ;
	    kl = 1 ;
	    }
	  else {
	    if (w0 == 0 || bitchk(w0, bb) == 0) goto Point2 ;
	    kl = 0 ;
	    }
	  m = rm[m-1] ;
	  if (kl == 0 && w0 != 0) G[(kx*Gy+jy)*2+kl] = bitclr(w0, bb) ;
	  if (kl == 1 && w1 != 0) G[(kx*Gy+jy)*2+kl] = bitclr(w1, bb) ;
	  goto Point1 ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	G_Alloc()
{
	long	i, n ;

	Gx = Nx / 32 + 1 ;
	Gy = Ny + 1 ;
	n = Gx * Gy * 2 ;
	if ((G = (unsigned long *)calloc(n, sizeof(*G))) == NULL) {
	  fprintf(stderr, " Can't alloc memory G.\n") ;
	  exit(-1) ;
	  }
	for (i = 0; i < n; i++) G[i] = 0 ;
	}
/* -------------------------------------------------------------------------- */
void	G_Set(val)
float	val ;
{
	long	ix, iy, k, m, kj ;

	for (iy = 0; iy < Ny; iy++) {
	  for (ix = 0; ix < Nx; ix++) {
	    if (Z[D(ix,iy)] == Und) continue ;
	    if (ix < Nx-1 && Z[D(ix+1,iy)] != Und) {
	      if ((Z[D(ix,iy)] >= val && Z[D(ix+1,iy)] <  val) ||
	          (Z[D(ix,iy)] <  val && Z[D(ix+1,iy)] >= val)) {
	        k = ix / 32 ;
	        m = ix % 32 ;
	        kj = (k * Gy + iy) * 2 ;
	        G[kj] = bitset(G[kj], m) ;
	        }
	      }
	    if (iy < Ny-1 && Z[D(ix,iy+1)] != Und) {
	      if ((Z[D(ix,iy)] >= val && Z[D(ix,iy+1)] <  val) ||
	          (Z[D(ix,iy)] <  val && Z[D(ix,iy+1)] >= val)) {
	        k = ix / 32 ;
	        m = ix % 32 ;
	        kj = (k * Gy + iy) * 2 + 1 ;
	        G[kj] = bitset(G[kj], m) ;
	        }
	      }
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	G_Shift(m, jx, jy, kx, bb)
long	*m, *jx, *jy, *kx, *bb ;
{
	(*m)++ ;
	if (*m == 1 || *m == 7 || *m == 12 || *m == 14) {
	  if (*m == 1 || *m == 7) (*jy)-- ;
	  (*jx)++ ;
	  (*bb)++ ;
	  if (*bb > 31) {
	    (*kx)++ ;
	    *bb = 0 ;
	    }
	  }
	else if (*m == 2 || *m == 8 || *m == 9 || *m == 15) {
	  if (*m == 9 || *m == 15) (*jy)++ ;
	  (*jx)-- ;
	  (*bb)-- ;
	  if (*bb < 0) {
	    (*kx)-- ;
	    *bb = 31 ;
	    }
	  }
	else if (*m ==  4 || *m ==  6) (*jy)++ ;
	else if (*m == 10 || *m == 16) (*jy)-- ;
	}
/* -------------------------------------------------------------------------- */
void	Draw_Seg(jx, jy, val, vv, vn, kl, x, y, mp)
long	jx, jy, kl, *mp ;
float	val, vv, vn, *x, *y ;
{
	static double	xr=0.0, yr=0.0 ;
	float	d ;

	*x = (double)jx * Dx ;
	*y = (double)jy * Dy ;
	d = (val - vv) / (vn - vv) ;
	if (kl) *y = *y + d * Dy ;
	else    *x = *x + d * Dx ;
	if (*mp == 1) {
	  xr = *x ;
	  yr = *y ;
	  *mp = 2 ;
	  }
	else {
	  if (*mp == 2) {
	    if (Cvo == 0) printf(Form, 'm', Xstr+xr, Ystr+yr) ;
	    *mp = 3 ;
	    }
	  if (Cvo == 0) printf(Form, 'l', Xstr+(*x), Ystr+(*y)) ;
	  }
	}
/* -------------------------------------------------------------------------- */
void	Draw_Val(x, y, val, ad)
double	x, y ;
float	val ;
long	ad ;
{
	long	i ;
	float	val2 ;
	char	fval[9] ;

	val2 = val / Zscale ;
	i = (long)val2 ;
	if (Intc == 1) sprintf(fval, "%ld\0",  i) ;
	else {
	       if (Zscale ==     1.0) sprintf(fval, "%8.0f\0", val2) ;
	  else if (Zscale <=    10.0) sprintf(fval, "%8.1f\0", val2) ;
	  else if (Zscale <=   100.0) sprintf(fval, "%8.2f\0", val2) ;
	  else if (Zscale <=  1000.0) sprintf(fval, "%8.3f\0", val2) ;
	  else if (Zscale <= 10000.0) sprintf(fval, "%8.4f\0", val2) ;
	  }
	if (ad != Str_Rotate) {
	  printf("Sr %ld\n", ad) ;
	  Str_Rotate = ad ;
	  }
	printf("m %12.2lf%12.2lf\n", Xstr+x, Ystr+y) ;
	if (Pen_Width != 1 && Cvo == 0) printf("Lw 1\n") ;
	printf("s ") ;
	for (i = 0; i < 9; i++) {
	  if (fval[i] == '\0') break ;
	  if (fval[i] != ' ') putchar(fval[i]) ;
	  }
	putchar('\n') ;
	if (Pen_Width != 1 && Cvo == 0) printf("Lw %ld\n", Pen_Width) ;
	}
