/*
*  Program name:             deloconC  ver. 1.0
*
*  Program function:         Color-Pattern Contour Program
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            January 9, 2003
*
*  Date of revision:
*
*  Arguments:                -f:  JKD file name           ( JK-DELONE File )
*                            -i:  Contour interval        ( default = 1.0 )
*                            -s:  Contour start val.      ( default = min )
*                            -e:  Contour end   val.      ( default = max )
*                            -lf: Legend table file name  ( JK-LEGEND File )
*                            -ls: Legend table start pos. ( default = 1 )
*                            -bw: Legend box width        ( default = 1.0 cm )
*                            -bx: Legend box x-position   ( default = Xmax )
*                            -by: Legend box y-position   ( default = Ymin )
*                            -dw: delene pen width        ( default = 0 )
*                            -dc: delene pen color        ( default = none )
*                            -cw: contour pen width       ( default = 1 )
*                            -cc: contour pen color       ( default = none )
*                            -pl: polygon plot            ( default = 0 )
*                            -ps: postscript print        ( default = 0 )
*                            -pn: plot item number        ( default = 1 )
*                            -head: Header no output      ( default = out )
*
*  Input:                    JKM file (program open)
*
*  Output:                   JKG file (to stdout)
*/

#include <stdio.h>
/* #include <math.h> */
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#define	MAXC	100000

double	Xmin, Ymin, Xmax, Ymax, Xwid, Ywid, Xstr, Ystr, Boxx, Boxy ;
int	Ntbl, Color[MAXC+1], Pattern[MAXC+1], PatternC[MAXC+1] ;
float	Und, Hmin, Hmax, Hscale, Hstr, Hend, Hint, Pn ;
int	Np, Nt, *T1, *T2, *T3 ;
float   *Xp, *Yp, *Hp ;
int	Cmax ;
int	Wid_d, Wid_c, Col_d, Col_c ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	float	boxw ;
	int	i, n, lstr, head ;
	char    dsn_c[80], dsn_t[80] ;
	void	Delone_Read(), Table_Read(), Cont4p(), Cont4s(), Cont4d() ;
/* -------------------------------------------------------------------------- */
	Hstr   = -9999.0 ;
	Hend   = -9999.0 ;
	Hint   = 1.0 ;
	Hscale = 1.0 ;
	Wid_d = 0 ;
	Wid_c = 1 ;
	Col_d = -1 ;
	Col_c = 0 ;
	lstr = 1 ;
	boxw = 1.0 ;
	head = 1 ;
	Boxx = 9.0e-9 ;
	Boxy = 9.0e-9 ;
	Ntbl = 0 ;
	Pn = 1 ;
	dsn_c[0] = '\0' ;
	dsn_t[0] = '\0' ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-f")  == 0) strcpy(dsn_c, argv[++i]) ;
	  else if (strcmp(argv[i], "-s")  == 0) Hstr = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-e")  == 0) Hend = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-i")  == 0) Hint = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-lf") == 0) strcpy(dsn_t, argv[++i]) ;
	  else if (strcmp(argv[i], "-ls") == 0) lstr = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-bw") == 0) boxw = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-bx") == 0) Boxx = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-by") == 0) Boxy = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-dw") == 0) Wid_d = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-cw") == 0) Wid_c = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-dc") == 0) Col_d = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-cc") == 0) Col_c = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-pn") == 0) Pn = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-head") == 0) head = 0 ;
	  else if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr," USAGE: deloconC -f $ -s # -e # -i # ") ;
	    fprintf(stderr,"-lf $ -ls # -bw # -bx # -by # -dw # -cw # -dc # -cc # ") ;
	    fprintf(stderr,"-head -h\n") ;
	    return(0) ;
	    }
	  }

	Delone_Read(dsn_c) ;

	if (Boxx == 9.0e-9) Boxx = Xmax ;
	if (Boxy == 9.0e-9) Boxy = Ymin ;
	Cmax = (Hend - Hstr) / Hint + 0.5 ;
	for (n = 0; n < Np ; n++) {
	  Hp[n] = (Hp[n] - Hstr) / Hint ;
/*	  if (Hp[n] <     0) Hp[n] = 0 ;
	  if (Hp[n] >= Cmax) Hp[n] = Cmax - 1 ; */
	  }
	fprintf(stderr, " Hstr, Hend, Hint, Cmax = %f %f %f %d %d\n",
	  Hstr, Hend, Hint, Cmax, MAXC-1) ;
	if (Cmax > MAXC-1) {
	  fprintf(stderr, " Cmax > MAXC-1.\n") ;
	  return(-1) ;
	  }

	if (head == 1) {
	  printf("A %lf %lf %lf %lf\n", Xmin, Ymin, Xmax, Ymax) ;
/*	  printf("Lw 5\n") ;
	  printf("m %lf %lf\n", Xmin, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymax) ;
	  printf("l %lf %lf\n", Xmax, Ymin) ;
	  printf("l %lf %lf\n", Xmin, Ymin) ;	*/
	  }

	if (dsn_t[0] != '\0') Table_Read(lstr, boxw, dsn_t) ;

	Cont4d() ;

	free(Xp) ;
	free(Yp) ;
	free(Hp) ;
	free(T1) ;
	free(T2) ;
	free(T3) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Cont4d()
{
	int	it, o, n, p, p1, p2, p3, o1, o2, o3, omin, omax, gauss(), c ;
	float	x1, x2, x3, y1, y2, y3, h1, h2, h3, xr[6], yr[6] ;
	void	where() ;

	if (Wid_c !=  0) printf("Lw %d\n", Wid_c) ;
	if (Wid_d !=  0) printf("Lw %d\n", Wid_d) ;
	if (Col_c != -1) printf("Lc %d\n", Col_c) ;
	if (Col_d != -1) printf("Lc %d\n", Col_d) ;
	for (it = 0; it < Nt; it++) {
	  p1 = T1[it] - 1 ;
	  p2 = T2[it] - 1 ;
	  p3 = T3[it] - 1 ;
	  x1 = Xp[p1] ;
	  x2 = Xp[p2] ;
	  x3 = Xp[p3] ;
	  y1 = Yp[p1] ;
	  y2 = Yp[p2] ;
	  y3 = Yp[p3] ;
	  h1 = Hp[p1] ;
	  h2 = Hp[p2] ;
	  h3 = Hp[p3] ;
	  o1 = gauss(h1) ;
	  o2 = gauss(h2) ;
	  o3 = gauss(h3) ;
	  omin = o1 ;
	  omax = o1 ;
	  if (o2 < omin) omin = o2 ;
	  if (o3 < omin) omin = o3 ;
	  if (o2 > omax) omax = o2 ;
	  if (o3 > omax) omax = o3 ;
	  if (Ntbl != 0) {
	    for (o = omin; o <= omax; o++) {
	      c = o ;
	      if (c < 0) c = MAXC ;
	      if (c > MAXC) c = MAXC ;
	      n = 0 ;
	      where(x1, y1, h1, o1, x2, y2, h2, o2, o, &n, xr, yr) ;
	      where(x2, y2, h2, o2, x3, y3, h3, o3, o, &n, xr, yr) ;
	      where(x3, y3, h3, o3, x1, y1, h1, o1, o, &n, xr, yr) ;
	      if (n > 0) {
	        if (Pattern[o] != 0 || Color[o] != 0) {
	          printf("Fb %d\n", Color[c]) ;
	          printf("Fp %d\n", Pattern[c]) ;
	          if (Pattern[c] != 0) printf("Fc %d\n", PatternC[c]) ;
	          printf("n\n") ;
	          printf("m %f %f\n", xr[0], yr[0]) ;
	          printf("m %f %f\n", xr[1], yr[1]) ;
	          if (xr[2] != xr[1] || yr[2] != yr[1]) printf("m %f %f\n", xr[2], yr[2]) ;
	          printf("m %f %f\n", xr[3], yr[3]) ;
	          if (n > 4) {
	            if (xr[4] != xr[3] || yr[4] != yr[3]) printf("m %f %f\n", xr[4], yr[4]) ;
	            printf("m %f %f\n", xr[5], yr[5]) ;
	            }
	          printf("f\n") ;
	          }
	        }
	      }
	    }
	  if (Wid_d != 0) {
	    if (Wid_c != 0) printf("Lw %d\n", Wid_d) ;
	    if (Col_c != -1) printf("Lc %d\n", Col_d) ;
	    printf("m %f %f\n", x1, y1) ;
	    printf("l %f %f\n", x2, y2) ;
	    printf("l %f %f\n", x3, y3) ;
	    printf("l %f %f\n", x1, y1) ;
	    }
	  if (Wid_c != 0) {
	    if (Wid_d != 0) printf("Lw %d\n", Wid_c) ;
	    if (Col_d != -1) printf("Lc %d\n", Col_c) ;
	    for (o = omin; o <= omax; o++) {
	      c = o ;
	      if (c < 0) c = MAXC ;
	      if (c > MAXC) c = MAXC ;
	      n = 0 ;
	      where(x1, y1, h1, o1, x2, y2, h2, o2, o, &n, xr, yr) ;
	      where(x2, y2, h2, o2, x3, y3, h3, o3, o, &n, xr, yr) ;
	      where(x3, y3, h3, o3, x1, y1, h1, o1, o, &n, xr, yr) ;
	      if (n > 0) {
	        if (xr[2] != xr[1] || yr[2] != yr[1]) {
	          printf("m %f %f\n", xr[1], yr[1]) ;
	          printf("l %f %f\n", xr[2], yr[2]) ;
	          }
	        if (n == 4) {
	          if (xr[0] != xr[3] || yr[0] != yr[3]) {
	            printf("m %f %f\n", xr[3], yr[3]) ;
	            printf("l %f %f\n", xr[0], yr[0]) ;
	            }
	          }
	        else {
	          if (xr[4] != xr[3] || yr[4] != yr[3]) {
	            printf("m %f %f\n", xr[3], yr[3]) ;
	            printf("l %f %f\n", xr[4], yr[4]) ;
	            }
	          if (xr[0] != xr[5] || yr[0] != yr[5]) {
	            printf("m %f %f\n", xr[5], yr[5]) ;
	            printf("l %f %f\n", xr[0], yr[0]) ;
	            }
	          }
	        }
	      }
	    }
	  }
	}
/* -------------------------------------------------------------------------- */
void	where(x1, y1, h1, o1, x2, y2, h2, o2, o, n, xr, yr)
int	*n, o1, o2, o ;
float	x1, y1, h1, x2, y2, h2, xr[6], yr[6] ;
{
	float	r1, r2, w ;

	if (o1 == o2) {
	  if (o1 == o) {
	    xr[*n  ] = x1 ;
	    yr[*n  ] = y1 ;
	    xr[*n+1] = x2 ;
	    yr[*n+1] = y2 ;
	    *n = *n + 2 ;
	    }
	  return ;
	  }
	if (o1 < o && o2 < o) return ;
	if (o1 > o && o2 > o) return ;
	r1 = ((float)o     - h1) / (h2 - h1) ;
	if (r1 < 0.0) r1 = 0.0 ;
	if (r1 > 1.0) r1 = 1.0 ;
	r2 = ((float)(o+1) - h1) / (h2 - h1) ;
	if (r2 < 0.0) r2 = 0.0 ;
	if (r2 > 1.0) r2 = 1.0 ;
	if (r2 < r1) {
	  w = r1 ;
	  r1 = r2 ;
	  r2 = w ;
	  }
	xr[*n]   = (x2 - x1) * r1 + x1 ;
	yr[*n]   = (y2 - y1) * r1 + y1 ;
	xr[*n+1] = (x2 - x1) * r2 + x1 ;
	yr[*n+1] = (y2 - y1) * r2 + y1 ;
	*n = *n + 2 ;
	return ;
	}
/* -------------------------------------------------------------------------- */
void	Delone_Read(dsn_c)
char	*dsn_c ;
{
	int	i, j, k ;
	FILE	*fp ;
	char	line[100] ;

	if (dsn_c[0] == '-' || dsn_c[0] == '\0') {
	  fprintf(stderr, " Read Delone-data from standard input.\n") ;
	  fp = stdin ;
	  }
	else if ((fp = fopen(dsn_c, "r")) == NULL) {
	  fprintf(stderr, " Can't open Delone file '%s'.\n", dsn_c) ;
	  exit(-1) ;
	  }

	Xmin =  9.9e+29 ;
	Xmax = -9.9e+29 ;
	Ymin =  9.9e+29 ;
	Ymax = -9.9e+29 ;
	Hmin =  9.9e+29 ;
	Hmax = -9.9e+29 ;

/* Point Read */
	fgets(line, 100, fp) ;
	fgets(line, 100, fp) ;
	sscanf(line, "%d", &Np) ;
	if ((Xp = (float *)calloc(Np, sizeof(*Xp))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone Xp.\n") ;
	  exit(-1) ;
	  }
	if ((Yp = (float *)calloc(Np, sizeof(*Yp))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone Yp.\n") ;
	  exit(-1) ;
	  }
	if ((Hp = (float *)calloc(Np, sizeof(*Hp))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone Hp.\n") ;
	  exit(-1) ;
	  }
	for (i = 0; i < Np; i++) {
	  if (fgets(line, 100, fp) == NULL) {
	    fprintf(stderr, "deloconC: read error\n") ;
	    exit(-1) ;
	    }
	  if (Pn == 1)
	    j = sscanf(line, "%*d %*d %f %f %f", &Xp[i], &Yp[i], &Hp[i]) ;
	  else if (Pn == 2)
	    j = sscanf(line, "%*d %*d %f %f %*f %f", &Xp[i], &Yp[i], &Hp[i]) ;
	  else if (Pn == 3)
	    j = sscanf(line, "%*d %*d %f %f %*f %*f %f", &Xp[i], &Yp[i], &Hp[i]) ;
	  else if (Pn == 4)
	    j = sscanf(line, "%*d %*d %f %f %*f %*f %*f %f", &Xp[i], &Yp[i], &Hp[i]) ;
	  if (j != 3) {
	    fprintf(stderr, "deloconC: read error %s\n", line) ;
	    exit(-1) ;
	    }
	  if (Xp[i] < Xmin) Xmin = Xp[i] ;
	  if (Yp[i] < Ymin) Ymin = Yp[i] ;
	  if (Hp[i] < Hmin) Hmin = Hp[i] ;
	  if (Xp[i] > Xmax) Xmax = Xp[i] ;
	  if (Yp[i] > Ymax) Ymax = Yp[i] ;
	  if (Hp[i] > Hmax) Hmax = Hp[i] ;
	  }
	if (Hstr == Hend) {
	  i = Hmin / Hint ;
	  if (Hmin < 0.0) i = i - 1 ;
	  Hstr = (double)i * Hint ;
	  i = Hmax / Hint + 1 ;
	  if (Hmax < 0.0) i = i - 1 ;
	  Hend = (double)i * Hint ;
	  }

/* Line Read */
	fgets(line, 100, fp) ;
	fgets(line, 100, fp) ;
	sscanf(line, "%d", &j) ;
	for (i = 0; i < j; i++) {
	  if (fgets(line, 100, fp) == NULL) {
	    fprintf(stderr, "deloconC: read error\n") ;
	    exit(-1) ;
	    }
	  }

/* Triangle Read */
	fgets(line, 100, fp) ;
	fgets(line, 100, fp) ;
	sscanf(line, "%d", &Nt) ;
	if ((T1 = (int *)calloc(Nt, sizeof(*T1))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone T1.\n") ;
	  exit(-1) ;
	  }
	if ((T2 = (int *)calloc(Nt, sizeof(*T2))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone T2.\n") ;
	  exit(-1) ;
	  }
	if ((T3 = (int *)calloc(Nt, sizeof(*T3))) == NULL) {
	  fprintf(stderr, " Can't alloc color Delone T3.\n") ;
	  exit(-1) ;
	  }
	for (i = 0; i < Nt; i++) {
	  if (fgets(line, 100, fp) == NULL ||
	      sscanf(line, "%d %d %d %d", &j, &T1[i], &T2[i], &T3[i]) != 4) {
	    fprintf(stderr, "deloconC: read error\n") ;
	    exit(-1) ;
	    }
	  }

	if (fp != stdin) fclose(fp) ;
	}
/* -------------------------------------------------------------------------- */
void	Table_Read(lstr, boxw, dsn_t)
int	lstr ;
float	boxw ;
char	*dsn_t ;
{
	int	i, Colorw, Patternw, Patternc, w ;
	FILE	*fp ;
	void	Draw_Legend() ;

	if (boxw != 0.0) {
	  printf("Ss %f\n", boxw*0.35/2.54*72.0) ;
	  printf("Sr 0.0\n") ;
	  printf("Sh 0.0\n") ;
	  printf("Sv 0.5\n") ;
	  printf("m %lf %lf\n", Boxx, Boxy) ;
	  }
	if (dsn_t[0] == '\0') for (i = 0; i < MAXC; i++) {
	  Color[i] = 0 ;
	  Pattern[i] = 0 ;
	  PatternC[i] = 0 ;
	  }
	else {
	  if ((fp = fopen(dsn_t, "r")) == NULL) {
	    fprintf(stderr, " Can't open color table file.\n") ;
	    exit(-1) ;
	    }
	  i = 0 ;
	  while ((fscanf(fp, "%d %d %d %d", &w, &Colorw, &Patternw, &Patternc)) == 4) {
	    if (i >= lstr-1) {
	      if (Ntbl < Cmax) {
	        if (boxw != 0.0) Draw_Legend(boxw,i,Ntbl,Colorw,Patternw,Patternc, Cmax) ;
	        Color[Ntbl] = Colorw ;
	        Pattern[Ntbl] = Patternw ;
	        PatternC[Ntbl] = Patternc ;
	        }
	      Ntbl++ ;
	      }
	    i++ ;
	    }
	  }
	fclose(fp) ;
	Color[Cmax] = Color[Cmax-1] ;
	}
/* -------------------------------------------------------------------------- */
void	Draw_Legend(boxw, nc, nt, cl, pt, pc, Cmax)
float	boxw ;
int	nc, nt, cl, pt ;
{
	char	text[80] ;

	printf("Lw 2\n") ;
	if (pt != 0 || cl != 0) {
	  printf("Fb %d\n", cl) ;
	  printf("Fp %d\n", pt) ;
	  if (pt != 0) printf("Fc %d\n", pc) ;
	  }
	else return ;
	printf("n\n") ;
	printf("rm %f %f\n", boxw,     boxw*(float)nc) ;
	printf("rl %f %f\n", boxw,     boxw*(float)nc+boxw*0.6) ;
	printf("rl %f %f\n", boxw*2.0, boxw*(float)nc+boxw*0.6) ;
	printf("rl %f %f\n", boxw*2.0, boxw*(float)nc) ;
	printf("rl %f %f\n", boxw,     boxw*(float)nc) ;
	printf("f\n") ;
	if ((int)(Hint*1000.0+0.5)%10 != 0) {
	  if (nt == 0) sprintf(text, "s Bellow %.3f\0", Hstr+Hint) ;
	  else if (nt == Cmax-1) sprintf(text, "Above %.3f\0", Hstr+Hint*(float)nt) ;
	  else sprintf(text, "s %.3f ~ %.3f\0", Hstr+Hint*(float)nt,
	    Hstr+Hint*(float)(nt+1)) ;
	  }
	else if ((int)(Hint*100.0+0.5)%10 != 0) {
	  if (nt == 0) sprintf(text, "s Bellow %.2f\0", Hstr+Hint) ;
	  else if (nt == Cmax-1) sprintf(text, "s Above %.2f\0", Hstr+Hint*(float)nt) ;
	  else sprintf(text, "s %.2f ~ %.2f\0", Hstr+Hint*(float)nt,
	    Hstr+Hint*(float)(nt+1)) ;
	  }
	else {
	  if (nt == 0) sprintf(text, "s Bellow %.1f\0", Hstr+Hint) ;
	  else if (nt == Cmax-1) sprintf(text, "s Above %.1f\0", Hstr+Hint*(float)nt) ;
	  else sprintf(text, "s %.1f ~ %.1f\0", Hstr+Hint*(float)nt,
	    Hstr+Hint*(float)(nt+1)) ;
	  }
	printf("rm %f %f\n", boxw*2.2, boxw*(float)nc+boxw*0.3);
	printf("Lw 1\n") ;
	printf("%s\n", text) ;
	}
/* -------------------------------------------------------------------------- */
int	gauss(x)
float	x ;
{
	if (x < 0.0) return((int)x - 1) ;
	return((int)x) ;
	}
