/*
*  Program name:             g_arcg  ver. 2.0
*
*  Program function:         JKM -> ARC/INFO Grid export file
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            July 19, 2007
*
*  Date of revision:
*
*  Arguments:                1. zscale (default = 1)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   ARC-INFO Grid export file ascii (to stdout)
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy ;
	long	n ;
	double	atof(), zscale=1.0 ;
	void	JKM_Read() ;

	if (argc > 1) {
	  if (!strcmp(argv[1], "-h")) {
	    fprintf(stderr, " USAGE: g_arcg {zscale}\n") ;
	    return(0) ;
	    }
	  zscale = atof(argv[1]) ;
	  if (zscale == 0.0) {
	    fprintf(stderr, " Argument zscale error.\n") ;
	    return(-1) ;
	    }
	  }

	JKM_Read(1) ;

	for (n = 0; n < Nx*Ny; n++) {
	  if (G[n] == Und) G[n] = -1.0 ;
	  else G[n] = G[n] * zscale ;
	  }

	printf("EXP  0 C:JKM_GRID_DATA.E00\n") ;
	printf("GRD  2\n") ;
	printf("%10d%10d 0%21.14E\n", Nx, Ny, Und) ;
	printf("%21.14E%21.14E\n", Dx, Dy) ;
	printf("%21.14E%21.14E\n", Xstr, Ystr) ;
	printf("%21.14E%21.14E\n", Xstr+Dx*(double)(Nx-1), Ystr+Dy*(double)(Ny-1)) ;
	for (iy = Ny-1; iy > -1; iy--) {
	  n = 0 ;
	  for (ix = 0; ix < Nx; ix++) {
	    n++ ;
	    printf("%14.7E", G[ix*Ny+iy]) ;
	    if (n%5 == 0) printf("\n") ;
	    }
	  if (n%5 != 0) printf("\n") ;
	  }
	printf("EOG\n") ;
	printf("EOS\n") ;
	free(G) ;
	return(0) ;
	}
