/*
*  Program name:             g_cal  ver. 2.0
*
*  Program function:         calculation of two JKM files
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Nov 13, 1993
*
*  Date of revision:         June 4, 1995 add option by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. calculation type ( ! ^ 2 + - * / x )
*                            2. v
*
*   calculation type
*      !  (Out = -IN)
*      ^  (Out = In**2)
*      2  (Out = In)
*      +  (Out = In + In2 ... + v)
*      -  (Out = In - In2 ... - v)
*      *  (Out = In * In2 ... * v)
*      /  (Out = In / In2 ... / v)
*      x  (Out = In (In2 != Und2))
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#include "jkm_lib.c"

long	Nx2, Ny2 ;
double	Xstr2, Ystr2, Dx2, Dy2 ;
float	Und2, gmin2, gmax2, G2 ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	long	n, ix, iy, jx, jy ;
	char	c, t ;
	double	v=0.0, v2=0.0, v3=0.0, atof(), pow() ;
	void	JKM_Read() ;
/* -------------------------------------------------------------------------- */
	if (argv == 0 || (argc > 1 && strcmp(argv[1], "-h") == 0)) {
	  fprintf(stderr, " USAGE: g_cal [! ^ 2 + - * / x] {value}\n") ;
	  return(0) ;
	  }
	if (argc > 1) c  = argv[1][0] ;
	if (argc > 2) v  = atof(argv[2]) ; else v  = 0.0 ;
	if (argc > 3) v2 = atof(argv[3]) ; else v2 = 0.0 ;
	fprintf(stderr, " calculation is %c, value is %f %f %f\n", c,v,v2,v3) ;
/* -------------------------------------------------------------------------- */
	JKM_Read(1) ;
	fprintf(stderr, " First grid read end.\n") ;
/* -------------------------------------------------------------------------- */
	if (c == '!') {
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = -G[n] ;
	  JKM_Write() ;
	  return(0) ;
	  }
	else if (c == '^') {
	  if (v == 0.0)
	    for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = G[n] * G[n] ;
	  else {
	     for (n = 0; n < Nx*Ny; n++) {
	       if (G[n] != Und) {
	         if (G[n] > 0.0) 
	           G[n] =  pow((double)(G[n]/Gmax), (double)v) * Gmax ;
	         else
	           G[n] = -pow((double)(G[n]/Gmin), (double)v) * Gmin ;
	         }
	       }
	     }
	  JKM_Write() ;
	  return(0) ;
	  }
	else if (c == 'S') {
	  if (v2 == 0.0) v2 = Gmax ;
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) {
	    if (G[n] > 0.0) {
	      if (G[n] < v2) {
	        v3 = (v2 - G[n]) / v2 ;
	        v3 = pow(v3, 3.0) * v ;
	        G[n] = (v3 + 1.0) * G[n]  ;
	        }
	      }
	    }
	  JKM_Write() ;
	  return(0) ;
	  }
	else if (c == '+' && argc > 2) {
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = G[n] + v ;
	  }
	else if (c == '-' && argc > 2) {
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = G[n] - v ;
	  }
	else if (c == '*' && argc > 2) {
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = G[n] * v ;
	  }
	else if (c == '/' && argc > 2) {
	  if (v == 0.0) {
	    fprintf(stderr, " JKM is devided by 0.\n") ;
	    return(1) ;
	    }
	  for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) G[n] = G[n] / v ;
	  }
/* -------------------------------------------------------------------------- */
	while ( 1 ) {
	  while ((t = getchar()) != '\n') if (t == EOF) {
	    JKM_Write() ;
	    free(G) ;
	    return(0) ;
	    }
	  if (scanf("%d %d %lf %lf %lf %lf %f %f %f",
	    &Nx2,&Ny2,&Xstr2,&Ystr2,&Dx2,&Dy2,&Und2,&gmin2,&gmax2) != 9) {
	    fprintf(stderr, " Can't read second JKM.\n") ;
	    return(-1) ;
	    }
	  while ((t = getchar()) != '\n') ;
	  Dx2 = Dx2 / Dx ;
	  Dy2 = Dy2 / Dy ;
	  Xstr2 = (Xstr2 - Xstr) / Dx + 1.5 - Dx2 ;
	  Ystr2 = (Ystr2 - Ystr) / Dy + 1.5 - Dy2 ;
	  for (ix = 0; ix < Nx2; ix++) for (iy = 0; iy < Ny2; iy++) {
	    scanf("%f", &G2) ;
	    jx = (long)((double)(ix) * Dx2 + Xstr2) ;
	    jy = (long)((double)(iy) * Dy2 + Ystr2) ;
	    if (jx < 0 || jx >= Nx || jy < 0 || jy >= Ny) continue ;
	    n = jx * Ny + jy ;
	    if (c == '2') {
	      if (G2 != Und2) G[n] = G2 ;
	      continue ;
	      }
	    if (G2 == Und2) {
	      G[n] = Und ;
	      continue ;
	      }
	    if (G[n] == Und) continue ;
	         if (c == '-') G[n] = G[n] - G2 ;
	    else if (c == '+') G[n] = G[n] + G2 ;
	    else if (c == '*') G[n] = G[n] * G2 ;
	    else if (c == '/') {
	      if (G2 == 0.0) G[n] = 0.0 ;
	      else G[n] = G[n] / G2 ;
	      }
	    }
	  while ( (t = getchar()) != '\n' ) ;
	  }
/* -------------------------------------------------------------------------- */
	return(0) ;
	}
