/*
*  Program name:             g_change  ver. 2.0
*
*  Program function:         JKM data interactive change
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            June 4, 1991
*
*  Date of revision:         June 24, 1995 rewrite using C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1: JKM file name
*                            2: -v (No view option)
*
*  Input:                    JKM data (program open)
*
*  Output:                   JKM data (program open)
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))

char	Cmnt[100] ;
int	Nx, Ny ;
double	Xstr, Ystr, Dx, Dy ;
float	Und, Zmin, Zmax, *Z ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	float	zw, zo ;
	char	dsn[40], com, name[4] ;
	int	ix, iy, ixs, iys, ix1, iy1, ix2, iy2, view=1 ;
	double	v ;
	void	JKM_Read(), JKM_Write() ;

	if (argc > 1 && strcmp(argv[1], "-h") == 0) {
	  fprintf(stderr, " USAGE: g_change {JKM-file}\n") ;
	  return(0) ;
	  }
	if (argc < 2) {
	  printf(" key-in JKM file name to be changed.\n") ;
	  fgets(dsn, 40, stdin) ;
	  dsn[strlen(dsn)-1] = '\0' ;
	  }
	else {
	  strcpy(dsn, argv[1]) ;
	  if (argc > 2 && !strcmp(argv[2], "-v")) view = 0 ;
	  }
	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open JKM file '%s'.\n", dsn) ;
	  return(-1) ;
	  }
	JKM_Read(fp) ;
	fclose(fp) ;

	ixs = 0 ;
	iys = 0 ;
	while (1) {
	  if (view) {
		for (iy = min(Ny-1, iys+20); iy >= iys; iy--) {
	      printf("%5d", iy+1) ;
	      for (ix = ixs; ix < min(Nx, ixs+9); ix++)
	        printf("%8.2f", Z[ix*Ny+iy]) ;
	      printf("\n") ;
		  }
	    printf("     ") ;
	    for (ix = ixs; ix < min(Nx, ixs+9); ix++) printf("%8d", ix+1) ;
	    printf("\n") ;
	    printf(" Key-in Command (d, u, l, r, j, c, t, m, s, q, v)\n") ;
	    }
	  while ((com = getchar()) == '\n' || com == ' ') ;
	       if (com == 'd') iys = max(0, min(Ny-1, iys-10)) ;
	  else if (com == 'u') iys = max(0, min(Ny-1, iys+10)) ;
	  else if (com == 'l') ixs = max(0, min(Nx-1, ixs- 5)) ;
	  else if (com == 'r') ixs = max(0, min(Nx-1, ixs+ 5)) ;
	  else if (com == 'j') {
	    if (view) printf(" Key-in ixs, iys\n") ;
	    scanf("%d %d", &ixs, &iys) ;
	    ixs = max(0, min(Nx-1, ixs-1)) ;
	    iys = max(0, min(Ny-1, iys-1)) ;
	    }
	  else if (com == 'c') {
	    if (view) printf(" Key-in ix, iy, new-Z\n") ;
	    scanf("%d %d %f", &ix, &iy, &zw) ;
	    ix-- ;
	    iy-- ;
	    if (ix>=0 && ix<Nx && iy>=0 && iy<Ny) Z[ix*Ny+iy] = zw ;
	    }
	  else if (com == 't') {
	    if (view) printf(" Key-in ix1, iy1, ix2, iy2, old-H, new-H\n") ;
	    scanf("%d %d %d %d %f %f", &ix1, &iy1, &ix2, &iy2, &zo, &zw) ;
	    ix1 = max(0, min(Nx-1, ix1-1)) ;
	    iy1 = max(0, min(Ny-1, iy1-1)) ;
	    ix2 = max(0, min(Nx-1, ix2-1)) ;
	    iy2 = max(0, min(Ny-1, iy2-1)) ;
	    for (ix = ix1; ix <= ix2; ix++)
	      for (iy = iy1; iy <= iy2; iy++)
	        if (Z[ix*Ny+iy] == zo) Z[ix*Ny+iy] = zw ;
	    }
	  else if (com == 'm') {
	    if (view) printf(" Key-in ix1, iy1, ix2, iy2, new-H\n") ;
	    scanf("%d %d %d %d %f", &ix1, &iy1, &ix2, &iy2, &zw) ;
	    ix1 = max(0, min(Nx-1, ix1-1)) ;
	    iy1 = max(0, min(Ny-1, iy1-1)) ;
	    ix2 = max(0, min(Nx-1, ix2-1)) ;
	    iy2 = max(0, min(Ny-1, iy2-1)) ;
	    for (ix = ix1; ix <= ix2; ix++)
	      for (iy = iy1; iy <= iy2; iy++)
	        Z[ix*Ny+iy] = zw ;
	    }
	  else if (com == 's') {
	    if (view) printf(" Do you want to save to '%s' ? ", dsn) ;
	    while ((com = getchar()) == '\n' || com == ' ') ;
	    if (com == 'y' || com == 'Y') {
	      if ((fp = fopen(dsn, "w")) == NULL) {
	        fprintf(stderr, " Can't write JKM file '%s'.\n", dsn) ;
	        return(-1) ;
	        }
	      JKM_Write(fp) ;
	      fclose(fp) ;
	      free(Z) ;
	      return(0) ;
	      }
	    }
	  else if (com == 'q') {
	    if (view) printf(" Do you want to quit ? ") ;
	    while ((com = getchar()) == '\n' || com == ' ') ;
	    if (com == 'y' || com == 'Y') {
	      free(Z) ;
	      return(0) ;
	      }
	    }
	  else if (com == 'v') {
	    if (view) printf(" Key-in Xstr/Ystr/Dx/Dy value\n") ;
	    scanf("%4s %lf", name, &v) ;
	    if (!strcmp(name, "Xstr")) Xstr = v ;
	    if (!strcmp(name, "Ystr")) Ystr = v ;
	    if (!strcmp(name, "Dx")) Dx = v ;
	    if (!strcmp(name, "Dy")) Dy = v ;
	    }
	  else if (view) printf(" Invalid command\n") ;
	  }
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
/* --- JKM data read/write programs ----------------------------------------- */
/* -------------------------------------------------------------------------- */
void	JKM_Read(fp)
FILE	*fp ;
{
	long	n ;

	if (fgets(Cmnt, 100, fp) == NULL) {
	  fprintf(stderr, " Can't read comment.\n") ;
	  exit(-1) ;
	  }
	Cmnt[strlen(Cmnt)] = '\0' ;
	if ((fscanf(fp, "%d %d %lf %lf %lf %lf %f %f %f",
	  &Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Zmin, &Zmax)) != 9) {
	  fprintf(stderr, " Can't read header.\n") ;
	  exit(-1) ;
	  }
	if ((Z = (float *)calloc(Nx*Ny, sizeof(*Z))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Z.\n") ;
	  exit(-1) ;
	  }
	for (n = 0; n < Nx*Ny; n++) if ((fscanf(fp, "%f", &Z[n])) != 1) {
	  fprintf(stderr, " Can't read Z.\n") ;
	  exit(-1) ;
	  }
	while (fgetc(fp) != '\n') ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Write(fp)
FILE	*fp ;
{
	long	n ;
	float	zw, zw1, zw2, undef ;
	char	form[6] ;

	Zmin =  99999999.9 ;
	Zmax = -99999999.9 ;
	for (n = 0; n < Nx*Ny; n++) if (Z[n] != Und) {
	  if (Zmin > Z[n]) Zmin = Z[n] ;
	  if (Zmax < Z[n]) Zmax = Z[n] ;
	  }
	zw1 = Zmin ;
	if (Zmin < 0.0 && Zmin > -10.0) zw1 = 99.0 ;
	if (zw1 < 0.0) zw1 = -zw1 * 10.0 ;
	zw2 = Zmax ;
	if (Zmax < 0.0 && Zmax > -10.0) zw2 = 99.0 ;
	if (zw2 < 0.0) zw2 = -zw2 * 10.0 ;
	zw = max(zw1, zw2) ;

	if (zw < 10.0) {
	  strcpy(form, "%8.5f") ;
	  undef = 9.99999 ;
	  }
	else if (zw < 100.0) {
	  strcpy(form, "%8.4f") ;
	  undef = 99.9999 ;
	  }
	else if (zw < 1000.0) {
	  strcpy(form, "%8.3f") ;
	  undef = 999.999 ;
	  }
	else if (zw < 10000.0) {
	  strcpy(form, "%8.2f") ;
	  undef = 9999.99 ;
	  }
	else if (zw < 100000.0) {
	  strcpy(form, "%8.1f") ;
	  undef = 99999.9 ;
	  }
	else if (zw < 1000000.0) {
	  strcpy(form, "%8.0f") ;
	  undef = 999999. ;
	  }
	else {
	  fprintf(stderr, " Data > 1000000.0 ; stop.\n") ;
	  exit(-1) ;
	  }
	fprintf(fp, "%s", Cmnt) ;
	fprintf(fp, "%5d%5d%12.2lf%12.2lf%11.3lf%11.3lf",
	  Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	fprintf(fp, form, undef) ;
	fprintf(fp, form, Zmin) ;
	fprintf(fp, form, Zmax) ;
	fprintf(fp, "\n") ;
	for (n = 0; n < Nx*Ny; n++) {
	  if (Z[n] == Und) fprintf(fp, form, undef) ;
	  else fprintf(fp, form, Z[n]) ;
	  if (n%10 == 9) fputc('\n', fp) ;
	  }
	if (n%10 != 0) fputc('\n', fp) ;
	}
