/*
*  Program name:             g_der  ver. 2.0
*
*  Program function:         calculation of 1st or 2nd derivative of grid
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 18, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. Derivative order
*                            2. Scale
*
*  Input:                    JKM format ( from standard input )
*
*  Output:                   JKM format ( to standard output )
*/

#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>

#include "jkm_lib.c"

#define		min(a, b)	(((a) < (b)) ? (a) : (b))
#define		max(a, b)	(((a) > (b)) ? (a) : (b))

float	Und2, *Gin ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ider, n, x, y ;
	double	xd, yd, sqrt(), g1, g2, g3, g4, g5, scale, atof() ;
	void	JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc > 1) ider  = atoi(argv[1]) ; else ider  = 1   ;
	if (argc > 2) scale = atof(argv[2]) ; else scale = 1.0 ;
/* -------------------------------------------------------------------------- */
	if ((fgets(Cmnt, 100, stdin)) == NULL) {
	  fprintf(stderr, " Can't read JKM comment.\n") ;
	  return(0) ;
	  }
	Cmnt[strlen(Cmnt)] = '\0' ;
	if ((scanf("%d %d %lf %lf %lf %lf %f %f %f",
	  &Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Gmin, &Gmax)) != 9) {
	  fprintf(stderr, " Can't read JKM header.\n") ;
	  return(0) ;
	  }
	if ((Gin = (float *)calloc(3*Ny, sizeof(*Gin))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Gin.\n") ;
	  return(0) ;
	  }
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
	  fprintf(stderr, " Can't alloc memory G.\n") ;
	  return(0) ;
	  }
	Und2 = 9.0e29 ;

	for (x = 0; x < Nx; x++) {
	  for (y = 0; y < Ny ; y++) {
	    Gin[y]    = Gin[y+Ny] ;
	    Gin[Ny+y] = Gin[y+Ny+Ny] ;
	    if (scanf("%f", &Gin[Ny+Ny+y]) != 1) {
	      fprintf(stderr, " Can't read Grid.\n") ;
	      return(0) ;
	      }
	    G[x*Ny+y] = Und2 ;
	    }
	  if (ider==1 && x>0) {
	    for (y = 0; y < Ny-1; y++) {
	      n = (x-1) * Ny + y ;
	      g1 = Gin[Ny+y] ;
	      g2 = Gin[Ny+Ny+y] ;
	      g3 = Gin[Ny+y+1] ;
	      if (g1==Und||g2==Und||g3==Und) G[n] = Und2 ;
	      else {
	        xd = (g2 - g1) / Dx ;
	        yd = (g3 - g1) / Dy ;
	        G[n] = sqrt(xd * xd + yd * yd) * scale ;
	        }
	      }
	    }
	  else if (ider==2 && x>1) {
	    for (y = 1; y < Ny-1; y++) {
	      n = (x-1) * Ny + y ;
	      g1 = Gin[y+Ny] ;
	      g2 = Gin[y+Ny+Ny] ;
	      g3 = Gin[y+Ny+1] ;
	      g4 = Gin[y] ;
	      g5 = Gin[y+Ny-1] ;
	      if (g1==Und||g2==Und||g3==Und||g4==Und||g5==Und) G[n] = Und2 ;
	      else {
	        xd = (g1 * 2.0 - g2 - g4) ;
	        yd = (g1 * 2.0 - g3 - g5) ;
	        G[n] = xd / (Dx * Dx) + yd / (Dy * Dy) ;
	        G[n] = G[n] * scale ;
	        }
	      }
	    }
	  }

	Und = Und2 ;
	JKM_Write() ;
	free(G) ;
	free(Gin) ;
	return(0) ;
	}
