/*
*  Program name:             g_inter ver. 2.0
*
*  Program function:         JKM grid-data interpolationg
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            July 21, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. x-ext (default:2)
*                            2. y-ext (default:2)
*                            3. number-of-iteration (default:1000)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))
char	Cmnt[100] ;
int	Nx, Ny ;
double	Xstr, Ystr, Dx, Dy ;
float	Und, Gmin, Gmax, *G, *G2 ;
int	Ex, Ey, Iter ;
short	*I ;
/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, ix, iy, jx, jy ;
	long	n, m ;
	double	a, b, fabs() ;
	void	JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc > 1 && !strcmp(argv[1], "-h")) {
		fprintf(stderr, " USAGE: g_inter {x-ext y-ext iter}\n") ;
		return(0) ;
		}
	if (argc > 1) Ex   = atoi(argv[1]) ; else Ex   =    2 ;
	if (argc > 2) Ey   = atoi(argv[2]) ; else Ey   =    2 ;
	if (argc > 3) Iter = atoi(argv[3]) ; else Iter = 1000 ;
/* -------------------------------------------------------------------------- */
	if (fgets(Cmnt, 100, stdin) == NULL) {
		fprintf(stderr, " Can't read JKM comment.\n") ;
		return(-1) ;
		}
	Cmnt[strlen(Cmnt)] = '\0' ;
	if ((scanf("%d %d %lf %lf %lf %lf %f %f %f",
		&Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Gmin, &Gmax)) != 9) {
		fprintf(stderr, " Can't read JKM header.\n") ;
		return(-1) ;
		}
	jx = (Nx - 1) * Ex + 1 ;
	jy = (Ny - 1) * Ey + 1 ;
	if ((G  = (float *)calloc(jx*jy, sizeof(*G ))) == NULL ||
	    (G2 = (float *)calloc(jx*jy, sizeof(*G2))) == NULL ||
	    (I  = (short *)calloc(jx*jy, sizeof(*I ))) == NULL) {
		fprintf(stderr, " Can't alloc memory G/G2/I.\n") ;
		return(-1) ;
		}
	for (n = 0; n < jx*jy; n++) {
		G[n] = Gmin ;
		I[n] = 0 ;
		}
	for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
		n = (ix * Ex) * jy + iy * Ey ;
		if ((scanf("%f", &G[n])) != 1) {
		  fprintf(stderr, " Can't read G.\n") ;
		  return(-1) ;
		  }
		if (G[n] != Und) I[n] = 1 ;
		}
	Nx = jx ;
	Ny = jy ;
	Dx = Dx / (double)Ex ;
	Dy = Dy / (double)Ey ;
	fprintf(stderr, " Grid read end\n") ;
/* -------------------------------------------------------------------------- */
	for (i = 0; i < Iter; i++) {
	b = 0.0 ;
	for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
		n = ix * Ny + iy ;
		if (I[n] == 1) {
			G2[n] = G[n] ;
			continue ;
			}
		m = 0 ;
		a = 0.0 ;
		if (ix > 0 && ix < Nx-1) {
			m = m + 2 ;
			a = a + G[n-Ny] + G[n+Ny] ;
			}
		if (iy > 0 && iy < Ny-1) {
			m = m + 2 ;
			a = a + G[n-1] + G[n+1] ;
			}
		if (m == 4) {
			m = m + 4 ;
			a = a + G[n-Ny-1] + G[n+Ny+1] + G[n+Ny-1] + G[n-Ny+1] ;
			}
		G2[n] = a / (float)m ;
		b = max(b, fabs(G2[n]-G[n])) ;
		}
	for (n = 0; n < Nx*Ny; n++) G[n] = G2[n] ;
	fprintf(stderr, " Iter = %d %lf\n", i, b) ;
	if (b < (Gmax-Gmin)*0.00001) break ;
	}
	JKM_Write() ;
	free(G) ;
	free(G2) ;
	free(I) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Write()
{
	long	n ;
	float	gw, gw1, gw2, undef ;
	char	form[6] ;

	Gmin =  99999999.9 ;
	Gmax = -99999999.9 ;
	for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) {
		if (Gmin > G[n]) Gmin = G[n] ;
		if (Gmax < G[n]) Gmax = G[n] ;
		}
	gw1 = Gmin ;
	if (Gmin < 0.0 && Gmin > -10.0) gw1 = 99.0 ;
	if (gw1 < 0.0) gw1 = -gw1 * 10.0 ;
	gw2 = Gmax ;
	if (Gmax < 0.0 && Gmax > -10.0) gw2 = 99.0 ;
	if (gw2 < 0.0) gw2 = -gw2 * 10.0 ;
	gw = max(gw1, gw2) ;

	if (gw < 10.0) {
		strcpy(form, "%8.5f") ;
		undef = 9.99999 ;
		}
	else if (gw < 100.0) {
		strcpy(form, "%8.4f") ;
		undef = 99.9999 ;
		}
	else if (gw < 1000.0) {
		strcpy(form, "%8.3f") ;
		undef = 999.999 ;
		}
	else if (gw < 10000.0) {
		strcpy(form, "%8.2f") ;
		undef = 9999.99 ;
		}
	else if (gw < 100000.0) {
		strcpy(form, "%8.1f") ;
		undef = 99999.9 ;
		}
	else if (gw < 1000000.0) {
		strcpy(form, "%8.0f") ;
		undef = 999999. ;
		}
	else {
		fprintf(stderr, " Data > 1000000.0 ; stop.\n") ;
		exit(-1) ;
		}
	printf("%s\n", Cmnt) ;
	printf("%5d%5d%12.2lf%12.2lf%11.3lf%11.3lf",
		Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	printf(form, undef) ;
	printf(form, Gmin) ;
	printf(form, Gmax) ;
	printf("\n") ;
	for (n = 0; n < Nx*Ny; n++) {
		if (G[n] == Und) printf(form, undef) ;
		else printf(form, G[n]) ;
		if (n%10 == 9) putchar('\n') ;
		}
	if (n%10 != 0) putchar('\n') ;
	}
