/*
*  Program name:             g_pgm  ver. 2.0
*
*  Program function:         JKM to PGM
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jan 13, 1993
*
*  Date of revision:         June 28, 1995 Change output format by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f: JKM file name      (JKM File)
*                            -i: Grid read interval (default = 1)
*                            -n: Normalize mode     (default = none)
*
*  Input:                    JKM file (program open)
*
*  Output:                   PGM file (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <malloc.h>

char	*Frame ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	xmin, ymin, xwid, ywid, und, hmin, hmax, wh ;
	int	nx, ny, nx2, ny2, i, j, ix, iy, iix, iiy, inter, norm, exp ;
	long	n ;
	int	atoi() ;
	char    grid[80] ;
	FILE	*fp ;
/* -------------------------------------------------------------------------- */
	inter = 1 ;
	norm  = 0 ;
	exp = 1 ;
	grid[0] = ' ' ;
	for (i = 1; i < argc; i++) {
	       if (!strcmp(argv[i], "-f")) strcpy(grid, argv[++i]) ;
	  else if (!strcmp(argv[i], "-i")) inter = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-e")) exp = atoi(argv[++i]) ;
	  else if (!strcmp(argv[i], "-n")) norm = 1 ;
	  else if (!strcmp(argv[i], "-h")) {
	    fprintf(stderr, " USAGE: g_pgm {-f $ -i # -e # -n}\n") ;
	    return(0) ;
	    }
	  }
	if (grid[0] == '-' || grid[0] == ' ') {
	  fprintf(stderr, " Read Grid-data from standard input.\n") ;
	  fp = stdin ;
	  }
	else if ((fp = fopen(grid, "r")) == NULL) {
	  fprintf(stderr, " Can't open JKM file '%s'.\n", grid) ;
	  return(-1) ;
	  }

	while ((getc(fp)) != '\n') ;
	fscanf(fp, "%d %d %lf %lf %lf %lf %lf %lf %lf",
		&nx, &ny, &xmin, &ymin, &xwid, &ywid, &und, &hmin, &hmax) ;
	nx2 = (nx + inter - 2) / inter + 1 ;
	ny2 = (ny + inter - 2) / inter + 1 ;
	if ((Frame = (char *)calloc(nx2*ny2, sizeof(*Frame))) == NULL) {
	  fprintf(stderr, " Can't alloc memory for Frame.\n") ;
	  return(-1) ;
	  }
	for (ix = 0; ix < nx; ix++) {
	  iix = (ix + inter - 1) / inter ;
	  for (iy = 0; iy < ny; iy++) {
	    iiy = (iy + inter - 1) / inter ;
	    n = iix * ny2 + iiy ;
	    if (norm == 1) {
	      fscanf(fp, "%lf", &wh) ;
	      Frame[n] = (int)((wh - hmin) / (hmax - hmin) * 255.0) ;
	      }
	    else {
	      fscanf(fp, "%f", &wh) ;
	      Frame[n] = (long)(wh) % 1000 ;
	      }
	    }
	  }
	if (grid[0] != '-' || grid[0] != ' ') fclose(fp) ;
	fprintf(stderr, " Grid read end.\n") ;

	nx = nx2 ;
	ny = ny2 ;
	printf("P5\n") ;
	printf("%d %d\n", (nx-1)*exp+1, (ny-1)*exp+1) ;
	printf("255\n") ;
	for (iy = ny-1; iy > -1; iy--)
	  for (i = 0; i < exp; i++)
	    for (ix = 0; ix < nx; ix++)
	      for (j = 0; j < exp; j++)
	        putc(Frame[ix*ny+iy], stdout) ;
	free(Frame) ;
	return(0) ;
	}
