/*
*  Program name:             g_point  ver. 2.0
*
*  Program function:         JKM to JKP
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            July 31, 1990
*
*  Date of revision:         June 18, 1995  rewrite using C by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. jx (Sampling interval X)
*                            2. jy (Sampling interval Y)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKP format (to stdout)
*/

#include <stdio.h>
#include <string.h>

int	Nx, Ny ;
double	Xstr, Ystr, Dx, Dy ;
float	Und, Zmin, Zmax, Z ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy, jx, jy, atoi() ;
	long	n ;
	char	c ;
	double	x, y ;

	if (argc > 1 && !strcmp(argv[1], "-h")) {
		fprintf(stderr, " USAGE: g_point {X-interval Y-interval}\n") ;
		return(0) ;
		}
	if (argc > 1) jx = atoi(argv[1]) ; else jx = 1 ;
	if (argc > 2) jy = atoi(argv[2]) ; else jy = 1 ;
	while ((c = getchar()) != '\n') if (c == EOF) {
		fprintf(stderr, " Can't read JKM comment.\n") ;
		return(1) ;
		}
	if (scanf("%d %d %lf %lf %lf %lf %f %f %f",
		&Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Zmin, &Zmax) != 9) {
		fprintf(stderr, " Can't read JKM header.\n") ;
		return(1) ;
		}
	n = 0 ;
	for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
		if (scanf("%f", &Z) != 1) {
			fprintf(stderr, " Can't read JKM data.\n") ;
			return(1) ;
			}
		if (ix%jx != 0 || iy%jy != 0 || Z == Und) continue ;
		n++ ;
		x = Xstr + (double)ix * Dx ;
		y = Ystr + (double)iy * Dy ;
		printf("%6d%12.2lf%12.2lf%9.2f\n", n, x, y, Z) ;
		}
	return(0) ;
	}
