/*
*  Program name:             g_ppm  ver. 2.0
*
*  Program function:         JKM ot PPM converter
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 15, 1990
*
*  Date of revision:         June 10, 1995 Change for PPM by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -lf $  (color table file name)
*                            -ls $  (color table start point)
*                            -cs #  (start value)
*                            -ce #  (end value)
*                            -ci #  (interval value)
*                            -fix   (1st item of col_tbl shows ID)
*                            -und # (color for undefine value)
*                            -nolim (nolimit value)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   PPM format (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>

#define	MAXC	100000

#include "jkm_lib.c"

int	Nh, Lstr, Fix, Nolim ;
float	Cstr, Cend, Cint, Val[MAXC] ;
unsigned short	Color[MAXC][3] ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy ;
	long	i, n, cund ;
	double	atof() ;
	unsigned short	r, g, b ;
	void	Color_Table(), JKM_Read() ;
/* -------------------------------------------------------------------------- */
	Lstr = 1 ;
	Nolim = 0 ;
	Fix = 0 ;
	Cstr = 0.0 ;
	Cend = 0.0 ;
	Cint = 0.0 ;
	cund = 255255255 ;
	Nh = 0 ;
	n = 0 ;
	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: g_ppm -lf $ {- ls # -cs # -ce # ") ;
	    fprintf(stderr, "-ci # -fix -und #}\n") ;
	    return(0) ;
	    }
	  else if (strcmp(argv[i], "-lf")  == 0) n = ++i ;
	  else if (strcmp(argv[i], "-ls")  == 0) Lstr = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-cs")  == 0) Cstr = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ce")  == 0) Cend = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-ci")  == 0) Cint = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-und") == 0) cund = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-fix") == 0) Fix  = 1 ;
	  else if (strcmp(argv[i], "-nolim") == 0) Nolim  = 1 ;
	  }
	if (n == 0) {
	  fprintf(stderr, " Color-table-file (-lf $) is neccesary.\n") ;
	  return(-1) ;
	  }
	Color_Table(argv[n]) ;

	r = cund / 1000000 ;
	g = (cund / 1000) % 1000 ;
	b = cund % 1000 ;

	JKM_Read(1) ;
/* -------------------------------------------------------------------------- */
	printf("P6\n") ;
	printf("%d %d\n", Nx, Ny) ;
	printf("255\n") ;
	for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
	  n = ix * Ny + iy ;
	  if (G[n] == Und)
	    printf("%c%c%c", r, g, b) ;
	  else if (Fix == 1) {
	    i = (long)(G[n] + 0.5) ;
	    if (i < 0 || i >= MAXC) printf("%c%c%c", r, g, b) ;
	    else printf("%c%c%c", Color[i][0], Color[i][1], Color[i][2]) ;
	    }
/*	  else if (G[n] >= Val[Nh-1])
	    printf("%c%c%c", Color[Nh-1][0], Color[Nh-1][1], Color[Nh-1][2]) ;*/
	  else if (Cint != 0.0 && G[n] < Cstr) {
	    if (Nolim == 1) 
	      printf("%c%c%c", Color[0][0], Color[0][1], Color[0][2]) ;
	    else
	      printf("%c%c%c", r, g, b) ;
	    }
	  else if (Cint != 0.0 && G[n] >= Cend) {
	    if (Nolim == 1) 
	      printf("%c%c%c", Color[Nh-1][0], Color[Nh-1][1], Color[Nh-1][2]) ;
	    else
	      printf("%c%c%c", r, g, b) ;
	    }
	  else {
	    i = 0 ;
	    for (i = 0; i < Nh; i++) {
	      if (G[n] < Val[i]) {
	        break ;
	        }
	      }
	    printf("%c%c%c", Color[i][0], Color[i][1], Color[i][2]) ;
	    }
	  }
	free(G) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Color_Table(dsn)
char	*dsn ;
{
	FILE	*fp ;
	int	i, r, g, b ;

	if ((fp = fopen(dsn, "r")) == NULL) {
	  fprintf(stderr, " Can't open color table file '%s'.\n", dsn) ;
	  exit(-1) ;
	  }
	if (Lstr > 1) for (i = 1; i < Lstr; i++) fgets(Cmnt, 100, fp) ;
	for (Nh = 0; Nh < MAXC; Nh++) {
	  if (fgets(Cmnt, 100, fp) == NULL ||
	      sscanf(Cmnt, "%f %d %d %d", &Val[Nh], &r, &g, &b) != 4) break ;
	  if (Fix == 1) {
	    i = (int)(Val[Nh] + 0.5) ;
	    if (i >= 0 && i < MAXC) {
	      Color[i][0] = r ;
	      Color[i][1] = g ;
	      Color[i][2] = b ;
	      }
	    }
	  else if (Cint == 0.0) {
	    Color[Nh][0] = r ;
	    Color[Nh][1] = g ;
	    Color[Nh][2] = b ;
	    }
	  else {
	    Val[Nh]      = Cstr + (float)(Nh + 1) * Cint ;
	    Color[Nh][0] = r ;
	    Color[Nh][1] = g ;
	    Color[Nh][2] = b ;
	    }
	  }
	fclose(fp) ;
	}
