/*
*  Program name:             g_prof  ver. 2.0
*
*  Program function:         Profile data output from JKM data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         June 22, 1995  rewrite using C by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -x   (down to up)
*                            -y   (left to right)
*                            -xy  (lower-left to upper-right)
*                            -yx  (upper-left to lower-right)
*                            -r   (reverse)
*                            -n # (colum/row number)
*                            -s # (value scale for Z)
*
*  Input:                    JKM file (from stdin)
*
*  Output:                   JKG data (to stdout)
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, ix, iy, jx, jy, ixy, kind=1, num=1, rev=0, atoi() ;
	float	scale=1.0 ;
	double	x, atof(), sqrt() ;
	char	head ;
	void	JKM_Read(), JKM_Write() ;

	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h") == 0) {
	    fprintf(stderr, " USAGE: g_prof {-x -y -xy -yx -r ") ;
	    fprintf(stderr, "-n # -s #}\n") ;
	    return(0) ;
	    }
	  else if (strcmp(argv[i], "-x")  == 0) kind = 1 ;
	  else if (strcmp(argv[i], "-y")  == 0) kind = 2 ;
	  else if (strcmp(argv[i], "-xy") == 0) kind = 3 ;
	  else if (strcmp(argv[i], "-yx") == 0) kind = 4 ;
	  else if (strcmp(argv[i], "-r")  == 0) rev = 1 ;
	  else if (strcmp(argv[i], "-n")  == 0) num = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-s")  == 0) scale = atof(argv[++i]) ;
	  }

	JKM_Read(1) ;
/*
	Nx = 5 ;
	Ny = 5 ;
	for (ix = 0; ix < 5; ix++) for (iy = 0; iy < 5; iy++)
	  G[ix*Ny+iy] = (ix + 1) * 10 + (iy + 1) ;
*/
	if (kind == 1) {
	  if (rev == 0) {
	    ix = num - 1 ; iy = 0 ;
	    jx = 0 ; jy = 1 ;
	    }
	  else {
	    ix = num - 1 ; iy = Ny - 1 ;
	    jx = 0 ; jy = -1 ;
	    }
	  }
	else if (kind == 2) {
	  if (rev == 0) {
	    ix = 0 ; iy = num - 1 ;
	    jx = 1 ; jy = 0 ;
	    }
	  else {
	    ix = Nx - 1 ; iy = num - 1 ;
	    jx = -1 ; jy = 0 ;
	    }
	  }
	else if (kind == 3) {
	  if (rev == 0) {
	    jx = 1 ; jy = 1 ;
	    if (num >= 0) {
	      ix = num - 1 ; iy = 0 ;
	      }
	    else {
	      ix = 0 ; iy = -num - 1 ;
	      }
	    }
	  else {
	    jx = -1 ; jy = -1 ;
	    if (num >= 0) {
	      ix = Nx - num ; iy = Ny - 1 ;
	      }
	    else {
	      ix = Nx - 1 ; iy = Ny + num ;
	      }
	    }
	  }
	else if (kind == 4) {
	  if (rev == 0) {
	    jx = 1 ; jy = -1 ;
	    if (num >= 0) {
	      ix = 0 ; iy = Ny - num ;
	      }
	    else {
	      ix = -num - 1 ; iy = Ny - 1 ;
	      }
	    }
	  else {
	    jx = -1 ; jy = 1 ;
	    if (num >= 0) {
	      ix = Nx - 1 ; iy = num - 1 ;
	      }
	    else {
	      ix = Nx + num ; iy = 0 ;
	      }
	    }
	  }

	head = 'm' ;
	x = 0.0 ;
	for (ixy = 0; ixy < max(Nx, Ny); ixy++) {
	  if (ix > -1 && iy > -1 && ix < Nx && iy < Ny && G[ix*Ny+iy] != Und) {
	    printf("%c %lf %f\n", head, x, G[ix*Ny+iy]*scale) ;
	    head = 'l' ;
	    }
	  x = x + sqrt((double)(jx*jx)*Dx*Dx + (double)(jy*jy)*Dy*Dy) ;
	  ix = ix + jx ;
	  iy = iy + jy ;
	  }
	free(G) ;
	return(0) ;
	}
