/*
*  Program name:             g_rotate  ver. 1.0
*
*  Program function:         JKM data rotate
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         Sep  4, 1991 by Y.Murata
*                            JUne 12, 1995 rewrite using C
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -r90   (default)
*                            -r180
*                            -r270
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, rotate ;
	void	JKM_Read(), JKM_Write_Rotate() ;

	JKM_Read(1) ;
	rotate = 90 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-r90")  == 0) rotate =  90 ;
	  else if (strcmp(argv[i], "-r180") == 0) rotate = 180 ;
	  else if (strcmp(argv[i], "-r270") == 0) rotate = 270 ;
	  }
	JKM_Write_Rotate(rotate) ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Write_Rotate(rotate)
int	rotate ;
{
	long	n, m, ix, iy ;
	float	gw, gw1, gw2, undef ;
	char	form[6] ;

	Gmin =  99999999.9 ;
	Gmax = -99999999.9 ;
	for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) {
		if (Gmin > G[n]) Gmin = G[n] ;
		if (Gmax < G[n]) Gmax = G[n] ;
		}
	gw1 = Gmin ;
	if (Gmin < 0.0 && Gmin > -10.0) gw1 = 99.0 ;
	if (gw1 < 0.0) gw1 = -gw1 * 10.0 ;
	gw2 = Gmax ;
	if (Gmax < 0.0 && Gmax > -10.0) gw2 = 99.0 ;
	if (gw2 < 0.0) gw2 = -gw2 * 10.0 ;
	gw = max(gw1, gw2) ;

	if (gw < 10.0) {
		strcpy(form, "%8.5f") ;
		undef = 9.99999 ;
		}
	else if (gw < 100.0) {
		strcpy(form, "%8.4f") ;
		undef = 99.9999 ;
		}
	else if (gw < 1000.0) {
		strcpy(form, "%8.3f") ;
		undef = 999.999 ;
		}
	else if (gw < 10000.0) {
		strcpy(form, "%8.2f") ;
		undef = 9999.99 ;
		}
	else if (gw < 100000.0) {
		strcpy(form, "%8.1f") ;
		undef = 99999.9 ;
		}
	else if (gw < 1000000.0) {
		strcpy(form, "%8.0f") ;
		undef = 999999. ;
		}
	else {
		fprintf(stderr, " Data (%f-%f) > 1000000.0 ; stop.\n", Gmin, Gmax) ;
		exit(-1) ;
		}
	printf("%s\n", Cmnt) ;
	if (rotate != 90 && rotate != 270)
		printf("%5d%5d%12.2lf%12.2lf%11.3lf%11.3lf",
			Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	else
		printf("%5d%5d%12.2lf%12.2lf%11.3lf%11.3lf",
			Ny, Nx, Xstr, Ystr, Dy, Dx) ;
	printf(form, undef) ;
	printf(form, Gmin) ;
	printf(form, Gmax) ;
	printf("\n") ;
	m = 0 ;
	if (rotate == 90) {
	  for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
	    n = ix * Ny + ix ;
	    if (G[n] == Und) printf(form, undef) ;
	    else printf(form, G[n]) ;
	    if (m%10 == 9) putchar('\n') ;
	    m++ ;
	    }
	  }
	else if (rotate == 180) {
	  for (ix = Nx-1; ix > -1; ix--) for (iy = Ny-1; iy > -1; iy--) {
	    n = ix * Ny + ix ;
	    if (G[n] == Und) printf(form, undef) ;
	    else printf(form, G[n]) ;
	    if (m%10 == 9) putchar('\n') ;
	    m++ ;
	    }
	  }
	else if (rotate == 270) {
	  for (iy = 0; iy < Ny; iy++) for (ix = Nx-1; ix > -1; ix--) {
	    n = ix * Ny + iy ;
	    if (G[n] == Und) printf(form, undef) ;
	    else printf(form, G[n]) ;
	    if (m%10 == 9) putchar('\n') ;
	    m++ ;
	    }
	  }
	if (m%10 != 0) putchar('\n') ;
	}
