/*
*  Program name:             g_sel  ver. 2.0
*
*  Program function:         JKM data selecting
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jul 31, 1990
*
*  Date of revision:         Sep 4, 1991 by Y. Murata
*                            Jun 12, 1995 rewrite using C
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -xs # : start x
*                            -ys # : start y
*                            -xe # : end x
*                            -ye # : end y
*                            -xi # : interval x
*                            -yi # : interval y
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   JKM format (to stdout)
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include "jkm_lib.c"

float	*G2 ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, ix, iy, jx, jy, nx, ny, ixs, iys, ixe, iye, ixi, iyi ;
	void	JKM_Read(), JKM_Write() ;

	JKM_Read(1) ;
	if ((G2 = (float *)calloc(Nx*Ny, sizeof(*G2))) == NULL) {
		fprintf(stderr, " Can't alloc memory G2.\n") ;
		return(-1) ;
		}
	for (i = 0; i < Nx*Ny; i++) G2[i] = G[i] ;
	free(G) ;

	ixs = 1 ;
	iys = 1 ;
	ixe = Nx ;
	iye = Ny ;
	ixi = 1 ;
	iyi = 1 ;
	for (i = 1; i < argc; i++) {
	       if (strcmp(argv[i], "-xs") == 0) ixs = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-ys") == 0) iys = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-xe") == 0) ixe = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-ye") == 0) iye = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-xi") == 0) ixi = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-yi") == 0) iyi = atoi(argv[++i]) ;
	  }
	if (ixs>ixe||iys>iye||ixi<1||iyi<1) {
	  fprintf(stderr, " Argument error.\n") ;
	  return(-1) ;
	  }

	nx = 0 ;
	for (ix = ixs-1; ix < ixe; ix = ix + ixi) nx++ ;
	ny = 0 ;
	for (iy = iys-1; iy < iye; iy = iy + iyi) ny++ ;

	if ((G = (float *)calloc(nx*ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		return(-1) ;
		}

	Gmin =  9.9e29 ;
	Gmax = -9.9e29 ;
	jx = -1 ;
	for (ix = ixs-1; ix < ixe; ix = ix + ixi) {
	  jx++ ;
	  jy = -1 ;
	  for (iy = iys-1; iy < iye; iy = iy + iyi) {
	    jy++ ;
	    if (ix>=0 && ix<Nx && iy>=0 && iy<Ny) {
	      G[jx*ny+jy] = G2[ix*Ny+iy] ;
	      if (Gmin > G2[jx*ny+jy]) Gmin = G2[jx*ny+jy] ;
	      if (Gmax < G2[jx*ny+jy]) Gmax = G2[jx*ny+jy] ;
	      }
	    else G[jx*ny+jy] = Und ;
	    }
	  }
	Nx = nx ;
	Ny = ny ;
	Xstr = Xstr + (double)(ixs - 1) * Dx ;
	Ystr = Ystr + (double)(iys - 1) * Dy ;
	Dx = Dx * (double)ixi ;
	Dy = Dy * (double)iyi ;
	JKM_Write() ;
	free(G) ;
	free(G2) ;
	return(0) ;
	}
