/*
*  Program name:             g_shd  ver. 2.0
*
*  Program function:         calculation of two mesh files
*                            shd = B + P * cos(s) + S * cos(t)
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            June 4, 1991
*
*  Date of revision:         June 11, 1995  Change to PGM by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -az # (illumination azimuth, north=0, default=315)
*                            -an # (illumination angle, ground=0, default=45)
*                            -he # (height emphasis, default=auto)
*                            -se # (shade emphasis, default=1.0)
*                            -P #  (P value, default=255)
*                            -S #  (S value, default=0)
*                            -K #  (K value, default=0)
*                            -B #  (B value, default=70)
*                            -M #  (minimum value, default=0)
*                            -col # (color table file)
*                            -jkm  (output JKM-format)
*                            -jkmi (output JKM-Int-format)
*                            -jkg  (output JKG-format)
*
*  Input:                    JKM format (from stdin)
*
*  Output:                   PGM/JKM/JKG file (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "jkm_lib.c"

#define	min(a, b)	(((a) < (b)) ? (a) : (b))
#define	max(a, b)	(((a) > (b)) ? (a) : (b))

double	Theta, Phi, Xt, Yt, Zt, Xh, Yh, Zh, B, P, S, K, SE ;
double	VC[1000] ;
int	Col, Ncol, RC[1000], GC[1000], BC[1000], Smin ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	i, x, y, undc, jkm, s, shade(), shade_v() ;
	int	r, g, b, r2, g2, b2 ;
	long	n, m, c ;
	float	zscale, z, z1, z2, z3, z4 ;
	double	t, atof(), fabs(), xd, yd, dx2, dy2 ;
	void	sunset(), JKM_Read(), JKM_Int_Write(), colset() ;
/* -------------------------------------------------------------------------- */
	Theta  = 315 ;
	Phi    = 45 ;
	zscale = 0.0 ;
	P      = 255.0 ;
	S      = 0.0 ;
	K      = 0.0 ;
	B      = 70.0 ;
	SE     = 1.0 ;
	undc   = 255 ;
	jkm    = 0 ;
	Smin   = 0 ;
	Col    = 0 ;
	for (i = 1; i < argc; i++) {
	  if (strcmp(argv[i], "-h")  == 0) {
	    fprintf(stderr, " USAGE: g_shd {-az # -an # -he # -P # -S # ") ;
	    fprintf(stderr, "-A # -B # -und # -jkm -jkmi}\n") ;
	    return(0) ;
	    }
	  else if (strcmp(argv[i], "-az")  == 0) Theta  = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-an")  == 0) Phi    = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-he")  == 0) zscale = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-se")  == 0) SE     = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-P")   == 0) P      = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-S")   == 0) S      = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-K")   == 0) K      = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-B")   == 0) B      = atof(argv[++i]) ;
	  else if (strcmp(argv[i], "-M")   == 0) Smin   = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-col") == 0) colset(argv[++i])        ;
	  else if (strcmp(argv[i], "-und") == 0) undc   = atoi(argv[++i]) ;
	  else if (strcmp(argv[i], "-jkm") == 0) jkm    = 1 ;
	  else if (strcmp(argv[i], "-jkmi")== 0) jkm    = 2 ;
	  else if (strcmp(argv[i], "-jkg") == 0) jkm    = 3 ;
	  else if (strcmp(argv[i], "-ppm") == 0) jkm    = 4 ;
	  }
	sunset() ;
	fprintf(stderr, "jkm = %d\n", jkm) ;
/* -------------------------------------------------------------------------- */
	while (1) {

	JKM_Read(0) ;
	if (strcmp(Cmnt, "NULL") == 0) break ;
	if (zscale == 0.0) {
	  m = 0 ;
	  t = 0.0 ;
	  for (y = 0; y < Ny-1; y++) for (x = 0; x < Nx-1; x++) {
	    n = x * Ny + y ;
	    if (G[n]==Und||G[n+1]==Und||G[n+Ny]==Und||G[n+Ny+1]==Und) continue ;
	    m = m + 2 ;
	    t = t + fabs(G[n+1] - G[n]) + fabs(G[n+Ny] - G[n]) ;
	    }
	  t = t / (double)m ;
	  zscale = (Dx + Dy) / (t * 8.0) ;
	  }
	if (zscale != 1.0) {
	  for (n = 0; n < Nx*Ny; n++) G[n] = G[n] * zscale ;
	  Und = Und * zscale ;
	  }
	fprintf(stderr, "zscale = %f\n", zscale) ;

	dx2 = Dx / 2.0 ;
	dy2 = Dy / 2.0 ;
/* -------------------------------------------------------------------------- */
	if (jkm == 0) {
	  printf("P5\n") ;
	  printf("%d %d\n", Nx, Ny) ;
	  printf("255\n") ;
	  for (y = Ny-1; y > -1; y--) for (x = 0; x < Nx; x++) {
	    if (y == Ny-1 || x == Nx-1) putchar(undc) ;
	    else {
	      n = x * Ny + y ;
	      s = shade(G[n],G[n+1],G[n+Ny+1],G[n+Ny],undc) ;
	      putchar(s) ;
	      }
	    }
	  }
	else if (jkm == 1) {
	  n = 0 ;
	  for (x = 0; x < Nx; x++) for (y = 0; y < Ny; y++) {
	    if (y == Ny-1 || x == Nx-1) G[n] = undc ;
	    else G[n] = shade(G[n],G[n+1],G[n+Ny+1],G[n+Ny],undc) ;
	    n++ ;
	    }
	  Und = 256.0 ;
	  JKM_Write() ;
	  }
	else if (jkm == 2) {
	  if ((Ig = (long *)calloc(Nx*Ny, sizeof(*Ig))) == NULL) {
	    fprintf(stderr, " Can't alloc memory Ig.\n") ;
	    exit(-1) ;
	    }
	  n = 0 ;
	  for (x = 0; x < Nx; x++) for (y = 0; y < Ny; y++) {
	    if (y == Ny-1 || x == Nx-1) {
	      Ig[n] = undc * 1000000 + undc * 1000 + undc ;
	      }
	    else {
	      m = shade(G[n],G[n+1],G[n+Ny+1],G[n+Ny],undc) ;
	      Ig[n] = m * 1000000 + m * 1000 + m ;
	      }
	    n++ ;
	    }
	  Iund = 256256256 ;
	  JKM_Int_Write() ;
	  free(Ig) ;
	  }
	else if (jkm == 3) {
	  printf("Fp 0\n") ;
	  printf("Lw 1\n") ;
	  for (x = 0; x < Nx-1; x++) {
	    fprintf(stderr, " Line %d / %d start.\n", x+1, Nx-1) ;
	    for (y = 0; y < Ny-1; y++) {
	      xd = Xstr + (double)x * Dx ;
	      yd = Ystr + (double)y * Dy ;
	      n = x * Ny + y ;
	      z1 = G[n] ;
	      z2 = G[n+1] ;
	      z3 = G[n+Ny+1] ;
	      z4 = G[n+Ny] ;
	      if (z1==Und || z2==Und || z3==Und || z4==Und) continue ;
	      z = (z1 + z2 + z3 + z4) / 4.0 ;
	      if (Col == 0) {
	        r = 255 ;
	        g = 255 ;
	        b = 255 ;
	        }
	      else {
	        for (i = Ncol-1; i >= 0; i--) {
	          if (VC[i]*zscale > z) {
	            r = RC[i] ;
	            g = GC[i] ;
	            b = BC[i] ;
	            break ;
	            }
	          }
	        }
/* 1 */
	      s = shade_v((z-(z2+z1)/2.0)/dx2, (z2-z1)/Dy) ;
	      r2 = max(1, r * s / 255) ;
	      g2 = max(1, g * s / 255) ;
	      b2 = max(1, b * s / 255) ;
	      c = -r2 * 1000000 - g2 * 1000 - b2 ;
	      printf("Fb %d\n", c) ;
	      printf("Lc %d\n", c) ;
	      printf("n\n") ;
	      printf("m %lf %lf\n", xd,     yd) ;
	      printf("l %lf %lf\n", xd,     yd+Dy) ;
	      printf("l %lf %lf\n", xd+dx2, yd+dy2) ;
	      printf("l %lf %lf\n", xd,     yd) ;
	      printf("f\n") ;
/* 2 */
	      s = shade_v((z3-z2)/Dx, ((z3+z2)/2-z)/dy2) ;
	      r2 = max(1, r * s / 255) ;
	      g2 = max(1, g * s / 255) ;
	      b2 = max(1, b * s / 255) ;
	      c = -r2 * 1000000 - g2 * 1000 - b2 ;
	      printf("Fb %d\n", c) ;
	      printf("Lc %d\n", c) ;
	      printf("n\n") ;
	      printf("m %lf %lf\n", xd,     yd+Dy) ;
	      printf("l %lf %lf\n", xd+Dx,  yd+Dy) ;
	      printf("l %lf %lf\n", xd+dx2, yd+dy2) ;
	      printf("l %lf %lf\n", xd,     yd+Dy) ;
	      printf("f\n") ;
/* 3 */
	      s = shade_v(((z3+z4)/2.0-z)/dx2, (z3-z4)/Dy) ;
	      r2 = max(1, r * s / 255) ;
	      g2 = max(1, g * s / 255) ;
	      b2 = max(1, b * s / 255) ;
	      c = -r2 * 1000000 - g2 * 1000 - b2 ;
	      printf("Fb %d\n", c) ;
	      printf("Lc %d\n", c) ;
	      printf("n\n") ;
	      printf("m %lf %lf\n", xd+Dx,  yd+Dy) ;
	      printf("l %lf %lf\n", xd+Dx,  yd) ;
	      printf("l %lf %lf\n", xd+dx2, yd+dy2) ;
	      printf("l %lf %lf\n", xd+Dx,  yd+Dy) ;
	      printf("f\n") ;
/* 4 */
	      s = shade_v((z4-z1)/Dx, (z-(z4+z1)/2)/dy2) ;
	      r2 = max(1, r * s / 255) ;
	      g2 = max(1, g * s / 255) ;
	      b2 = max(1, b * s / 255) ;
	      c = -r2 * 1000000 - g2 * 1000 - b2 ;
	      printf("Fb %d\n", c) ;
	      printf("Lc %d\n", c) ;
	      printf("n\n") ;
	      printf("m %lf %lf\n", xd+Dx,  yd) ;
	      printf("l %lf %lf\n", xd,     yd) ;
	      printf("l %lf %lf\n", xd+dx2, yd+dy2) ;
	      printf("l %lf %lf\n", xd+Dx,  yd) ;
	      printf("f\n") ;
	      }
	    }
	  }
	else if (jkm == 4) {
	  printf("P6\n") ;
	  printf("%d %d\n", Nx, Ny) ;
	  printf("255\n") ;
	  for (y = Ny-1; y > -1; y--) for (x = 0; x < Nx; x++) {
	    if (y == Ny-1 || x == Nx-1) {
	      putchar(undc) ;
	      putchar(undc) ;
	      putchar(undc) ;
	      }
	    else {
	      n = x * Ny + y ;
	      s = shade(G[n],G[n+1],G[n+Ny+1],G[n+Ny],undc) ;
	      putchar(s) ;
	      putchar(s) ;
	      putchar(s) ;
	      }
	    }
	  }
	free(G) ;
	}
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
int	shade(z1, z2, z3, z4, undc)
float	z1, z2, z3, z4 ;
int	undc ;
{
	int	shade_v() ;
	float	xx, yy ;

	if (z1==Und || z2==Und || z3==Und || z4==Und) return(undc) ;
	xx = ((z3 + z4) / 2.0 - (z2 + z1) / 2.0) / Dx ;
	yy = ((z3 + z2) / 2.0 - (z4 + z1) / 2.0) / Dy ;
	return(shade_v(xx, yy)) ;
	}
/* -------------------------------------------------------------------------- */
int	shade_v(xx, yy)
float	xx, yy ;
{
	double	s, s2, sqrt() ;

	s = - (xx*Xt + yy*Yt - Zt) / sqrt(xx*xx + yy*yy + 1.0) ;
	if (s <= 0.0) return(max(Smin, (int)B)) ;
	if (SE != 1.0) s = max(0.0, (s - Zt) * SE + Zt) ;
	s = P * s ;
	if (K != 0.0) {
	  s2 = - (xx*Xh + yy*Yh - Zh) / sqrt(xx*xx + yy*yy + 1.0) ;
	  s2 = K / (K + 1.0 - s2) ;
	  s = s + S * s2 * s2 ;
	  }
	return(max(Smin, min(255, (int)(s+B)))) ;
	}
/* -------------------------------------------------------------------------- */
void	sunset()
{
	double	asin(), cos(), sin(), sqrt() ;

	Theta = Theta + 180.0 ;
	if (Theta > 360.0) Theta = Theta - 360.0 ;
	if (Theta <   0.0) Theta = Theta + 360.0 ;
	if (Theta > 360.0) Theta = Theta - 360.0 ;
	if (Theta <   0.0) Theta = Theta + 360.0 ;
	Theta = -Theta / 90.0 * asin(1.0) ;
	Phi   =   Phi  / 90.0 * asin(1.0) ;
	Xt =   sin(Theta) * cos(Phi) ;
	Yt = - cos(Theta) * cos(Phi) ;
	Zt =   sin(Phi) ;
	Xh =  Xt        / sqrt(Xt*Xt + Yt*Yt + (Zt+1.0)*(Zt+1.0)) ;
	Yh =  Yt        / sqrt(Xt*Xt + Yt*Yt + (Zt+1.0)*(Zt+1.0)) ;
	Zh = (Zt + 1.0) / sqrt(Xt*Xt + Yt*Yt + (Zt+1.0)*(Zt+1.0)) ;
	fprintf(stderr, "Zt = %f\n", Zt*255.0) ;
	}
/* -------------------------------------------------------------------------- */
void	colset(fn)
char	*fn ;
{
	FILE	*fp ;

	if ((fp=fopen(fn, "r")) == NULL) {
	  fprintf(stderr, "Color Table File not Found.") ;
	  exit(-1) ;
	  }

	fprintf(stderr, "%s\n", fn) ;
	Col = 1 ;
	Ncol = 0 ;
	while (fscanf(fp, "%lf %d %d %d\n", &VC[Ncol], &RC[Ncol], &GC[Ncol], &BC[Ncol]) == 4) Ncol++ ;
	fclose(fp) ;
	fprintf(stderr, "%d\n", Ncol) ;
	}
