/*
*  Program name:             g_undef  ver. 2.0
*
*  Program function:         Undefine area search using JKP data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            July 31, 1990
*
*  Date of revision:         Jun 4, 1991 By Y. Murata
*                            May 26, 1995  rewrite by C by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. JKP file name
*                            2. innerpol distance
*                            3. extrapol distance
*
*  Input:                    JKM data (from stdin)
*                            JKP data (program open)
*
*  Output:                   JKM data (to stdout)
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "jkm_lib.c"

short	*B ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ip, mz, ndstx1, ndsty1, ndstx2, ndsty2 ;
	int	m, n, nz, ix, iy, ixo, iyo, ix2, iy2, ixs, iys, ixe, iye ;
	int	lxs[4], lxe[4], lys[4], lye[4] ;
	double	dist1, dist2, atof() ;
	double	x, y, xs, ys, xe, ye ;
	int	bitest() ;
	void	JKM_Read(), JKM_Write(), iibset() ;
	FILE	*fp ;

	if (argc > 1 && strcmp(argv[1], "-h") == 0) {
		fprintf(stderr, " USAGE: g_undef JKP-file {") ;
		fprintf(stderr, "innerpol-distance extrapol-distance}\n") ;
		return(0) ;
		}
	dist1 = atof(argv[2]) ;
	if (argc > 2) dist2 = atof(argv[3]) ; else dist2 = 0.0 ;
	fprintf(stderr, " dist1, dist2 = %f %f\n", dist1, dist2) ;
	if (dist1<=0.0 || dist2<0.0) {
		fprintf(stderr, " Dist1 or dist2 error.\n") ;
		return(0) ;
		}
	JKM_Read(1) ;
	if ((B = (short *)calloc(Nx*Ny/4+1, sizeof(*B))) == NULL) {
		fprintf(stderr, " Can't alloc memory B.\n") ;
		return(0) ;
		}
	for (n=0; n<Nx*Ny/4+1; n++) B[n] = 0 ;
	ndstx1 = dist1 / Dx ;
	ndsty1 = dist1 / Dy ;
	ndstx2 = dist2 / Dx ;
	ndsty2 = dist2 / Dy ;
	lxs[0] = - ndstx1 ;
	lxe[0] = 0 ;
	lys[0] = - ndsty1 ;
	lye[0] = 0 ;
	lxs[1] = - ndstx1 ;
	lxe[1] = 0 ;
	lys[1] = 1 ;
	lye[1] = ndsty1 ;
	lxs[2] = 1 ;
	lxe[2] = ndstx1 ;
	lys[2] = 1 ;
	lye[2] = ndsty1 ;
	lxs[3] = 1 ;
	lxe[3] = ndstx1 ;
	lys[3] = - ndsty1 ;
	lye[3] = 0 ;
	xs = Xstr - max(dist1, dist2) ;
	ys = Ystr - max(dist1, dist2) ;
	xe = Xstr + (double)(Nx-1) * Dx + max(dist1, dist2) ;
	ye = Ystr + (double)(Ny-1) * Dy + max(dist1, dist2) ;
	if ((fp = fopen(argv[1], "r")) == NULL) {
		fprintf(stderr, " Can't open JKP.\n") ;
		return(-1) ;
		}
	m = 0 ;
	ixo = 0 ;
	iyo = 0 ;
	while (fscanf(fp, "%d %lf %lf", &n, &x, &y) == 3) {
		while (getc(fp) != '\n') ;
		m = m + 1 ;
		if (m%1000 == 0) fprintf(stderr, " %d\n", m) ;
		if (x<xs || x>xe || y<ys || y>ye) continue ;
		ix = (int)((x - Xstr) / Dx + 0.5) ;
		iy = (int)((y - Ystr) / Dy + 0.5) ;
		if (ix == ixo && iy == iyo) continue ;
		ixo = ix ;
		iyo = iy ;
		if (dist2 != 0.0) {
			ixs = max(   0, ix - ndstx2) ;
			ixe = min(Nx-1, ix + ndstx2) ;
			iys = max(   0, iy - ndsty2) ;
			iye = min(Ny-1, iy + ndsty2) ;
			for (ix2 = ixs; ix2 <= ixe; ix2++)
			for (iy2 = iys; iy2 <= iye; iy2++) {
				n = ix2 * Ny + iy2 ;
				nz = n / 4 ;
				mz = (n % 4) * 4 ;
				iibset(&B[nz], mz    ) ;
				iibset(&B[nz], mz + 1) ;
				iibset(&B[nz], mz + 2) ;
				iibset(&B[nz], mz + 3) ;
				}
			}
		for (ip = 0; ip < 4; ip++) {
			ixs = max(   0, ix + lxs[ip]) ;
			iys = max(   0, iy + lys[ip]) ;
			ixe = min(Nx-1, ix + lxe[ip]) ;
			iye = min(Ny-1, iy + lye[ip]) ;
			for (ix2 = ixs; ix2 <= ixe; ix2++)
			for (iy2 = iys; iy2 <= iye; iy2++) {
				n = ix2 * Ny + iy2 ;
				nz = n / 4 ;
				mz = (n % 4) * 4 ;
				iibset(&B[nz], mz + ip) ;
				}
			}
		}
	fclose(fp) ;
	fprintf(stderr, " Undef area search end.\n") ;
	for (n = 0; n < Nx*Ny; n++) {
		nz = n / 4 ;
		mz = (n % 4) * 4 ;
		for (ip = 0; ip < 4; ip++) {
			if (bitest(B[nz], mz + ip) != 0) continue ;
			G[n] = Und ;
			break ;
			}
		}
	free(B) ;
	JKM_Write() ;
	free(G) ;
	return(0) ;
	}

void	iibset(i, n)
short	*i ;
int	n ;
{
	*i = (1 << n) | (*i) ;
	}

int	bitest(i, n)
short	i ;
int	n ;
{
	return((1 << n) & i) ;
	}
