/*
*  Program name:             jkg_seg  ver. 1.0
*
*  Program function:         jkg segments cut for bird's eye view plot
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 1, 1995
*
*  Date of revision:         May 1, 2003 by Y. Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. Nx
*                            2. Ny
*                            3. Xstr
*                            4. Ystr
*                            5. Dx
*                            6. Dy
*
*
*  Input:                    JKG format (from stdin)
*
*  Output:                   segment data (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>

#define	MAXD	100000

#define	MAX(a, b)	(((a) > (b)) ? (a) : (b))
#define	MIN(a, b)	(((a) < (b)) ? (a) : (b))
#define	GAUSS(a)	(((a) >= 0.0) ? (int)(a) : ((int)(a)-1))

int	Nx, Ny ;
double	Xstr, Ystr, Xend, Yend, Dx, Dy ;
long	Color, Width, Nxy=0 ;
int	Px[MAXD], Py[MAXD] ;
double	X[MAXD], Y[MAXD] ;

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	atoi(), nrec ;
	double	atof(), xmin, ymin, xmax, ymax ;
	char	tx[500] ;
	void	SEG_Cut() ;

	if (argc < 7) {
		fprintf(stderr, " USAGE: jkg_seg nx ny xstr ystr dx dy\n") ;
		return(0) ;
		}
	Nx   = atoi(argv[1]) ;
	Ny   = atoi(argv[2]) ;
	Xstr = atof(argv[3]) ;
	Ystr = atof(argv[4]) ;
	Dx   = atof(argv[5]) ;
	Dy   = atof(argv[6]) ;
	Xend = Xstr + (double)(Nx-1) * Dx ;
	Yend = Ystr + (double)(Ny-1) * Dy ;
	nrec = 0 ;
	while (fgets(tx, 500, stdin) != NULL) {
		nrec++ ;
		if (tx[0] == 'm') {
/*			fprintf(stderr, " Nxy = %d %d\n", Nxy, nrec) ; */
			if (xmax < Xstr) Nxy = 0 ;
			if (xmin > Xend) Nxy = 0 ;
			if (ymax < Ystr) Nxy = 0 ;
			if (ymin > Yend) Nxy = 0 ;
			if (Nxy > 1) SEG_Cut() ;
			Nxy = 0 ;
			sscanf(tx, "%*c %lf %lf", &X[0], &Y[0]) ;
			xmin = X[Nxy] ;
			ymin = Y[Nxy] ;
			xmax = X[Nxy] ;
			ymax = Y[Nxy] ;
			Nxy = 1 ;
			}
		else if (tx[0] == 'l') {
			sscanf(tx, "%*c %lf %lf", &X[Nxy], &Y[Nxy]) ;
			xmin = MIN(xmin, X[Nxy]) ;
			ymin = MIN(ymin, Y[Nxy]) ;
			xmax = MAX(xmax, X[Nxy]) ;
			ymax = MAX(ymax, Y[Nxy]) ;
			Nxy++ ;
			if (Nxy >= MAXD) {
				fprintf(stderr, " 1st Nxy >= MAXD.\n") ;
				return(-1) ;
				}
			}
		else if (tx[0] == 'L') {
			if (xmax < Xstr) Nxy = 0 ;
			if (xmin > Xend) Nxy = 0 ;
			if (ymax < Ystr) Nxy = 0 ;
			if (ymin > Yend) Nxy = 0 ;
			if (Nxy > 1) SEG_Cut() ;
			Nxy = 0 ;
			if (tx[1] == 'c') sscanf(tx, "%*2c %ld", &Color) ;
			if (tx[1] == 'w') sscanf(tx, "%*2c %ld", &Width) ;
			}
		}
	if (Nxy > 1) SEG_Cut() ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	SEG_Cut()
{
	void	SEG_XY_Cut() ;
	int	i, j, is, ix, iy, ixo, iyo ;

	for (i = 0; i < Nxy; i++) {
		X[i] = (X[i] - Xstr) / Dx ;
		Y[i] = (Y[i] - Ystr) / Dy ;
		}
	SEG_XY_Cut(X, Y, Nx) ;
/*	printf("%5d\n", Nxy) ;
	for (i = 0; i < Nxy; i++) printf("%d %lf %lf\n", i, X[i], Y[i]) ; */
	SEG_XY_Cut(Y, X, Ny) ;
/*	printf("%5d\n", Nxy) ;
	for (i = 0; i < Nxy; i++) printf("%d %lf %lf\n", i, X[i], Y[i]) ; */
/*	printf("m %lf %lf\n", X[0], Y[0]) ;
	for (i = 1; i < Nxy; i++) printf("l %lf %lf\n", X[i], Y[i]) ; */
	is = 0 ;
	for (i = 1; i <= Nxy; i++) {
		ix = GAUSS((X[i] + X[i-1]) / 2.0) ;
		iy = GAUSS((Y[i] + Y[i-1]) / 2.0) ;
		if (i == 1) goto For_End ;
		if (i == Nxy || ix != ixo || iy != iyo) {
			if (ixo<0 || iyo<0 || ixo>Nx-1 || iyo>Ny-1) {
				is = i - 1 ;
				goto For_End ;
				}
			printf("%4d %4d %10d %2d %2d",
				ixo, iyo, Color, Width, i-is) ;
			for (j = is; j < i; j++)
				printf(" %4.2lf %4.2lf",
				X[j]-(double)ixo, Y[j]-(double)iyo) ;
			printf("\n") ;
			is = i - 1 ;
			}
For_End:	ixo = ix ;
		iyo = iy ;
		}
	}
/* -------------------------------------------------------------------------- */
void	SEG_XY_Cut(x, y, n)
double	*x, *y ;
int	n ;
{
	int	i, j, jd, k, p1, p2, p2x, abs() ;
	double	x1, x2, a, b ;

	p1 = MAX(-1, MIN(n, GAUSS(x[0]))) ;
	x1 = x[0] - (double)p1 ;
/*	printf("C 0 %lf %lf\n", x[0], y[0]) ; */
	for (i = 1; i < Nxy; i++) {
		p2 = MAX(-1, MIN(n, GAUSS(x[i]))) ;
		p2x = p2 ;
		x2 = x[i] - (double)p2 ;
/*		printf("%d %d %d %lf %lf\n", i, p1, p2, x1, x2) ; */
		if (p1 == p2) goto For_End ;
		if (p2 > p1) {
			if (x2 == 0.0) p2-- ;
			j = p2 - p1 ;
			p1++ ;
			jd = 1 ;
			}
		else {
			if (x1 == 0.0) p1-- ;
			j = p1 - p2 ;
			p2++ ;
			jd = -1 ;
			}
		if (j == 0) goto For_End ;
		Nxy = Nxy + j ;
		if (Nxy >= MAXD) {
			fprintf(stderr, " 2nd Nxy >= MAXD.\n") ;
			exit(-1) ;
			}
		for (k = Nxy-j-1; k >= i; k--) {
			x[k+j] = x[k] ;
			y[k+j] = y[k] ;
			}
		a = (y[i] - y[i-1]) / (x[i] - x[i-1]) ;
		b = y[i] - a * x[i] ;
		for (j = p1; j != p2+jd; j=j+jd) {
			x[i] = (double)j ;
			y[i] = a * x[i] + b ;
/*			printf("C %d %lf %lf added %d %d %d\n", i, x[i], y[i], p1, p2, j) ; */
			i++ ;
			}
For_End:	p1 = p2x ;
		x1 = x2 ;
/*		printf("C %d %lf %lf\n", i, x[i], y[i]) ; */
		}
/*	printf("%d\n", Nxy) ; */
/*	exit(0) ; */
	}
