/* -------------------------------------------------------------------------- */
/* --- JKM data read/write programs ----------------------------------------- */
/* -------------------------------------------------------------------------- */
#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))
char	Cmnt[100] ;
int	Nx, Ny ;
double	Xstr, Ystr, Dx, Dy ;
float	Und, Gmin, Gmax, *G ;
long	Iund, Igmin, Igmax, *Ig ;
/* -------------------------------------------------------------------------- */
void	JKM_Read(stat)
int	stat ;
{
	long	n ;
	int	c ;

	if (fgets(Cmnt, 100, stdin) == NULL) {
		strcpy(Cmnt, "NULL\0") ;
		if (stat == 1) {
			fprintf(stderr, " Can't read JKM comment.\n") ;
			exit(-1) ;
			}
		return ;
		}
	Cmnt[strlen(Cmnt)-1] = '\0' ;
	if ((scanf("%d %d %lf %lf %lf %lf %f %f %f",
		&Nx, &Ny, &Xstr, &Ystr, &Dx, &Dy, &Und, &Gmin, &Gmax)) != 9) {
		fprintf(stderr, " Can't read JKM header.\n") ;
		exit(-1) ;
		}
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		exit(-1) ;
		}
	for (n = 0; n < Nx*Ny; n++) {
		if ((scanf("%f", &G[n])) != 1) {
			fprintf(stderr, " Can't read G.\n") ;
			exit(-1) ;
			}
		}
	while ((c=getchar()) != '\n') if (c == EOF) break ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Write()
{
	long	n ;
	float	gw, gw1, gw2, undef ;
	char	form[6] ;

	Gmin =  99999999.9 ;
	Gmax = -99999999.9 ;
	for (n = 0; n < Nx*Ny; n++) if (G[n] != Und) {
		if (Gmin > G[n]) Gmin = G[n] ;
		if (Gmax < G[n]) Gmax = G[n] ;
		}
	gw1 = Gmin ;
	if (Gmin < 0.0 && Gmin > -10.0) gw1 = 99.0 ;
	if (gw1 < 0.0) gw1 = -gw1 * 10.0 ;
	gw2 = Gmax ;
	if (Gmax < 0.0 && Gmax > -10.0) gw2 = 99.0 ;
	if (gw2 < 0.0) gw2 = -gw2 * 10.0 ;
	gw = max(gw1, gw2) ;

	if (gw < 10.0) {
		strcpy(form, "%8.5f") ;
		undef = 9.99999 ;
		}
	else if (gw < 100.0) {
		strcpy(form, "%8.4f") ;
		undef = 99.9999 ;
		}
	else if (gw < 1000.0) {
		strcpy(form, "%8.3f") ;
		undef = 999.999 ;
		}
	else if (gw < 10000.0) {
		strcpy(form, "%8.2f") ;
		undef = 9999.99 ;
		}
	else if (gw < 100000.0) {
		strcpy(form, "%8.1f") ;
		undef = 99999.9 ;
		}
	else if (gw < 1000000.0) {
		strcpy(form, "%8.0f") ;
		undef = 999999. ;
		}
	else {
		fprintf(stderr, " Data (%f-%f) > 1000000.0 ; stop.\n", Gmin, Gmax) ;
		exit(-1) ;
		}
	printf("%s\n", Cmnt) ;
	printf("%d %d %lf %lf %lf %lf",
                        Nx, Ny, Xstr, Ystr, Dx, Dy) ;
/*	if (Xstr>=1000000.0||Xstr<=-100000.0||Ystr>=1000000.0||Ystr<=-100000.0)
		printf("%5d%5d%12.2lf%12.2lf%11.4lf%11.4lf",
			Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	else
		printf("%5d%5d%12.3lf%12.3lf%11.4lf%11.4lf",
			Nx, Ny, Xstr, Ystr, Dx, Dy) ; */
	printf(form, undef) ;
	printf(form, Gmin) ;
	printf(form, Gmax) ;
	printf("\n") ;
	for (n = 0; n < Nx*Ny; n++) {
		if (G[n] == Und) printf(form, undef) ;
		else printf(form, G[n]) ;
		if (n%10 == 9) putchar('\n') ;
		}
	if (n%10 != 0) putchar('\n') ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Int_Read()
{
	int	n ;

	if (fgets(Cmnt, 100, stdin) == NULL) {
		fprintf(stderr, " Can't read JKM comment.\n") ;
		exit(-1) ;
		}
	Cmnt[strlen(Cmnt)-1] = '\0' ;
	if ((scanf("%d %d %lf %lf %lf %lf %d %d %d", &Nx, &Ny,
		&Xstr, &Ystr, &Dx, &Dy, &Iund, &Igmin, &Igmax)) != 9) {
		fprintf(stderr, " Can't read JKM header.\n") ;
		exit(-1) ;
		}
	if ((Ig = (long *)calloc(Nx*Ny, sizeof(*Ig))) == NULL) {
		fprintf(stderr, " Can't alloc memory Ig.\n") ;
		exit(-1) ;
		}
	for (n = 0; n < Nx*Ny; n++) if (scanf("%ld", &Ig[n]) != 1) {
		fprintf(stderr, " Can't read Ig.\n") ;
		exit(-1) ;
		}
	while (getchar() != '\n') ;
	}
/* -------------------------------------------------------------------------- */
void	JKM_Int_Write()
{
	long	n ;

	Igmin =  99999999 ;
	Igmax = -99999999 ;
	for (n = 0; n < Nx*Ny; n++) {
		if (Ig[n] == Iund) continue ;
		if (Igmin > Ig[n]) Igmin = Ig[n] ;
		if (Igmax < Ig[n]) Igmax = Ig[n] ;
		}
	printf("%s\n", Cmnt) ;
	printf("%5d%5d%11.3lf%11.3lf%9.3lf%9.3lf%10ld%10ld%10ld\n",
		Nx, Ny, Xstr, Ystr, Dx, Dy, Iund, Igmin, Igmax) ;
	for (n = 0; n < Nx*Ny; n++) {
		if (Ig[n] == Iund) printf(" %7ld", Iund) ;
		else printf(" %7ld", Ig[n]) ;
		if (n%10 == 9) putchar('\n') ;
		}
	if (n%10 != 0) putchar('\n') ;
	}
