/*
*  Program name:             lattic_g  ver. 2.0
*
*  Program function:         ArcInfo lattice export file -> JKM data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Sep 12, 1991
*
*  Date of revision:         June 30, 1995 rewrite using C by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. zscale (default = 1)
*
*  Input:                    ArcInfo lattice export file ascii (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	ix, iy ;
	double	atof(), zscale=1.0 ;
	char	txt[100] ;
	void	JKM_Write(), Error() ;

	if (argc > 1) {
	  if (!strcmp(argv[1], "-h")) {
	    fprintf(stderr, " USAGE: lattic_g {zscale}\n") ;
	    return(0) ;
	    }
	  zscale = atof(argv[1]) ;
	  if (zscale == 0.0) {
	    fprintf(stderr, " Argument zscale error.\n") ;
	    return(-1) ;
	    }
	  }

	if (fgets(Cmnt, 100, stdin) == NULL)
	  Error(" Can't read lattice 1st comment.") ;
	if (fgets(txt, 100, stdin) == NULL || strncmp(txt, "LAT", 3))
	  Error(" Input isn't lattice export file.") ;
	if (fgets(txt, 100, stdin) == NULL || sscanf(txt, "%d %d", &Nx, &Ny) != 2)
	  Error(" Can't read lattice header Nx, Ny.") ;
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL)
	  Error(" Can't alloc memory G.") ;
	if (fgets(txt, 100, stdin) == NULL ||
	  sscanf(txt, "%lf%lf%lf%lf", &Xstr,&Ystr,&Dx,&Dy) != 4)
	  Error(" Can't read lattice header Xstr, Ystr, Dx, Dy.") ;
	if (fgets(txt, 100, stdin) == NULL ||
	  sscanf(txt, "%lf%lf", &Gmin, &Gmax) != 2)
	  Error(" Can't read lattice header Gmin, Gmax.") ;
	for (iy = Ny-1; iy > -1; iy--) {
	  for (ix = 0; ix < Nx; ix++) {
	    if (scanf("%f", &G[ix*Ny+iy]) != 1)
	      Error(" Can't read lattice data.") ;
	    }
	  }
	Und = Gmax + 999.9 ;
	JKM_Write() ;
	return(0) ;
	}
/* -------------------------------------------------------------------------- */
void	Error(cmnt)
char	*cmnt ;
{
	fprintf(stderr, "%s\n", cmnt) ;
	exit(-1) ;
	}
