/*
*  Program name:             pgm_g  ver. 2.1
*
*  Program function:         PGM to JKM
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jan 13, 1993
*
*  Date of revision:         June 28, 1995 Change output format by Y.Murata
*                            August 18, 1998 Add -f argument
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f: JKM file name      (JKM File)
*                            or
*                            xstr, ystr, dx, dy
*
*  Input:                    PGM file (program open)
*
*  Output:                   JKM file (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <malloc.h>
#include <stdlib.h>

#include "jkm_lib.c"

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	int	ix, iy, to_int=0 ;
	unsigned char	c ;
	unsigned int	ic ;
	double	atof() ;
	char	tx[100] ;
/* -------------------------------------------------------------------------- */
	if (strcmp(argv[1], "-f") == 0) {
		if (argc < 2 || (fp = fopen(argv[2], "r")) == NULL) {
			fprintf(stderr, "ppm_g: JKM file open error\n") ;
			return(-1) ;
			}
		fgets(tx, 100, fp) ;
		fscanf(fp, "%d %d %lf %lf %lf %lf",
			&ix, &iy, &Xstr, &Ystr, &Dx, &Dy) ;
		fclose(fp) ;
		}
	else {
		if (argc < 5) {
			fprintf(stderr, "ppm_g: argument error\n") ;
			exit(-1) ;
			}
		Xstr = atof(argv[1]) ;
		Ystr = atof(argv[2]) ;
		Dx   = atof(argv[3]) ;
		Dy   = atof(argv[4]) ;
		}

	if (argc > 5 && strcmp(argv[5], "int") == 0) to_int = 1 ;
	Iund  = 256256256 ;
	Und  = 256.0 ;

	fgets(tx, 100, stdin) ;
	fgets(tx, 100, stdin) ;
	sscanf(tx, "%d %d", &Nx, &Ny) ;

	if (to_int == 0) {
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		return(-1) ;
		}
	fgets(tx, 100, stdin) ;
	fprintf(stderr, "%d %d %lf %lf %lf %lf\n", Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
		c = getchar() ;
		G[ix*Ny+iy] = c ;
		}
	JKM_Write() ;
	}

	else {
	if ((Ig = (long *)calloc(Nx*Ny, sizeof(*Ig))) == NULL) {
		fprintf(stderr, " Can't alloc memory Ig.\n") ;
		return(-1) ;
		}
	fgets(tx, 100, stdin) ;
	fprintf(stderr, "%d %d %lf %lf %lf %lf\n", Nx, Ny, Xstr, Ystr, Dx, Dy) ;
	for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
		c = getchar() ;
		ic =c ;
		Ig[ix*Ny+iy] = ic * 1000000 + ic * 1000 + ic ;
		}
	JKM_Int_Write() ;
	}
	return(0) ;
	}
