/*
*  Program name:             pnm_g  ver. 2.1
*
*  Program function:         PNM to JKM
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Jan 13, 1993
*
*  Date of revision:         June 28, 1995 Change output format by Y.Murata
*                            August 18, 1998 Add -f argument
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                -f JKM file name
*                            -h xstr, ystr, dx, dy
*                            -s shift value
*
*  Input:                    PGM file (program open)
*
*  Output:                   JKM file (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#include "jkm_lib.c"

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	int	p, ix, iy, vmax, shift ;
	unsigned char	r, g, b ;
	double	atof() ;
	char	tx[100], head[10] ;
	void	JKM_Int_Write() ;
/* -------------------------------------------------------------------------- */
	Xstr = 0.0 ;
	Ystr = 0.0 ;
	Dx = 0.0 ;
	Dy = 0.0 ;
	shift = 0 ;
	for (p = 1; p < argc; p++) {
	  if (!strcmp(argv[p], "-f")) {
	    if ((fp = fopen(argv[p+1], "r")) == NULL) {
	      fprintf(stderr, "pnm_g: JKM file open error\n") ;
	      return(-1) ;
	      }
	    fgets(tx, 100, fp) ;
	    fscanf(fp, "%d %d %lf %lf %lf %lf", &ix,&iy,&Xstr,&Ystr,&Dx,&Dy) ;
	    fclose(fp) ;
	    }
	  else if (!strcmp(argv[p], "-h")) {
	    Xstr = atof(argv[p+1]) ;
	    Ystr = atof(argv[p+2]) ;
	    Dx   = atof(argv[p+3]) ;
	    Dy   = atof(argv[p+4]) ;
	    }
	  else if (!strcmp(argv[p], "-s")) {
	    shift = atoi(argv[p+1]) ;
	    }
	  }
	if (Dx == Dy) {
	  fprintf(stderr, "pnm_g: argument error\n") ;
	  return(-1) ;
	  }

	fgets(head, 10, stdin) ;
	p = 0 ;
	if (strncmp(head, "P3", 2) == 0) p = 3 ;
	else if (strncmp(head, "P5", 2) == 0) p = 5 ;
	fgets(tx, 100, stdin) ;
	sscanf(tx, "%d %d", &Nx, &Ny) ;
	if ((Ig = (long *)calloc(Nx*Ny, sizeof(*Ig))) == NULL) {
	  fprintf(stderr, " Can't alloc memory Ig.\n") ;
	  return(-1) ;
	  }
	fgets(tx, 100, stdin) ;
	sscanf(tx, "%d", &vmax) ;
	fprintf(stderr, "p, vmax = %d %d\n", p, vmax) ;

	fprintf(stderr, "%d %d %lf %lf %lf %lf\n", Nx, Ny, Xstr, Ystr, Dx, Dy) ;

	for (iy = Ny-1; iy > -1; iy--) for (ix = 0; ix < Nx; ix++) {
	  if (p == 3) {
	    r = getchar() ;
	    g = getchar() ;
	    b = getchar() ;
	    Ig[ix*Ny+iy] = ((long)r * 1000000 + (long)g * 1000 + (long)b) ;
	    }
	  else if (p == 5 && vmax == 255) {
	    r = getchar() ;
	    Ig[ix*Ny+iy] = (long)r ;
	    }
	  else if (p == 5 && vmax == 65535) {
	    r = getchar() ;
	    g = getchar() ;
	    Ig[ix*Ny+iy] = r * 256 + g + shift ;
/*	    printf("2 %d %d %d %d\n", r, g, r*256+g, Ig[ix*Ny+iy]) ; */
	    }
	  }

	strcpy(Cmnt, "Generated by pnm_g.\0") ;
	if (p == 3) {
	  Iund  = 256256256 ;
	  Igmin = 0 ;
	  Igmax = 255255255 ;
	  }
	else if (p == 5) {
	  Iund  = vmax + 1 ;
	  Igmin = 0 ;
	  Igmax = vmax ;
	  }
	JKM_Int_Write() ;
	return(0) ;
	}
