/*
*  Program name:             point_g  ver. 2.0
*
*  Program function:         Make JKM using polynomial function
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Nov 13, 1993
*
*  Date of revision:         June 30, 1995 rewrite using C by Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. number of polynomial functions (1-3)
*                            2. nx
*                            3. ny
*                            4. xstr
*                            5. ystr
*                            6. dx
*                            7. dy
*
*  Input:                    JKP data (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))
#define	MAXP	100000

#include "jkm_lib.c"

double	A[MAXP][11] ;
long	Nd[MAXP] ;
double	Xd[MAXP], Yd[MAXP], Zd[MAXP] ;

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	FILE	*fp ;
	long	npol, i, ix, iy, md, np, n ;
	double	p1, p2, p3, p4, p5, p6, p7, p8, p9, p10 ;
	double	x, y, z, xend, yend, atof(), House_Holder(), s ;
	char	txt[100] ;
	void	JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc < 8 || !strcmp(argv[1], "-h")) {
	  fprintf(stderr, " USAGE: point_g npoly ") ;
	  fprintf(stderr, "nx ny xstr ystr dx dy {err-file}\n") ;
	  return(0) ;
	  }
	npol = atoi(argv[1]) ;
	Nx   = atoi(argv[2]) ;
	Ny   = atoi(argv[3]) ;
	Xstr = atof(argv[4]) ;
	Ystr = atof(argv[5]) ;
	Dx   = atof(argv[6]) ;
	Dy   = atof(argv[7]) ;
	if (Nx == 0 || Ny == 0 || Dx == 0.0 || Dy == 0.0) {
	  fprintf(stderr, " Argument error.\n") ;
	  fprintf(stderr, " USAGE: point_g npoly ") ;
	  fprintf(stderr, "nx ny xstr ystr dx dy\n") ;
	  return(-1) ;
	  }
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		return(-1) ;
		}
	xend = Xstr + Dx * (double)(Nx-1) ;
	yend = Ystr + Dy * (double)(Ny-1) ;

	for (np = 0; np < MAXP; np++) {
	  if (fgets(txt, 100, stdin) == NULL) break ;
	  if (sscanf(txt, "%ld %lf %lf %lf",
	    &Nd[np], &Xd[np], &Yd[np], &Zd[np]) != 4) {
	    fprintf(stderr, " Point data error.\n") ;
	    continue ;
	    }
	  if (Xd[np]<Xstr||Xd[np]>xend||Yd[np]<Ystr||Yd[np]>yend) continue ;
	  x = Xd[np] - Xstr ;
	  y = Yd[np] - Ystr ;
	  A[np][0] = 1.0 ;
	  A[np][1] = x ;
	  A[np][2] = y ;
	  if (npol == 1) md = 3 ;
	  else if (npol == -1) {
	    A[np][3] = x * y ;
	    md = 4 ;
	    }
	  else if (npol == 2) {
	    A[np][3] = x*y ;
	    A[np][4] = x*x ;
	    A[np][5] = y*y ;
	    md = 6 ;
	    }
	  else if (npol == 3) {
	    A[np][3] = x*y ;
	    A[np][4] = x*x ;
	    A[np][5] = y*y ;
	    A[np][6] = x*x*x ;
	    A[np][7] = x*x*y ;
	    A[np][8] = x*y*y ;
	    A[np][9] = y*y*y ;
	    md = 10 ;
	    }
	  A[np][md] = Zd[np] ;
	  }

	fprintf(stderr, " np, md, npol = %ld %ld %ld", np, md, npol) ;
	s = House_Holder(np, md) ;
	fprintf(stderr, " s = %lf\n", s) ;

	p1  = A[ 0][md] ;
	p2  = A[ 1][md] ;
	p3  = A[ 2][md] ;
	p4  = A[ 3][md] ;
	p5  = A[ 4][md] ;
	p6  = A[ 5][md] ;
	p7  = A[ 6][md] ;
	p8  = A[ 7][md] ;
	p9  = A[ 8][md] ;
	p10 = A[ 9][md] ;
	Gmin =  99999999.9 ;
	Gmax = -99999999.9 ;
	for (ix = 0; ix < Nx; ix++) for (iy = 0; iy < Ny; iy++) {
	  x = Dx * (double)ix ;
          y = Dy * (double)iy ;
	  n = ix * Ny + iy ;
	  G[n] = p1 + p2*x + p3*y ;
	  if (npol == -1)
	    G[n] = G[n] + p4*x*y ;
	  if (npol > 1)
	    G[n] = G[n] + p4*x*y + p5*x*x + p6*y*y ;
	  if (npol > 2)
	    G[n] = G[n] + p7*x*x*x + p8*x*x*y + p9*x*y*y + p10*y*y*y ;
	  Gmin = min(Gmin, G[n]) ;
	  Gmax = max(Gmax, G[n]) ;
	  }
	Und = 99999999.9 ;
	strcpy(Cmnt, "Genarated by point_g") ;
	JKM_Write() ;

	if (argc < 9) exit(0) ;
	fp = fopen(argv[8], "w") ;
	for (i = 0; i < np; i++) {
	  x = Xd[i] - Xstr ;
	  y = Yd[i] - Ystr ;
	  z = p1 + p2*x + p3*y ;
	  if (npol == -1)
	    z = z + p4*x*y ;
	  if (npol > 1)
	    z = z + p4*x*y + p5*x*x + p6*y*y ;
	  if (npol > 2)
	    z = z + p7*x*x*x + p8*x*x*y + p9*x*y*y + p10*y*y*y ;
	  fprintf(fp, "%5d%12.3lf%12.3lf%9.3f%9.3f\n",
	    Nd[i], Xd[i], Yd[i], Zd[i], z) ;
	  }
	fclose(fp) ;
	return(0) ;
	}
/* ----------------------------------------------------------------------------
*     householder method
*               M. Saito(1983):Algorithms for Least Squares Method
*                     BUTSURI-TANKO, vol.36, no.2, pp.24-38.
----------------------------------------------------------------------------- */
double	House_Holder(n, m)
long	n, m ;
{
	long	i, j, k ;
	double	u, c1, d, x, v, s, sqrt() ;

	for (k = 0; k < m; k++) {
	  s = 0.0 ;
	  for (j = k; j < n; j++) {
	    s = s + A[j][k] * A[j][k] ;
	    }
	  if (A[k][k] > 0.0) s = -sqrt(s) ;
	  else s = sqrt(s) ;
	  u = s - A[k][k] ;
	  for (i = k+1; i < m+1; i++) {
	    c1 = 0.0 ;
	    for (j = k; j < n; j++) {
	      c1 = c1 + A[j][k] * A[j][i] ;
	      }
	    c1 = c1 / s ;
	    d = (c1 - A[k][i]) / u ;
	    A[k][i] = c1 ;
	    for (j = k+1; j < n; j++) {
	      A[j][i] = A[j][i] - d * A[j][k] ;
	      }
	    }
	  A[k][k] = s ;
	  }

/* backward substitution */
	A[m-1][m] = A[m-1][m] / A[m-1][m-1] ;
	for (j = m-2; j > -1; j--) {
	  x = A[j][m] ;
	  for (i = j+1; i < m; i++) {
	    x = x - A[j][i] * A[i][m] ;
	    }
	  A[j][m] = x / A[j][j] ;
	  }

/* variance */
	v = 0.0 ;
	for (j = m; j < n; j++) {
	  v = v + A[j][m] * A[j][m] ;
	  }
	s = v / (double)n ;
	return(s) ;
	}
