/*
*  Program name:             poly_g  ver. 1.0
*
*  Program function:         JKM making from JKG polygon data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            May 1, 1995
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. Nx
*                            2. Ny
*                            3. Xstr
*                            4. Ystr
*                            5. Dx
*                            6. Dy
*
*
*  Input:                    JKG data (from stdin)
*
*  Output:                   JKM data (to stdout)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#define	MAXD	100000

double	X[MAXD], Y[MAXD] ;
int	N, Col ;

#include "jkm_lib.c"

main(argc, argv)
int	argc ;
char	*argv[] ;
{
	int	n, atoi() ;
	char	line[100] ;
	double	atof() ;
	void	poly_grid_hatch() ;

	Nx   = atoi(argv[1]) ;
	Ny   = atoi(argv[2]) ;
	Xstr = atof(argv[3]) ;
	Ystr = atof(argv[4]) ;
	Dx   = atof(argv[5]) ;
	Dy   = atof(argv[6]) ;

	strcpy(Cmnt, "Grid data generated by poly_g") ;
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
		fprintf(stderr, " Can't alloc memory G.\n") ;
		return(-1) ;
		}

	Und = -99999.0 ;
	for (n = 0; n < Nx*Ny; n++) G[n] = Und ;

	N = -1 ;
	Col = 0 ;
	while (fgets(line, 100, stdin) != NULL) {
		if (line[0] == 'F' && line[1] == 'b') {
			sscanf(line, "%*s %d", &Col) ;
/*			fprintf(stderr, "Color = %d\n", Col) ;	*/
			}
		else if (line[0] == 'n') N = 0 ;
		else if (N == -1) continue ;
		else if (line[0] == 'm' || line[0] == 'l') {
			if (N < MAXD) sscanf(line, "%*c %lf %lf", &X[N],&Y[N]) ;
			N++ ;
			}
		else if (line[0] == 'f') {
			if (N >= MAXD) fprintf(stderr, " N (%d) >= MAXD.\n",N) ;
			else poly_grid_hatch() ;
			N = -1 ;
			}
		}
	JKM_Write() ;
	return(0) ;
	}
/* --------------------------------------------------- hatch area ----------- */
void	poly_grid_hatch()
{

	double	xc[1000], yc ;
	int	i, xs, xe, ic, iy ;
	int	poly_grid_cross() ;
	void	poly_grid_hatch_sub() ;

	for (i = 0 ; i < N ; i++) {
		X[i] = (X[i] - Xstr) / Dx ;
		Y[i] = (Y[i] - Ystr) / Dy ;
		if (X[i]-(int)X[i] == 0.0) X[i] = X[i] + 0.01 ;
		if (Y[i]-(int)Y[i] == 0.0) Y[i] = Y[i] + 0.01 ;
		}
	if (X[0] != X[N-1] || Y[0] != Y[N-1]) {
/*		fprintf(stderr, " line start != end\n") ;	*/
		X[N] = X[0] ;
		Y[N] = Y[0] ;
		N++ ;
		}
	for (iy = 0 ; iy <= Ny ; iy++) {
		yc = (double)(iy) ;
		ic = poly_grid_cross(yc, xc) ;
		for (i = 0 ; i < ic ; i = i + 2) {
			xs = (int)xc[i] + 1 ;
			xe = (int)xc[i+1]   ;
			if (xs <     0) xs =    0 ;
			if (xe >  Nx-1) xe = Nx-1 ;
			if (xs <=   xe) poly_grid_hatch_sub(iy, xs, xe) ;
			}
		}
	}
/* ------------------------------------------------- hatch pixel set -------- */
void	poly_grid_hatch_sub(iy, xs, xe)
int	iy, xs, xe ;
{
	int	ix, n ;

	if (iy < 0 || iy >= Ny) return ;
	for (ix = xs; ix <= xe; ix++) {
		if (ix >= 0 && ix < Nx) {
			n = ix * Ny + iy ;
			G[n] = (float)Col ;
			}
		}
	}
/* ------------------------------------------------- cross check ------------ */
int	poly_grid_cross(ys, xcrs)
double	ys, xcrs[] ;
{
	int	i, icrs, iu1, iu2 ;
	void	poly_grid_sort() ;

	icrs = 0 ;
	if (Y[0] > ys) iu1 = 1 ;
	else iu1 = -1 ;
	for (i = 1; i < N; i++) {
		if (Y[i] > ys) iu2 = 1 ;
		else iu2 = -1 ;
		if (iu1 * iu2 > 0) continue ;
		iu1 = iu2 ;
		xcrs[icrs] = X[i-1] + (X[i] - X[i-1]) *
				(ys - Y[i-1]) / (Y[i] - Y[i-1]) ;
		icrs = icrs + 1 ;
		}
	if(icrs % 2 != 0) {
		fprintf(stderr," #### cross error ys = %f\n", ys) ;
		for (i = 0; i < N; i++)
			fprintf(stderr,"%d %f %f\n",i,X[i],Y[i]) ;
		exit(-1) ;
		}
	poly_grid_sort(xcrs, icrs) ;
	return(icrs) ;
	}
/* ------------------------------------------------- data sort -------------- */
void	poly_grid_sort(xcrs, icrs)
int	icrs ;
double	xcrs[] ;
{
	int	i, j, k, l ;
	double	xcrs2 ;

	for (i = 1; i < icrs; i++) {
		for (j = 0; j <= i-1; j++) {
			l = 1 ;
			if(xcrs[i] < xcrs[j]) {
				xcrs2 = xcrs[i] ;
				for (k = i-1; k >= j; k--) {
					xcrs[k+1] = xcrs[k] ;
					}
				xcrs[j] = xcrs2 ;
				l = 0 ;
				break ;
				}
			if (l == 0) break ;
			}
		}
	}
