/*
*  Program name:             svf_g  ver. 1.1
*
*  Program function:         svf (single variable file) of ArcInfo to JKM
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            September 10, 1990
*
*  Date of revision:         June 26, 1995 Y.Murata
*                            May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. Xstr  (for JKM)
*                            2. Ystr  (for JKM)
*                            3. Dx    (for JKM)
*                            4. Dy    (for JKM)
*                            5. Cmnt  (for JKM)
*
*  Input:                    SVF file (from stdin)
*
*  Output:                   JKM file (to stdout)
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>

short	nx, ny, gmin, gmax, *g ;

main(argc, argv)
int	argc ;
char	*argv[] ;

{
	short	v ;
	int	ix, iy ;
	double	xstr, ystr, dx, dy, atof() ;
	int	w1, w2, n, nn, m, nm, l ;

	if (argc < 6 || !strcmp(argv[1], "-h")) {
		fprintf(stderr, " USAGE: svf_g xstr ystr dx dy cmnt\n") ;
		return(0) ;
		}

	w1 = getchar() ; w2 = getchar() ;
	ny = w1 * 256 + w2 ;
	w1 = getchar() ; w2 = getchar() ;
	nx = w1 * 256 + w2 ;
	fprintf(stderr, " nx = %d  ny = %d\n", nx, ny) ;
	xstr = atof(argv[1]) ;
	ystr = atof(argv[2]) ;
	dx   = atof(argv[3]) ;
	dy   = atof(argv[4]) ;
	if ((g = (short *)calloc(nx*ny, sizeof(*g))) == NULL) {
		fprintf(stderr, " Can't alloc memory g.\n") ;
		return(-1) ;
		}
	gmin =  9999 ;
	gmax = -9999 ;
	for (iy = ny-1; iy > -1; iy--) {
		ix = -1 ;
		w1 = getchar() ;
		w2 = getchar() ;
		if (w1 != 255 || w2 != 0) {
			fprintf(stderr, " File error %d %d\n", w1, w2) ;
			return(-1) ;
			}
		nn = getchar() ;
		for (n = 0; n < nn; n++) {
			nm = getchar() ;
			w1 = getchar() ; w2 = getchar() ;
			v = w1 * 256 + w2 ;
			if (gmin > v) gmin = v ;
			if (gmax < v) gmax = v ;
			for (m = 0; m < nm; m++) {
				ix++ ;
				l = ix * ny + iy ;
				if (l > nx*ny-1) {
					fprintf(stderr, " Count l error.\n") ;
					return(-1) ;
					}
				g[l] = v ;
				}
			}
		if (ix != nx-1) {
			fprintf(stderr, " Count ix error %d\n", ix) ;
			return(-1) ;
			}
		}
	fprintf(stderr, " min max = %d %d\n", gmin, gmax) ;

	printf("%s\n", argv[5]) ;
	printf("%5d%5d%12.2lf%12.2lf%11.3lf%11.3lf",
		nx, ny, xstr, ystr, dx, dy) ;
	printf("%6d.0%6d.0%6d.0\n", 999, gmin, gmax) ;
	for (n = 0; n < nx*ny; n++) {
		printf("%6d.0", g[n]) ;
		if (n%10 == 9) putchar('\n') ;
		}
	if (n%10 != 0) putchar('\n') ;
	return(0) ;
	}
