/*
*  Program name:             xyz_g  ver. 1.0
*
*  Program function:         Make JKM from xyz data
*
*  Programmed by:            Yasuaki MURATA,
*                            Geological Survey of Japan, Tsukuba
*                            Mar 15, 2002
*
*  Date of revision:         May 1, 2003 Distribution version by Y. Murata
*
*  Arguments:                1. nx
*                            2. ny
*                            3. xstr
*                            4. ystr
*                            5. dx
*                            6. dy
*
*  Input:                    xyz data (from stdin)
*
*  Output:                   JKM format (to stdout)
*/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#define	max(a, b)	(((a) > (b)) ? (a) : (b))
#define	min(a, b)	(((a) < (b)) ? (a) : (b))

#include "jkm_lib.c"

/* -------------------------------------------------------------------------- */
main(argc, argv)
int	argc ;
char	*argv[] ;
{
	double	x, y, z ;
	int	ix, iy, n ;
	void	JKM_Write() ;
/* -------------------------------------------------------------------------- */
	if (argc < 7 || !strcmp(argv[1], "-h")) {
	  fprintf(stderr, " USAGE: xyz_g ") ;
	  fprintf(stderr, "nx ny xstr ystr dx dy\n") ;
	  return(0) ;
	  }
	Nx   = atoi(argv[1]) ;
	Ny   = atoi(argv[2]) ;
	Xstr = atof(argv[3]) ;
	Ystr = atof(argv[4]) ;
	Dx   = atof(argv[5]) ;
	Dy   = atof(argv[6]) ;
	if (Nx == 0 || Ny == 0 || Dx == 0.0 || Dy == 0.0) {
	  fprintf(stderr, " Argument error.\n") ;
	  fprintf(stderr, " USAGE: xyz_g ") ;
	  fprintf(stderr, "nx ny xstr ystr dx dy\n") ;
	  return(-1) ;
	  }
	if ((G = (float *)calloc(Nx*Ny, sizeof(*G))) == NULL) {
	  fprintf(stderr, " Can't alloc memory G.\n") ;
	  return(-1) ;
	  }
	Gmin =  99999999.9 ;
	Gmax = -99999999.9 ;
	Und = 99999999.9 ;
	for (ix = 0 ; ix < Nx; ix++) {
	  for (iy = 0 ; iy < Ny; iy++) {
	    n = ix * Ny + iy ;
	    G[n] = Und ;
	    }
	  }
	while (1) {
	  if (scanf("%lf %lf %lf", &x, &y, &z) != 3) break ;
	  ix = (long)((x - Xstr) / Dx + 0.5) ;
	  iy = (long)((y - Ystr) / Dy + 0.5) ;
	  if (ix < 0 || ix >= Nx) continue ;
	  if (iy < 0 || iy >= Ny) continue ;
	  n = ix * Ny + iy ;
	  G[n] = z ;
	  Gmin = min(Gmin, G[n]) ;
	  Gmax = max(Gmax, G[n]) ;
	  }
	strcpy(Cmnt, "Generated by xyz_g") ;
	JKM_Write() ;
	return(0) ;
	}
